!
!     program Create_parameter_files_for_update_of_provisional_model_storages
!
!     Adapted from CPFFUOPMS, TEC II, Sep88, 26oct88 (for PC)
!     Modified 20oct92 by TSH & TECII for 4-byte dates and move to UNIX
!     Updated by TEC II, 21mar95 for change in EBSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and for change in use of forecast
!      start date instead of provisional data end date.
!     Modified 04nov97 by TSH & TECII to force execution of XEVAPOUT regardless
!      of whether or not there is new provisional data to be processed; must
!      generate the EVPPARAM.xxx file either way.
!     Modified 02aug99 by TSH to combine functionality for evaporation model
!      parameter files and LBRM parameter files into one source code file.  
!      This facilitates the port from command-line based to Windows 
!      GUI-based operation.  Also changed syntax from Fortran 77 to Fortran 90.
!     
!
!     This routine creates the overlake parameter file for use with the Lake
!     Evaporation model (and the Large Basin Runoff Model) to update the model 
!     heat storages (and moisture storages) to the end of the provisional data
!     period.
!
!     The file(s) created by this program (EVPPARAM.xxx or SUPARM??.INT) is/are
!     used by the appropriate model (XEVAPOUT or WATOUT).  File formats are
!     documented below.
!
!     THIS PROGRAM SHOULD BE CALLED AFTER 'DISAVMET' AND BEFORE 'XEVAPOUT' (1st
!     time)  ** OR **  AFTER 'DISAVMET' AND BEFORE 'WATOUT'.
!
!
!     ----- Files read/written when processing for evaporation model -----
!
!     The EBSNNM file is a formatted, direct access file with
!     a record length of 12 (11-byte records plus a line feed).
!     It has the following structure:
! 
!      1st record      LakeName              FORMAT(A11)
! 
!      2nd record      Program Designator    FORMAT(3X, I3, 5X)
!                      |                |
!                      |________________|___ This integer is used for
!                                            sequencing the programs in the
!                                            evaporation forecast package. In
!                                            this program the valid values are
!                                            -10, -9, 4, and 7.
! 
!      3rd record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|_yr, climatic data end date + 1 d
!                      | | |___|______mo, climatic data end date + 1 d
!                      |_|____________dy, climatic data end date + 1 d
!                                     This field is also called the BASE DATE.
! 
!      4th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of provisional data end
!                      | | |___|________month of provisional data end
!                      |_|______________day of provisional data end
! 
!      5th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of forecast end
!                      | | |___|________month of forecast end
!                      |_|______________day of forecast end
! 
!      6th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of start for model stats output
!                      | | |___|________month of start for model stats output
!                      |_|______________day of start for model stats output
! 
!      7th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of earliest changed data
!                      | | |___|________month of earliest changed data
!                      |_|______________day of earliest changed data
! 
!
!
!     The parameter files for this program, PARAMEVP.xxx and EVPPARAM.xxx 
!     (xxx = 'SUP', 'MIC', 'HUR', 'NHU', 'GEO', 'STC', 'ERI', 'ONT'), 
!     are formatted, sequential access files with the following structure:
! 
!     1st record:      Ext Comment           FORMAT(A3, A77)
!                      | | |     |
!                      | | |_____|___documentation comment
!                      |_|_________string denoting provisional ('PRV') or
!                                  climatic ('CLM') data file to use
! 
!     2nd through 14th records: evaporation parameters 1 - 13 respectively
!                               FORMAT(E10.4E1)
! 
!     15th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of simulation start date
!                      | | |___|______month of simulation start date
!                      |_|__________day of simulation start date
! 
!     16th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of simulation end date
!                      | | |___|______month of simulation end date
!                      |_|__________day of simulation end date
! 
!     17th record:     SurfaceTemperature    FORMAT(E13.7E2)
!                      |
!                      |___initial water surface temperature, degrees Celcius
! 
!     18th record:     IceAmount             FORMAT(E13.7E2)  NOT USED!!!
!                      |                                      NOT USED!!!
!                      |___initial ice cover, cubic meters    NOT USED!!!
! 
!     19th record:     IceArea               FORMAT(E13.7E2)
!                      |
!                      |___initial ice area, decimal fraction of lake area
! 
!     20th record:     IceDepth              FORMAT(E13.7E2)
!                      |
!                      |___initial ice depth, meters
! 
!     21st record:     IceTemp               FORMAT(E13.7E2)
!                      |
!                      |___initial ice pack temperature, deg.C
! 
!     22nd record:     NumberOfBlocks        FORMAT(E13.7E2)  NOT USED!!!
!                      |                                      NOT USED!!!
!                      |___initial number of ice blocks       NOT USED!!!
! 
!     23rd record:     AgeOfCurrentLayer     FORMAT(E13.7E2)
!                      |
!                      |___wind history of the current layer (on first day of
!                          simulation), meters/sec
! 
!     24th record:     NoOfLayers            FORMAT(I11)
!                      |
!                      |___number of layers in the heat superposition model on
!                          the first day of the simulation
! 
!     25th through 24+3*NoOfLayers records: heat storage parameters for all
!                                           layers in the heat superposition
!                                           model on the first day of the
!                                           simulation; each record has the
!                                           FORMAT(E13.7E2)
! 
!     NOTE THAT RECORDS 17 THROUGH 24 + 3 * NoOfLayers ARE FILLED IN BY A
!     PREVIOUS RUN OF THIS PROGRAM WITH AN END SIMULATION DATE ON THE DAY
!     BEFORE THE START SIMULATION DATE IN THIS PARAMETER FILE.  THESE REPRESENT
!     INITIAL CONDITIONS FOR THE PROGRAM SIMULATION AND ARE ARBITRARY FOR A
!     LONG SIMULATION WHERE THE INTEREST IS ONLY IN THE LATTER PART OF THE
!     SIMULATION.
! 
!           ----- Files read/written when processing for LBRM -----
!
!     The file: PARMTABL.xxx contains the model parameters and initial and
!     boundary conditions for each subbasin.  Currently, up to 25 sets of
!     boundary conditions are allowed for each subbasin parameter set.  This
!     file has the following format:
!
!     1st record: name (FORMAT: A10)
!                 |  |
!                 |__|_____basin/subbasin name, left adjusted, in upper case,
!                          the left-most three characters of which must equal
!                          the basin name (xxx).
!
!     2nd record through 11th record: model parameter (FORMAT: e13.6e2)
!
!              TBASE    base Temperature for heat available for evapotrans.
!              ALBEDS   degree-day factor for snowmelt computations
!              ALPPER   coefficient for percolation from upper soil zone
!              ALPUEV   coefficient for evapotrans. from upper soil zone
!              ALPINT   coefficient for interflow from lower soil zone
!              ALPDPR   coefficient for deep percolation from lower soil zone
!              ALPLEV   coefficient for evapotrans. from lower soil zone
!              ALPGW    coefficient for groundwater flow from groundwater zone
!              ALPSF    coefficient for basin outflow from surface storage zone
!              CONS     coefficient for heat available for evapotrans.
!
!    12th record: id im iy uszm lszm gzm ss snw (FORMAT: 2I2, I4, 5E13.6E2)
!                 || || || |  | |  | | | || | |
!                 || || || |  | |  | | | || |_|_snow pack storage (centimeters)
!                 || || || |  | |  | | | ||___surface storage (centimeters)
!                 || || || |  | |  | |_|____groundwater storage (centimeters)
!                 || || || |  | |__|______lower soil zone storage (centimeters)
!                 || || || |__|_________upper soil zone storage (centimeters)
!                 || || ||____________date's year
!                 || ||_____________date's month
!                 ||______________date's day
!
!     This record represents initial model storage conditions on the first day
!     of the calibration period.  These initial conditions are never used in
!     any of the forecast programs but are necessary for climatic data
!     preparation programs and are included for completeness.  The format of
!     record 12 is repeated between 0 and 24 more times for various boundary
!     conditions that the user might wish to include; these may represent field
!     measurements, for example.  Any values present in this manner will be
!     used by the forecast package to replace model generated storage values
!     on the dates specified.
!
!     The format of records 1 through 12-36 is repeated successively for each
!     subbasin, respectively.  This program will continue to read initial or
!     boundary conditions until a read error (reading the next basin/subbasin
!     name with an numeric format) or end-of-file is encountered.
!
!
!     The file: AREA.xxx (where xxx = 'SUP', 'MIC', 'HUR', 'NHU', 'GEO', 'STC',
!     'ERI', 'ONT', or 'CHA') contains the areas for each of the subbasins in
!     the Great Lake basin; units are square meters.  Record 1 contains
!     subbasin 1 area, record 2 contains subbasin 2 area, and so on up to the
!     number of subbasins present.  The last record, numbered one greater than
!     the number of subbasins, contains the area of the lake surface.
!
!=================================================================================
!

MODULE CPF_Subs

      USE FcstUtil

      CHARACTER (LEN=80) :: ThisLakeDir

CONTAINS

!-----------------------------------------------------------------------------
      SUBROUTINE EvapParm
      IMPLICIT   NONE

      INTEGER  :: Day, I, Id, J, IOS, U1, U2, U3
      INTEGER  :: Last, LpD, LpM, LpY, Month, Year
      INTEGER  :: NoOfLayers, SequenceNumber, Start, St_D, St_M, St_Y
      CHARACTER (LEN=1)  :: Lf
      CHARACTER (LEN=2)  :: CrLf
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=80) :: Line
      CHARACTER (LEN=99) :: File1, File2, File3

      Lf = CHAR(10)
      CrLf = CHAR(13)//CHAR(10)

!     CALL MyMsg_API(WinHandle, 'Building evap parameter file in CPFFUOPM')

!
!     Input basin name and simulation start date.
!
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,                &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      READ(U1, 1001, REC=1, ERR=812) BnS

!
!     Get Forecast Start Date, then convert it to a Provisional Data    
!     End Date.                                                         
!
      READ(U1, 1010, REC=5, ERR=812) lpD, lpM, lpY                              

      CALL DateSequence(lpD, lpM, lpY, Last); IF (ErrorLevel .NE. 0) GOTO 899
      Last = Last - 1                       ! end of required provisional model updates 
      CALL SequenceDate(lpD, lpM, lpY, Last); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Force run of XEVAPOUT program from the BASE DATE forward.  Read BASE
!     DATE and use as the earliest date of changed/added data by eDSAVGMET
!     since the last time that program: XEVAPOUT was run.  This is done since
!     initial conditions for XEVAPOUT are kept only for the BASE DATE + 1 and
!     the FORECAST DATE + 1 since they can be extensive in their use of
!     storage if kept every day.
!
      READ(U1, 1010, REC=3, ERR=812) St_D, St_M, St_Y
      CALL DateSequence(St_D, St_M, St_Y, Start); IF (ErrorLevel .NE. 0) GOTO 899

      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Write a new parameter file for use in continuing simulation.
!
      File2 = TRIM(ThisLakeDir) // 'PARAMEVP.'//BnS
      U2 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='OLD', ERR=821)
      CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899

      File3 = TRIM(ThisLakeDir) // 'EVPPARAM.'//BnS
      U3 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', ERR=31)
      CLOSE(U3, STATUS='DELETE')
   31 OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='NEW', ERR=831)
      CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899

      READ(U2, 1011, ERR=822) Line
      Ext = Line(1:3)
      IF (Ext .NE. 'PRV') THEN
        WRITE(ErrorMessage, 2011)
        CALL PassMsg
        WRITE(ErrorMessage, 2012)
        CALL PassMsg
        WRITE(ErrorMessage, 2013) TRIM(File2), Ext
        CALL PassMsg
        CLOSE (U2)
        CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
        CLOSE (U3, STATUS = 'DELETE')
        CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
        GOTO 898
      END IF
      WRITE(U3, 1011, ERR=833) TRIM(Line)
      DO Id = 1, 13
        READ(U2, 1011, ERR=822) Line
        WRITE(U3, 1011, ERR=833) TRIM(Line)
      END DO
      READ(U2, 1010, ERR=822) Day, Month, Year

!
!     Check initial conditions on starting date from existing parameter file.
!
      CALL DateSequence(Day, Month, Year, SequenceNumber)
      IF (ErrorLevel .NE. 0) GOTO 899
      IF (SequenceNumber .LT. Start) THEN
        WRITE(ErrorMessage, 2011)
        CALL PassMsg
        WRITE(ErrorMessage, 2012)
        CALL PassMsg
        WRITE(ErrorMessage, 2014) TRIM(File2)
        CALL PassMsg
        CLOSE (U2)
        CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
        CLOSE (U3, STATUS = 'DELETE')
        CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
        GOTO 898
      END IF
      WRITE(U3, 1010, ERR=833) St_D, St_M, St_Y
 
      READ(U2, 1010, ERR=822)
      WRITE(U3, 1010, ERR=833) lpD, lpM, lpY

      DO Id = 1, 7
        READ(U2, 1011, ERR=822) Line
        WRITE(U3, 1011, ERR=833) TRIM(Line)
      END DO

      READ(U2, 1004, ERR=822) NoOfLayers
      WRITE(U3, 1004, ERR=833) NoOfLayers
      DO Id = 1, NoOfLayers * 3
        READ(U2, 1011, ERR=822) Line
        WRITE(U3, 1011, ERR=833) TRIM(Line)
      END DO
      CLOSE (U2)
      CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (U3)
      CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899

!
!     4 = program designator
!
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,                &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1002, REC=2, ERR=813) 4, Lf
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CPFFUOPM: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CPFFUOPM: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'CPFFUOPM: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'CPFFUOPM: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CPFFUOPM: Error opening new file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File3)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : EvapParm...'
      CALL PassMsg
      RETURN

!
! FORMATs
!
 1001 FORMAT(A3, 8X, A1)
 1002 FORMAT(3X, I3, 5X, A1)
 1004 FORMAT(I11)
 1010 FORMAT(2I3, I5)
 1011 FORMAT(A)

 2011 FORMAT(' Wrong version of evaporation parameter file present')
 2012 FORMAT(' for creation of provisional update parameter file')
 2013 FORMAT(' File ', A, ' has designator ', A)
 2014 FORMAT(' File ', A, ' has wrong start date and initial conditions.')

      END SUBROUTINE EvapParm


!-----------------------------------------------------------------------------
      SUBROUTINE RunParm
      IMPLICIT   NONE

      INTEGER (KIND=INTEGER2), PARAMETER  :: iNSn = 35  ! iNSn = max. number of subbasins allowed
      INTEGER (KIND=INTEGER2), PARAMETER  :: NoBC = 25  ! NoBC = number of allowable boundary cond.
      INTEGER (KIND=INTEGER2), PARAMETER  :: Hdr  = 2   ! number of header records in *.SUM

      INTEGER (KIND=INTEGER2)  ::  iQst(5), ScaleNum
      INTEGER       ::  Kount(iNSn), Date(iNSn, 3, NoBC), NoSubbasins
      INTEGER       ::  I, Std, Stm, Sty, Start, Lpd, Lpm, Lpy, iD, iM, iY
      INTEGER       ::  Subbasin, IOS, Last, J, K, M, P, SumStart, ValidEnd
      INTEGER       ::  U1, U2, U3, U4, U5, U6, U7, U8, Seq
      REAL          ::  SubArea
      REAL          ::  qArea(iNSn), qParm(iNSn, 10), Qst(iNSn, 5, 0:NoBC)
      CHARACTER (LEN=1)  :: Lf
      CHARACTER (LEN=2)  :: CrLf, SB
      CHARACTER (LEN=3)  :: BnS
      CHARACTER (LEN=4)  :: OverTheLake
      CHARACTER (LEN=8)  :: iStart, iEnd, iDum
      CHARACTER (LEN=13) :: Name, Names
      CHARACTER (LEN=99) :: Heading(iNSn)
      CHARACTER (LEN=99) :: File1, File2, File3, File4
      CHARACTER (LEN=99) :: File5, File6, File7, File8

      Lf = CHAR(10)
      CrLf = CHAR(13)//CHAR(10)

!      ErrorMessage = 'Building runoff parameter files in CPFFUOPM'
!      CALL PassMsg

!
!     Input basin name and simulation start date.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,                 &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1207, REC=1, ERR=812) Name
      READ(U1, 1208, REC=2, ERR=812) NoSubbasins, I, OverTheLake

!
!     Get earliest date of changed/added data by DISAVMET since the last
!     time that program: LBRM was run (date of last valid             
!     provisional data + 1).                                            
!
      READ(U1, 1205, REC=8, ERR=812) Std, Stm, Sty                         
      CALL DateSequence(Std, Stm, Sty, Start)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get Forecast Start Date and convert it to a provisional data      
!     end date.                                                         
!
      READ(U1, 1205, REC=5, ERR=812) Lpd, Lpm, Lpy                         
      CALL DateSequence(Lpd, Lpm, Lpy, Last); IF (ErrorLevel .NE. 0) GOTO 899
      Last = Last - 1                         ! end of required provisional model updates 
      CALL SequenceDate(Lpd, Lpm, Lpy, Last); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Delete the file SKIP.INT if it exists. 
!
      CALL DeleteFile('SKIP.INT')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!       No new data requiring processing since the last time LBRM      
!       was run; skip further provisional data processing.               
!
      IF (Start .GT. Last) THEN
         File2 = TRIM(ThisLakeDir) // 'SKIP.INT'
         U2 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='NEW', RECL=12,             &
     &        ACCESS='DIRECT', FORM='FORMATTED', ERR=821)
         CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(U2, 1207, REC=1, ERR=823) Name, Lf
         CLOSE(U2)
         CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(U1, 1208, REC=2, ERR=823) NoSubbasins, 6, OverTheLake, Lf
         CLOSE (U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         ErrorMessage = 'No new data for LBRM to process...'
         CALL PassMsg
         ErrorLevel = 6
         GOTO 899
      END IF

      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
      BnS = Name(1:3)

!
!     Check initial conditions on starting date - 1 from summary files.
!     (All are assumed to start on same date.)
!
      File3 = TRIM(ThisLakeDir) // BnS//'01.SUM'
      U3 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U3, REC=1, ERR=832) ScaleNum, SumStart
      READ(U3, REC=2, ERR=832) ValidEnd
      CLOSE (U3)
      CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Start - 1 < SumStart ?   or   Start - 1 > ValidEnd ?
!
      IF (Start .LE. SumStart .OR. Start .GT. ValidEnd + 1) THEN
        File1 = TRIM(ThisLakeDir) // 'BSNNM'
        U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
        OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,                &
     &       ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
        CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
        WRITE(U1, 1208, REC=2, ERR=812) NoSubbasins, -1, OverTheLake, Lf
        CLOSE (U1)
        CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
        WRITE(ErrorMessage, 2501) TRIM(File1)
        CALL PassMsg
        WRITE(ErrorMessage, 2502)
        CALL PassMsg
        GOTO 898
      ENDIF

!
!     Read areas of subbasins.
!
      File4 = TRIM(ThisLakeDir) // 'AREA.' // BnS
      U4 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U4, FILE=TRIM(File4), STATUS='OLD', ERR=841)
      CALL FileWasOpened(U4); IF (ErrorLevel .NE. 0) GOTO 899
      DO Subbasin = 1, NoSubbasins
        READ(U4, 1206, ERR=842) qArea(Subbasin)
      END DO
      CLOSE (U4)
      CALL FileWasClosed(U4); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Input subbasin parameters.  Read from file PARAMnn.xxx
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File5 = TRIM(ThisLakeDir) // 'PARAM' // SB // '.' // BnS
         U5 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U5, FILE=TRIM(File5), STATUS='OLD',                      &
     &        ACCESS='SEQUENTIAL', FORM='FORMATTED', ERR=851)
         CALL FileWasOpened(U5); IF (ErrorLevel .NE. 0) GOTO 899
         J = 0
         READ(U5, 1501, ERR=852) Heading(Subbasin)
         READ(U5, 1502, ERR=852) SubArea
         IF (SubArea .NE. qArea(Subbasin)) THEN
            WRITE(ErrorMessage, 9005) TRIM(File5), BnS
            CALL PassMsg
            GOTO 898
         END IF
         READ(U5, *, ERR=852)   ! skip date line
         READ(U5, *, ERR=852)   ! skip date line
         DO I = 1, 10
            READ(U5, 1502, ERR=852) qParm(Subbasin, I)
         END DO
         CLOSE (U5)
         CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Input initial conditions.  These are read from the xxxnn.SUM file
!     for the day prior to the start of the run.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File5 = TRIM(ThisLakeDir) // BnS // SB // '.SUM'
         U5 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U5, FILE=TRIM(File5), STATUS='OLD', RECL=12,                  &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
         CALL FileWasOpened(U5); IF (ErrorLevel .NE. 0) GOTO 899
         J = Hdr + (Start - SumStart + 1) - 1
         READ(U5, REC=1, ERR=852) ScaleNum, I, ScaleNum       ! throw away first 6 bytes...
         READ(U5, REC=J, ERR=852) ScaleNum, (iQst(I), I=1,5)  ! throw away first 2 bytes...
         DO I = 1, 5
           Qst(Subbasin, I, 0) = iQst(I) / 100.
         ENDDO
         Qst(Subbasin, 3, 0) = (iQst(3) / 100.) * ScaleNum
         CLOSE (U5)
         CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Input boundary conditions.  Read from BNDCNDnn.xxx
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File6 = TRIM(ThisLakeDir) // 'BNDCND' // SB // '.' // BnS
         U6 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD', ERR=861)
         CALL FileWasOpened(U6); IF (ErrorLevel .NE. 0) GOTO 899
         READ(U6, *, ERR=862)
         READ(U6, *, ERR=862)
         J = 1
         IOS = 0
         DO WHILE ((J .LE. NoBC) .AND. (IOS .EQ. 0))
           READ(U6, 1601, IOSTAT=IOS) (Date(Subbasin, I, J), I=1,3),       &
     &                                (Qst(Subbasin, I, J),  I=1,5)
           IF (IOS .NE. 0) EXIT       ! drop out of loop - no error
           J = J + 1
         END DO
         CLOSE (U6)
         CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!      Throw away early boundary conditions.
!
      Seq = Start - 1                   ! initial value forces 1st WHILE loop
      DO WHILE (J .GE. 1 .AND. Seq .LT. Start)
         iD = Date(Subbasin, 1, 1)
         iM = Date(Subbasin, 2, 1)
         iY = Date(Subbasin, 3, 1)
         CALL DateSequence(iD, iM, iY, Seq); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Seq .LT. Start) THEN
            DO M = 1, J - 1                ! skipped if J = 1
              DO I = 1, 3
                Date(Subbasin, I, M) = Date(Subbasin, I, M + 1)
              END DO
              DO I = 1, 5
                Qst(Subbasin, I, M) = Qst(Subbasin, I, M + 1)
              END DO
            END DO
            J = J - 1
         END IF
      END DO
      Kount(Subbasin) = J

!
!     Write subbasin parameter files.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File7 = TRIM(ThisLakeDir) // 'PARAM' // SB // '.INT'
         U7 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U7, FILE = TRIM(File7), STATUS = 'OLD', ERR=70)
         CLOSE(U7, STATUS='DELETE')
  70     OPEN(UNIT=U7, FILE = TRIM(File7), STATUS = 'NEW', ERR=871)
         CALL FileWasOpened(U7); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(U7, 1501, ERR=873) Heading(Subbasin)
         WRITE(U7, 1502, ERR=873) qArea(Subbasin)
         CALL SequenceDate(Std, Stm, Sty, Start)
         CALL SequenceDate(Lpd, Lpm, Lpy, Last)
         WRITE(U7, 1503, ERR=873) Std, Stm, Sty, Std, Stm, Sty
         WRITE(U7, 1503, ERR=873) Lpd, Lpm, Lpy
         DO I = 1, 10
            WRITE(U7, 1502, ERR=873) qParm(Subbasin, I)
         END DO
         CLOSE(U7)
         CALL FileWasClosed(U7); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Write initial and boundary condition files.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File7 = TRIM(ThisLakeDir) // 'BNDCND' // SB // '.INT'
         U7 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U7, FILE=TRIM(File8), STATUS='OLD', ERR=75)
         CLOSE(U7, STATUS='DELETE')
  75     OPEN(UNIT=U7, FILE=TRIM(File8), STATUS='NEW', ERR=871)
         CALL FileWasOpened(U7); IF (ErrorLevel .NE. 0) GOTO 899
         CALL SequenceDate(Lpd, Lpm, Lpy, Start)
         WRITE(U7, 1602, ERR=873) Lpd, Lpm, Lpy, (Qst(Subbasin, I, 0), I=1,5)
         DO K = 1, Kount(Subbasin)
            Lpd = Date(Subbasin, 1, K)
            Lpm = Date(Subbasin, 2, K)
            Lpy = Date(Subbasin, 3, K)
            WRITE(U7, 1602, ERR=873) Lpd,Lpm,Lpy, (Qst(Subbasin, I, K), I=1,5)
         END DO
         CLOSE(U7)
         CALL FileWasClosed(U7); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Write LBRMnn.CFG files for each subbasin.  As the LBRM is run
!     for each subbasin all we will have to do is copy LBRMnn.CFG 
!     to LBRM.CFG
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File8 = TRIM(ThisLakeDir) // 'LBRM' // SB // '.CFG'
         U8 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U8, FILE=TRIM(File8), STATUS = 'OLD', ERR=80)
         CLOSE(U8, STATUS='DELETE')
  80     OPEN(UNIT=U8, FILE=TRIM(File8), STATUS = 'NEW', ERR=881)
         CALL FileWasOpened(U8); IF (ErrorLevel .NE. 0) GOTO 899
         File1 = TRIM(ThisLakeDir) // 'PARAM' // SB // '.INT'
         WRITE(U8, 1801, ERR=883) 'PARMFILE', TRIM(File1)
         File1 = TRIM(ThisLakeDir) // 'BNDCND' // SB // '.INT'
         WRITE(U8, 1801, ERR=883) 'BCFILE', TRIM(File1)
         File1 = TRIM(ThisLakeDir) // 'CLMASC' // SB // '.' // BnS
         WRITE(U8, 1801, ERR=883) 'METFILE', TRIM(File1)
         File1 = TRIM(ThisLakeDir) // 'LBRM' // SB // '.OUT'
         WRITE(U8, 1801, ERR=883) 'OUTFILE', TRIM(File1)
         WRITE(U8, 1801, ERR=883) 'WORKDIR', TRIM(ThisLakeDir)
         WRITE(U8, 1801, ERR=883) 'DATAFILE', ' '    ! flow data not needed
         CLOSE(U8)
         CALL FileWasClosed(U8); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     4 = program designator for successful completion; update BSNNM file.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,                 &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1208, REC=2, ERR=813) NoSubbasins, 4, OverTheLake, Lf
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CPFFUOPM: Error reading file '    // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File2)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'CPFFUOPM: Error reading file '    // TRIM(File3)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'CPFFUOPM: Error reading file '    // TRIM(File4)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File5)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'CPFFUOPM: Error reading file '    // TRIM(File5)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'CPFFUOPM: Error reading file '    // TRIM(File6)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File7)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  881 ErrorMessage = 'CPFFUOPM: Error opening file '    // TRIM(File8)
      CALL PassMsg; GOTO 898
  883 ErrorMessage = 'CPFFUOPM: Error writing to file ' // TRIM(File8)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : RunParm...'
      CALL PassMsg
      RETURN


!
! FORMATs
!
 1100 FORMAT(A8, 5E13.6E2)
 1205 FORMAT(2I3, I5)
 1206 FORMAT(E13.6E2)
 1207 FORMAT(A11, A1)
 1208 FORMAT(2I3, A4, 1X, A1)
 1300 FORMAT(2I2.2, I4.4)
 1305 FORMAT(I2.2)
 1501 FORMAT(A)
 1502 FORMAT(E13.6E2)
 1503 FORMAT(2I2, I4, 2I2, I4)
 1601 FORMAT(2I2, I4, 5E13.6E2)
 1602 FORMAT(2I2.2, I4, 5E13.6E2)
 1801 FORMAT(A, '=', A)
 2501 FORMAT(' Start date in ', A, ' is earlier or later')
 2502 FORMAT(' than model storages are recorded.')
 9005 FORMAT(' CPFFUOPM: subbasin area mismatch in ', A, ' and AREA.', A3)

      END SUBROUTINE RunParm

END MODULE CPF_Subs

!============================================================================

MODULE CPF_Main

CONTAINS

!-----------------------------------------------------------------------------
!-----------------------------------------------------------------------------
      SUBROUTINE DO_CPF(RunEvp, ErrStatus, HandleIn, Lake)
      USE CPF_Subs
      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: RunEvp, ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER  :: I
      CHARACTER (LEN=20) :: S

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!  Call routine to do the processing
!
      IF (RunEvp .EQ. 0) THEN
         CALL EvapParm()
      ELSE
         CALL RunParm()
      END IF
      IF (ErrorLevel .EQ. 6) GOTO 999     ! No new data to process
      IF (ErrorLevel .NE. 0) GOTO 899
      GOTO 999

!
!  Error Handling
! 
 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_CPF...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

      END SUBROUTINE DO_CPF

END MODULE CPF_Main

