MODULE FrcstSubs

      USE FcstUtil

CONTAINS

!----------------------------------------------------------------------------
      SUBROUTINE GetDateLimits (Lake, Start, Finish, FcStart, FcEnd)
!
!     Get and check lake basin name and read base dates & provisional end
!     dates in BSNNM & EBSNNM and in xxx00.CLM & xxxMD.CLM.
!
      IMPLICIT NONE

      INTEGER,           INTENT(OUT) :: Start, Finish, FcStart, FcEnd
      CHARACTER (LEN=3), INTENT(IN)  :: Lake

      INTEGER, PARAMETER  :: Zero = 0
      INTEGER, PARAMETER  :: zOne = 1

      INTEGER (KIND=INTEGER2)  :: I2
      INTEGER  :: I, J, L, IOS, Day, Month, Year, Date, bLink
      INTEGER  :: BaseDateR, BaseDateE, PrvsnlEndR, PrvsnlEndE
      INTEGER  :: ClmMdStart, Clm00Start, U1, U2, U3, U4, U5, U6
      CHARACTER (LEN=2) :: CrLf
      CHARACTER (LEN=3) :: BnS
      CHARACTER (LEN=99) :: File1, File2, File3, File4, File5, File6

      CrLf = CHAR(13)//CHAR(10)

!
!     Get and check lake basin name and read base dates in BSNNM.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1102, REC=1, ERR=812) BnS
      IF (BnS .NE. Lake) THEN
         WRITE(ErrorMessage, 9001) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      ENDIF
      READ(U1, 1104, REC=3, ERR=812) Day, Month, Year
      CALL DateSequence(Day, Month, Year, BaseDateR); IF (ErrorLevel .NE. 0) GOTO 899
      Start = BaseDateR
      READ(U1, 1104, REC = 4, ERR=812) Day, Month, Year
      CALL DateSequence(Day, Month, Year, PrvsnlEndR); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get date of last provisional data for runoff modeling.
!
      File2 = TRIM(ThisLakeDir) // 'PROVSNAL.'//BnS
      U2 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='OLD', RECL=26,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', IOSTAT=IOS)
      IF (IOS .EQ. 0) THEN
         CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899
         bLink = zOne
         DO WHILE (bLink .NE. Zero)
            READ(U2, REC=bLink, ERR=822) Date, bLink     ! ignore rest of each record
         END DO
         CLOSE (U2)
         CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
      ELSE
         Date = Start - 1
      END IF
      IF (Date .NE. PrvsnlEndR) THEN
         WRITE(ErrorMessage, 9021)
         CALL PassMsg
         WRITE(ErrorMessage, 9091)
         CALL PassMsg
         GOTO 898
      END IF

      File3 = TRIM(ThisLakeDir) // BnS // '00.CLM'
      U3 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', RECL=6,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U3, REC=1, ERR=832) I2, Clm00Start
      READ(U3, REC=2, ERR=832) Finish
      CLOSE (U3)
      CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
      IF (Finish .NE. BaseDateR - 1) THEN
         WRITE(ErrorMessage, 9011) 
         CALL PassMsg
         WRITE(ErrorMessage, 9012) BnS
         CALL PassMsg
         WRITE(ErrorMessage, 9091)
         CALL PassMsg
         GOTO 898
      END IF

!
!     Get and check lake basin name and read base dates in EBSNNM.
!
      File4 = TRIM(ThisLakeDir) // 'EBSNNM'
      U4 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U4, FILE=TRIM(File4), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=841)
      CALL FileWasOpened(U4); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U4, 1102, REC=1, ERR=842) BnS
      IF (BnS .NE. Lake) THEN
         WRITE(ErrorMessage, 9006) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF
      READ(U4, 1104, REC=3, ERR=842) Day, Month, Year
      CALL DateSequence(Day, Month, Year, BaseDateE); IF (ErrorLevel .NE. 0) GOTO 899
      Finish = BaseDateE
      READ(U4, 1104, REC=4, ERR=842) Day, Month, Year
      CALL DateSequence(Day, Month, Year, PrvsnlEndE); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (U4)
      CALL FileWasClosed(U4); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get date of last provisional data for evaporation modeling.
!
      File5 = TRIM(ThisLakeDir) // 'EPRVSNAL.'//BnS
      U5 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U5, FILE=TRIM(File5), STATUS='OLD', RECL=26,            &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', IOSTAT=I)
      IF (I .EQ. 0) THEN
         CALL FileWasOpened(U5); IF (ErrorLevel .NE. 0) GOTO 899
         bLink = zOne
         DO WHILE (bLink .NE. Zero)
           READ(U5, REC=bLink, ERR=852) Date, bLink
         END DO
         CLOSE (U5)
         CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
      ELSE
         Date = Finish - 1
      END IF
      IF (Date .NE. PrvsnlEndE) THEN
         WRITE(ErrorMessage, 9026)
         CALL PassMsg
         WRITE(ErrorMessage, 9091)
         CALL PassMsg
         GOTO 898
      END IF

      File6 = TRIM(ThisLakeDir) // BnS//'MD.CLM'
      U6 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD', RECL=8,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
      READ(U6, REC=1, ERR=862) I2, ClmMdStart
      READ(U6, REC=2, ERR=862) L
      CLOSE (U6)
      CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899
      IF (L .NE. BaseDateE - 1) THEN
         WRITE(ErrorMessage, 9016)
         CALL PassMsg
         WRITE(ErrorMessage, 9017) BnS
         CALL PassMsg
         WRITE(ErrorMessage, 9091)
         CALL PassMsg
         GOTO 898
      END IF

!
!     Forecast Start Date must be after the end of the historical data files.
!
      FcStart = MAX(BaseDateR, BaseDateE)

!
!     Forecast Start Date must be on or before the end of the provisional data
!     plus one day (forecast can begin on day after provisional data end).
!
      FcEnd = MIN(PrvsnlEndR, PrvsnlEndE) + 1

!
!     Climatic data files end on the day before the base date.
!     Will not allow any forecast requests that require historical data beyond
!     the end of the climatic data.
!
      Finish = MIN(BaseDateR, BaseDateE) - 1

!
!     Will not allow any forecast requests that require historical data before
!     the beginning of the climatic data.
!
      Start = MAX(Clm00Start, ClmMdStart)

      RETURN  

!
!
 811  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File1)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File1)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File2)
      CALL PassMsg; GOTO 898
 822  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File2)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File3)
      CALL PassMsg; GOTO 898
 832  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File3)
      CALL PassMsg; GOTO 898
 841  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File4)
      CALL PassMsg; GOTO 898
 842  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File4)
      CALL PassMsg; GOTO 898
 851  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File5)
      CALL PassMsg; GOTO 898
 852  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File5)
      CALL PassMsg; GOTO 898
 861  ErrorMessage = 'FRCST: Error opening file '   // TRIM(File6)
      CALL PassMsg; GOTO 898
 862  ErrorMessage = 'FRCST: Error reading file '   // TRIM(File6)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : GetDateLimits...'
      CALL PassMsg
      RETURN

!
!
 1102 FORMAT(A3)
 1104 FORMAT(2I3, I5)

 9001 FORMAT(' Mismatch between lake name in BSNNM & SIMULATE files in ', A)
 9006 FORMAT(' Mismatch between lake name in EBSNNM & SIMULATE files in ', A)
 9011 FORMAT(' Mismatch between base date in BSNNM file and end date in ')
 9012 FORMAT(A3, '00.CLM file!  It must end the day before the base date.')
 9016 FORMAT(' Mismatch between base date in EBSNNM file and end date in ')
 9017 FORMAT(A3, 'MD.CLM file!  It must end the day before the base date.')
 9021 FORMAT(' Provisional data end date in BSNNM is incorrect!')
 9026 FORMAT(' Provisional data end date in EBSNNM is incorrect!')
 9091 FORMAT(' Problem is located in directory: ', A)

      END SUBROUTINE GetDateLimits

!----------------------------------------------------------------------------
      LOGICAL FUNCTION CanGetFileForecastLength (Length, BnS)
!
!     Returns the forecast "Length" (number of months of outlook) read from
!     the "LENGTH" file in the application directory.  Returns "TRUE" if
!     successful.
!
      IMPLICIT NONE
      INTEGER, INTENT(OUT) :: Length
      CHARACTER (LEN=*), INTENT(IN) :: BnS

      INTEGER  :: U1
      CHARACTER (LEN=99) :: File1

      CanGetFileForecastLength = .FALSE.
      File1 = TRIM(ThisLakeDir) // 'LENGTH.'//BnS
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=52,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 2101, REC=1, ERR=812) Length
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
      CanGetFileForecastLength = .TRUE.
      RETURN
!
!
 811  CanGetFileForecastLength = .FALSE.
      RETURN
 812  ErrorMessage = 'FRCST: Error reading file '//TRIM(File1)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : CanGetFileForecastLength...'
      CALL PassMsg
      RETURN

 2101 FORMAT(45X, I5)

      END FUNCTION CanGetFileForecastLength

!----------------------------------------------------------------------------
      INTEGER FUNCTION AllowableForecastLength(HistDay, HistMonth, BeginDate, EndngDate, FrcstStrt)
!
!     Calculates the maximum possible forecast length (in months); must enter
!     with "HistDay", "HistMonth", "BeginDate", "EndngDate", & "FrcstStrt".
!
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: HistDay, HistMonth, FrcstStrt
      INTEGER, INTENT(IN) :: BeginDate, EndngDate

      INTEGER :: D, M, Y, Length
      INTEGER :: EarliestHSD, HistEndDate, FrcstEnd

!
!     First, calculate earliest possible historical start date.
!
      Y = HistStartYear(BeginDate, HistDay, HistMonth)
      IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(HistDay, HistMonth, Y, EarliestHSD)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Second, calculate maximum allowable length; start w/ trial length.
!
      HistEndDate = EarliestHSD
      Length = (EndngDate - HistEndDate) / 31 - 1
      IF (Length .LT. 0) Length = 0
      DO WHILE (HistEndDate .LE. EndngDate)
         Length = Length + 1
         CALL SequenceDate(D, M, Y, FrcstStrt)  ! start w/ forecast start date
         IF (ErrorLevel .NE. 0) GOTO 899
         M = M + Length                         ! add forecast length (months)
         IF (D .GT. 1) M = M + 1                ! include partial 1st month as extra
         DO WHILE (M .GT. 12)                   ! determine forecast end date
           M = M - 12
           Y = Y + 1
         END DO
         CALL DateSequence(1, M, Y, FrcstEnd)
         IF (ErrorLevel .NE. 0) GOTO 899
         FrcstEnd = FrcstEnd - 1
         HistEndDate = EarliestHSD + FrcstEnd - FrcstStrt
      END DO
      IF (Length .GT. 0) Length = Length - 1

!
!     Third, if the length is 0 then there is not enough data available for
!     any forecasts.
!
      AllowableForecastLength = Length
      RETURN

!
 899  ErrorMessage = '[traceback] : AlllowableForecastLength...'
      CALL PassMsg
      RETURN

      END FUNCTION AllowableForecastLength

!----------------------------------------------------------------------------
      INTEGER FUNCTION NumberOfForecastsPossible                          &
     &   (HistDay, HistMonth, Length, BeginDate, EndngDate, FrcstStrt)
!
!     Calculates the number of forecasts possible; must enter with these
!     variables defined: 
!        HistDay         = historical start day
!        HistMonth       = historical start month
!        ForecastLength  = length, in months
!        BeginDate       = beginning date of CLM files
!        EndngDate       = end date of CLM files
!        FrcstStrt       = forecast start date
!
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: HistDay, HistMonth, FrcstStrt
      INTEGER, INTENT(IN) :: BeginDate, EndngDate

      INTEGER :: D, M, Y, Length, DaysInFrcst
      INTEGER :: EarliestHSD, HistEndDate, FrcstEnd
      INTEGER :: HistStrtDate, Number

!
!     First, calculate forecast end date.
!
      CALL SequenceDate(D, M, Y, FrcstStrt)  ! start w/ forecast start date
      IF (ErrorLevel .NE. 0) GOTO 899
      M = M + Length - 1                     ! add forecast length (months)
      IF (D .GT. 1) M = M + 1                ! include partial 1st month as extra
      DO WHILE (M .GT. 12)                   ! determine forecast end date
        M = M - 12
        Y = Y + 1
      END DO
      D = DaysInMonth(M, Y);  IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(D, M, Y, FrcstEnd);  IF (ErrorLevel .NE. 0) GOTO 899
      DaysInFrcst = FrcstEnd - FrcstStrt + 1

!
!     Second, calculate earliest possible historical start date.
!
      Y = HistStartYear(BeginDate, HistDay, HistMonth)
      IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(HistDay, HistMonth, Y, HistStrtDate)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Third, find number of outlooks possible in historical data set.
!
      HistEndDate = HistStrtDate + DaysInFrcst - 1
      Number = 0
      DO WHILE (HistEndDate .LE. EndngDate)
        Number = Number + 1
        CALL SequenceDate(D, M, Y, HistStrtDate); IF (ErrorLevel .NE. 0) GOTO 899
        Y = Y + 1
        CALL DateSequence(HistDay, HistMonth, Y, HistStrtDate); IF (ErrorLevel .NE. 0) GOTO 899
        HistEndDate = HistStrtDate + DaysInFrcst - 1
      END DO
      NumberOfForecastsPossible = Number
      RETURN

  899 ErrorMessage = '[traceback] : NumberOfForecastsPossible...'
      CALL PassMsg
      RETURN

      END FUNCTION NumberOfForecastsPossible 

!----------------------------------------------------------------------------
      INTEGER FUNCTION HistStartYear (BeginDate, HDay, HMonth)
!
!     Calculate earliest possible historical start date.
!
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: BeginDate, HDay, HMonth
      INTEGER :: D, M, Y, HistStrtDate

      CALL SequenceDate(D, M, Y, BeginDate);  IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(HDay, HMonth, Y, HistStrtDate); IF (ErrorLevel .NE. 0) GOTO 899
      IF (HistStrtDate .LT. BeginDate) Y = Y + 1
      HistStartYear = Y
      RETURN

  899 ErrorMessage = '[traceback] : HistStartYear...'
      CALL PassMsg
      RETURN

      END FUNCTION HistStartYear

!----------------------------------------------------------------------------
      SUBROUTINE SaveOutlookYears(Years, NmbrYears, CrrntDate, HistDay, HistMonth)
!
!     Build data files as required for forecast package: YEARFILE.INT,
!     ONLYONE.INT, & DAT1xxxx.INT, for xxxx = all valid years.
!
      IMPLICIT NONE

      INTEGER, INTENT(IN) :: Years(1:MaxNoYears), NmbrYears
      INTEGER, INTENT(IN) :: CrrntDate, HistDay, HistMonth

      INTEGER  :: I, Day, Month, Year, U1, U2, U3
      CHARACTER (LEN=4)  :: YearStr
      CHARACTER (LEN=99) :: File1, File2, File3

      File1 = TRIM(ThisLakeDir) // 'YEARFILE.INT'
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN (UNIT=U1, FILE=TRIM(File1), STATUS='OLD', ERR=11)
      CLOSE (U1, STATUS='DELETE')
  11  OPEN (UNIT=U1, FILE=TRIM(File1), STATUS='NEW', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      CALL SequenceDate(Day, Month, Year, CrrntDate); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, NmbrYears
        WRITE(U1, 1001, ERR=813) Years(I)
        WRITE(YearStr, 1001) Years(I)
        File2 = TRIM(ThisLakeDir) // 'DAT1' // TRIM(YearStr) // '.INT'
        U2 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
        OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='OLD', ERR=21)
        CLOSE(U2, STATUS='DELETE')
  21    OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='NEW', ERR=821)
        CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899
        WRITE(U2, 1001, ERR=823) HistDay
        WRITE(U2, 1001, ERR=823) HistMonth
        WRITE(U2, 1001, ERR=823) Years(I)
        WRITE(U2, 1001, ERR=823) Day
        WRITE(U2, 1001, ERR=823) Month
        WRITE(U2, 1001, ERR=823) Year
        WRITE(U2, 1001, ERR=823) 0
        CLOSE(U2)
        CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
      END DO
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      File3 = TRIM(ThisLakeDir) // 'ONLYONE.INT'
      U3 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', ERR=31)
      CLOSE (U3, STATUS='DELETE')
  31  IF (NmbrYears .EQ. 1) THEN
        OPEN(UNIT=U3, FILE='ONLYONE.INT', STATUS='NEW', ERR=831)
        CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
        WRITE(U3, 1001, ERR=833) 1
        CLOSE (U3)
        CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
      END IF
      RETURN
!
!
 811  ErrorMessage = 'FRCST: Error opening '      // TRIM(File1)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'FRCST: Error reading from ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 813  ErrorMessage = 'FRCST: Error writing to '   // TRIM(File1)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'FRCST: Error opening '      // TRIM(File2)
      CALL PassMsg; GOTO 898
 822  ErrorMessage = 'FRCST: Error reading from ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 823  ErrorMessage = 'FRCST: Error writing to '   // TRIM(File2)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'FRCST: Error opening '      // TRIM(File3)
      CALL PassMsg; GOTO 898
 832  ErrorMessage = 'FRCST: Error reading from ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 833  ErrorMessage = 'FRCST: Error writing to '   // TRIM(File3)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : SaveOutlookYears...'
      CALL PassMsg
      RETURN

 1001 FORMAT(I4)

      END SUBROUTINE SaveOutlookYears

!----------------------------------------------------------------------------
      SUBROUTINE PutForecastLength (Length, BnS)
!
!     Puts forecast "Length" (number of months of outlook) into the "LENGTH"
!     file in the application directory (if it exists).
!
      IMPLICIT NONE
      INTEGER,           INTENT(IN)  :: Length
      CHARACTER (LEN=3), INTENT(IN)  :: BnS

      INTEGER :: U1
      CHARACTER (LEN=99) :: File1

!
!     Put number of months of outlook.
!
      File1 = TRIM(ThisLakeDir) // 'LENGTH.' // BnS
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=52,           &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) RETURN
      WRITE(U1, 1001, REC=1, ERR=813) Length
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) RETURN
      RETURN

!
 811  ErrorMessage = 'FRCST: Error opening '    // TRIM(File1)
      CALL PassMsg;  GOTO 898
 813  ErrorMessage = 'FRCST: Error writing to ' // TRIM(File1)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : PutForecastLength...'
      CALL PassMsg
      RETURN

 1001 FORMAT('Do not change this value manually;  length is', I5)

      END SUBROUTINE PutForecastLength

!----------------------------------------------------------------------------
      SUBROUTINE UpdateBsnnmFiles (NameFile, FrcstStrt, FrcstEnd, Length)
!
!     Writes forecast start date into record 5, forecast end date into record
!     6, and "plot" start date into record 7 (18 months and odd days, if
!     necessary, before the start of the forecast).
!
      IMPLICIT NONE

      INTEGER,           INTENT(IN) :: FrcstStrt, FrcstEnd, Length
      CHARACTER (LEN=*), INTENT(IN) :: NameFile

      INTEGER           :: Day, Month, Year, U1
      INTEGER           :: IOS
      CHARACTER (LEN=1), PARAMETER :: LF = CHAR(10)

      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(NameFile), STATUS='OLD', RECL=12,           &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      CALL SequenceDate(Day, Month, Year, FrcstEnd); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1001, REC=6, ERR=813) Day, Month, Year, LF

      CALL SequenceDate(Day, Month, Year, FrcstStrt); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1001, REC=5, ERR=813) Day, Month, Year, LF

      Month = Month - 18         ! Calculate the "plot" start date.
      DO WHILE (Month .LT. 1)
        Month = Month + 12
        Year = Year - 1
      END DO
      WRITE(U1, 1001, REC=7, ERR=813) 1, Month, Year, LF

      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN
!
!
 811  ErrorMessage = 'FRCST: Error opening '    // TRIM(NameFile)
      GOTO 898
 813  ErrorMessage = 'FRCST: Error writing to ' // TRIM(NameFile)
      GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : UpdateBsnnmFiles...'
      CALL PassMsg
      RETURN

!
 1001 FORMAT(2I3, I5, A1)

      END SUBROUTINE UpdateBsnnmFiles

END MODULE FrcstSubs

!==============================================================================

MODULE FrcstMain

CONTAINS

      SUBROUTINE DO_FRCST (ErrStatus, HandleIn, Lake)

      USE FrcstSubs

!
!     Prepare a forecast of lake evaporation, basin runoff, and water levels
!     with the RUNOFF FORECAST PACKAGE. - 21mar95, TECII
!
!     The earlier version of this program was broken into two pieces: a
!     WINDOWS program, OUTLOOK, and this DOS program, FRCST.  The WINDOWS
!     program prepares, from user input, the SIMULATE file that is now
!     simply read by this program, checked, and used to prepare files for the
!     start of the forecast procedure.  The SIMULATE file may also be prepared
!     with a simple text editor (hence the necessity of validity checking
!     here).
!
      IMPLICIT NONE

      INTEGER, INTENT(INOUT)  ::  ErrStatus
      INTEGER, INTENT(IN)     ::  HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER :: I, J, K, L, AFL, NOFP, Dy, Mn, Yr
      INTEGER :: FrcstStrt, HistDay, HistMonth, CrrntDate
      INTEGER :: Length, NmbrYears, Years(1:MaxNoYears)
      INTEGER :: HsBegDate, HsEndDate, FcBegDate, FcEndDate
      INTEGER :: HistStrtDate, FrcstEnd, HistEndDate
      LOGICAL :: Good, Flag
      CHARACTER (LEN=3)  :: BnS
      CHARACTER (LEN=10) :: S
      CHARACTER (LEN=99) :: FName

      WinHandle = HandleIn
      ErrorLevel = 0
      ErrStatus = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
      CALL ReadSIMULATEfile(BnS, FrcstStrt, HistDay, HistMonth,                &
     &                      CrrntDate, Length, NmbrYears, Years)
      IF (ErrorLevel .NE. 0) GOTO 899

      CALL GetDateLimits(BnS, HsBegDate, HsEndDate, FcBegDate, FcEndDate)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  If there are no time-series to use then we stop.
!
      IF (NmbrYears .LE. 0) THEN
         ErrorMessage = ' Number of time series (years) from historical record'
         CALL PassMsg
         ErrorMessage = ' must be greater than 0.   Forecast aborted!'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  If the forecast length is less than 1 then we stop.
!
      IF (Length .LE. 0) THEN
         ErrorMessage = ' Forecast length must be greater than 0.  Forecast aborted!'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  The forecast start date must be within the provisional data period (including the
!  day after the provisional data end date.)  
!
      IF (FrcstStrt .GT. FcEndDate .OR. FrcstStrt .LT. FcBegDate) THEN
         ErrorMessage = ' Forecast start date not within allowable provisional data period.'
         CALL PassMsg
         ErrorMessage = ' Forecast aborted!'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  Can we read the forecast length file?
!
      Flag = CanGetFileForecastLength(L, BnS)
      IF (ErrorLevel .NE. 0) GOTO 899
      IF (.NOT. Flag) THEN
         ErrorMessage = ' Error in opening LENGTH.'//BnS//' file.'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  What is the allowable forecast length?  If it ends up being <= 0 we stop.
!
      AFL = AllowableForecastLength(HistDay, HistMonth, HsBegDate, HsEndDate, FrcstStrt)
      IF (ErrorLevel .NE. 0) GOTO 899
      IF (AFL .LE. 0) THEN
         ErrorMessage = ' Insufficient historical data available for any forecasts.'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  If the specified forecast length is too large we stop.
!
      IF (Length .GT. AFL) THEN
         ErrorMessage = ' Forecast length is too large.'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  If too many time-series are listed in the SIMULATE file we stop
!
      NOFP = NumberOfForecastsPossible(HistDay, HistMonth, Length, HsBegDate, HsEndDate, FrcstStrt)
      IF (ErrorLevel .NE. 0) GOTO 899
      IF (NmbrYears .GT. NOFP) THEN
         ErrorMessage = ' Too many time series (years) requested from historical record.'
         CALL PassMsg
         WRITE(ErrorMessage, 9020) NmbrYears, NOFP
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  Verify that ALL of the time-series specified are good (valid) ones.
!
      Good = .TRUE.
      CALL SequenceDate(Dy, Mn, Yr, FrcstStrt); IF (ErrorLevel .NE. 0) GOTO 899
      Mn = Mn + Length
      IF (Dy .GT. 1) Mn = Mn + 1
      DO WHILE (Mn .GT. 12)
         Mn = Mn - 12
         Yr = Yr + 1
      END DO
      CALL DateSequence(1, Mn, Yr, FrcstEnd); IF (ErrorLevel .NE. 0) GOTO 899
      FrcstEnd = FrcstEnd - 1       ! forecast end date
      DO I = 1, NmbrYears
         CALL DateSequence(HistDay, HistMonth, Years(I), HistStrtDate)
         IF (ErrorLevel .NE. 0) GOTO 899
         HistEndDate = HistStrtDate + FrcstEnd - FrcstStrt
         Good = (HistEndDate .LE. HsEndDate) .AND. Good
      END DO

!
!  If ANY of the specified time-series won't work we will stop.
!
      IF (.NOT. Good) THEN
         ErrorMessage = ' Time series (years) requested from historical record are not available.'
         CALL PassMsg
         WRITE(ErrorMessage, 9091) TRIM(ThisLakeDir)
         CALL PassMsg
         GOTO 898
      END IF

!
!  Write a new value into LENGTH.xxx if needed
!
      IF (Length .NE. L) CALL PutForecastLength(Length, BnS)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Write the time-series years into 
!
      CALL SaveOutlookYears(Years, NmbrYears, CrrntDate, HistDay, HistMonth)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Update the BSNNM and EBSNNM files
!
      FName = TRIM(ThisLakeDir) // 'BSNNM'
      CALL UpdateBsnnmFiles(FName, FrcstStrt, FrcstEnd, Length)
      IF (ErrorLevel .NE. 0) GOTO 899

      FName = TRIM(ThisLakeDir) // 'EBSNNM'
      CALL UpdateBsnnmFiles(FName, FrcstStrt, FrcstEnd, Length)
      IF (ErrorLevel .NE. 0) GOTO 899

      ErrStatus = 0
      GOTO 999

!
!  Error Handling
! 
 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_FRCST...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
!  FORMATs
!
 9020 FORMAT(' SIMULATE file contains ', I0, ' years, but max is ', I0)
 9091 FORMAT(' Problem is located in directory: ', A)


      END SUBROUTINE DO_FRCST

END MODULE FrcstMain

