!
!     Updated by TEC II 28sep88, 26oct88 (for PC)
!     Updated by TSH and TEC II 7oct92 for 4-byte dates and move to UNIX.
!     Updated by TEC II 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and to suppress writing of status info
!      if file: "QUIET.INT" exists in default directory.
!
!     This routine reads the file: SUPARM01.INT to determine the date on which
!     to begin updating and the length of the update.  It then reads modeled
!     subbasin runoff and storages from files: xxx??.SUM, lumps them, and
!     writes the lumped values to file xxxALL.SUM.  It assumes, of course, that
!     the dates are valid for all subbasins.
!
MODULE LumpStorMain

CONTAINS

      SUBROUTINE DO_LUMPSTOR(ErrStatus, HandleIn, Lake)

      USE FcstUtil

      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus
      INTEGER, INTENT(IN)    :: HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      !
      !   iNSbsns = Maximum number of subbasins.
      !   iLength = Buffer length, set as large as possible
      !             to reduce processing and i/o time if neces.
      !   Hdr     = number of header records in *.SUM
      !
      INTEGER (KIND=INTEGER2), PARAMETER :: iNSbsns = 35
      INTEGER (KIND=INTEGER2), PARAMETER :: iLength = 730
      INTEGER (KIND=INTEGER2), PARAMETER :: Hdr = 2

      INTEGER (KIND=INTEGER2)  :: iQ(6), Scale, ScaleT

      INTEGER :: I, isD, isM, isY, lD, lM, lY, bD, bM, bY, IOS
      INTEGER :: NoSubbasins, Last, PrgD, Length, Pd
      INTEGER :: Start, Subbasin, J, SRN, SRNl, Day
      INTEGER :: NoPieces, Piece, pLength, ValidEnd, TotalEnd

      LOGICAL :: Flag

      REAL :: Qq(iLength, 6), qArea(iNSbsns), qAc(iNSbsns, 6), qAt(6), qTotal

      CHARACTER (LEN=1)  :: Lf, Cr
      CHARACTER (LEN=2)  :: CrLf
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=4)  :: OverTheLake
      CHARACTER (LEN=13) :: Name, S
      CHARACTER (LEN=99) :: File21, File31, File41, File51, File61, FName

      Lf = CHAR(10)
      Cr = CHAR(13)
      CrLf = Cr//Lf

      WinHandle = HandleIn
      ErrorLevel = 0
      ErrorMessage = ' '
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read parameter file for necessary dates.
!
      File21 = TRIM(ThisLakeDir) // 'PARAM01.INT'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Skip ID header line and subbasin area line
!
      READ(21, *, ERR=822)
      READ(21, *, ERR=822)

!
!     Get starting and last meteorological dates of simulation.
!
      READ(21, 2102, ERR=822) isD, isM, isY
      READ(21, 2102, ERR=822) lD, lM, lY
      bD = isD
      bM = isM
      bY = isY

      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read basin name and previous program designator.
!
      File31 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM = 'FORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      READ(31, 3103, REC=1, ERR=832) Name
      READ(31, 3105, REC=2, ERR=832) NoSubbasins, Pd, OverTheLake

!
!     Compute sequence no. used in determining length of update.
!     Last day of LBRM run.
!
      CALL DateSequence(lD, lM, lY, Last); IF (ErrorLevel .NE. 0) GOTO 899

!
!     "IF"-Test description....
!          (1) If this is TRUE then LBRM was just run the 1st time
!          (2) If this is TRUE then LBRM was just run the 2nd time
!          (3) LBRM just run as part of a "setup" or simulation procedure 
!              to find *.SUM corresponding to the *.CLM data sets.
!
      IF (Pd .EQ. 5) THEN        ! **** (1) ****
         PrgD = 6
         READ(31, 3104, REC=9, ERR=832) isD, isM, isY 
         CALL DateSequence(isD, isM, isY, Start); IF (ErrorLevel .NE. 0) GOTO 899
         Length = Last - Start + 1
         CLOSE (31)
         CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899
      ELSE
         IF (Pd .EQ. 8) THEN     ! **** (2) ****
            PrgD = 9
         ELSE                    ! **** (3) ****
            PrgD = 6
         END IF
         CALL DateSequence(isD, isM, isY, Start)    ! start sequence no. for update
         IF (ErrorLevel .NE. 0) GOTO 899
         Length = Last - Start + 1
         CALL DateSequence(bD, bM, bY, Start); IF (ErrorLevel .NE. 0) GOTO 899
         CLOSE (31)
         CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

!
!     Adjust the value of Last (and lD,lM,lY) to 1 day AFTER end of run
!
      Last = Last + 1
      CALL SequenceDate(lD, lM, lY, Last); IF (ErrorLevel .NE. 0) GOTO 899
      BnS = Name(1:3)

!
!     Read subbasin areas.
!
      File41 = TRIM(ThisLakeDir) // 'AREA.'//BnS
      OPEN(UNIT=41, FILE=TRIM(File41), STATUS='OLD', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
      qTotal = 0.
      DO Subbasin = 1, NoSubbasins
         READ(41, 4101, ERR=842) qArea(Subbasin)
         qTotal = qTotal + qArea(Subbasin)
      ENDDO
      CLOSE (41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Assign values for area array.
!
      DO J = 0, 5
        DO Subbasin = 1, NoSubbasins
          qAc(Subbasin, J + 1) = qArea(Subbasin)
        END DO
      END DO

!
!*****MODULE-1*********************************************** This module must
!************************************************************ be present in
!*                                                        !** programs:
!*    This module eliminates subbasin storages            !** QUANTILES and
!*    as per user-supplied code.                          !** CUVOCDASF, in the
!*                                                        !** QUANTILES proce-
!*    if(BnS .eq. '???') qAc(??, ? + 1) = 0.              !** dure, and program
!*                                                        !** LUMPSTOR in the
!************************************************************ FORECAST proce-
!************************************************************ dure.
!
      IF (BnS .EQ. 'MIC') THEN
        qAc(21, 4) = 0.  !4=groundwater
        qAc(23, 4) = 0.
        qAc(24, 4) = 0.
        qAc(25, 4) = 0.
        qAc(26, 4) = 0.
        qAc(27, 4) = 0.
      END IF
      IF (BnS .EQ. 'HUR') THEN
        qAc( 2, 4) = 0.  !4=groundwater
        qAc( 3, 4) = 0.
        qAc( 4, 4) = 0.
        qAc( 7, 4) = 0.
      END IF
      IF (BnS .EQ. 'GEO') THEN
        qAc( 4, 4) = 0.  !4=groundwater
        qAc( 5, 4) = 0.
        qAc( 6, 4) = 0.
        qAc( 7, 4) = 0.
        qAc( 8, 4) = 0.
        qAc(11, 4) = 0.
      END IF

!************************************************************

      DO J = 0, 5
        qAt(J + 1) = 0.
        DO Subbasin = 1, NoSubbasins
          qAt(J + 1) = qAt(J + 1) + qAc(Subbasin, J + 1)
        END DO
      END DO

!
!     Open a subbasin model storage file.
!
      File51 = TRIM(ThisLakeDir) // BnS // '01.SUM'
      OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
      READ(51, REC=1, ERR=852) Scale, SRN             ! skip first two bytes
      READ(51, REC=2, ERR=852) ValidEnd, TotalEnd
      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899

!
!     SRN = record no. of 1st day of update minus 1
!
      SRN = (Start - SRN + 1) + Hdr - 1

!
!     Open lumped basin model storage file.
!
      File61 = TRIM(ThisLakeDir) // BnS // 'ALL.SUM'
      OPEN(UNIT=61, FILE=TRIM(File61), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
      READ(61, REC=1, ERR=862) Scale, SRNl, ScaleT         ! skip first two bytes
      CLOSE (61)
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899

!
!     SRNl = record no. of 1st day of update minus 1
!
      SRNl = (Start - SRNl + 1) + Hdr - 1

!
!     Lump runoff and storages from all subbasins and write to "lumped" file.
!
      NoPieces = Length / iLength
      IF (NoPieces * iLength .NE. Length) NoPieces = NoPieces + 1
      pLength = MIN(iLength, Length)
      SRN = SRN - pLength
      SRNl = SRNl - pLength

      DO Piece = 1, NoPieces
        SRN = SRN + pLength
        SRNl = SRNl + pLength
        pLength = MIN(iLength, Length - (Piece - 1) * iLength)
        DO Day = 1, pLength
          DO I = 1, 6
            Qq(Day, I) = 0.
          END DO
        END DO
        DO Subbasin = 1, NoSubbasins
          WRITE(File51, 8001) TRIM(ThisLakeDir), BnS, Subbasin
          OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,         &
     &         ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
          CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
          READ(51, REC=1, ERR=852) Scale, Scale, Scale, Scale
          DO Day = 1, pLength
            READ(51, REC=SRN + Day, ERR=852) (iQ(I), I = 1, 6)
            DO I = 1, 6
              Qq(Day, I) = Qq(Day, I) + iQ(I) * qAc(Subbasin, I)
            END DO
            Qq(Day, 4) = Qq(Day, 4) - iQ(4) * qAc(Subbasin, 4) + iQ(4)    &
     &                   * qAc(Subbasin, 4) * Scale
          END DO
          CLOSE (51)
          CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
        END DO

        File61 = TRIM(ThisLakeDir) // BnS // 'ALL.SUM'
        OPEN(UNIT=61, FILE=TRIM(File61), STATUS='OLD', RECL=12,           &
     &       ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
        CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
        DO Day = 1, pLength
          DO I = 1, 6
            Qq(Day, I) = Qq(Day, I) / qAt(I)
          END DO
          Qq(Day, 4) = Qq(Day, 4) / ScaleT
          DO I = 1, 6
            iQ(I) = NINT(Qq(Day, I))
          END DO
          WRITE(61, REC = SRNl + Day, ERR=863) (iQ(I), I = 1, 6)
        END DO
        CLOSE (61)
        CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

      File61 = TRIM(ThisLakeDir) // BnS // 'ALL.SUM'
      OPEN(UNIT=61, FILE=TRIM(File61), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, REC=2, ERR=863) ValidEnd, TotalEnd
      CLOSE (61)
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Update BSNNM file.
!     The date written into record 9 is either the day after the end    21mar95
!     of the provisional data update (if PrgD is 6) or                  21mar95
!     the forecast start date (if PrgD is 9).   This is                 21mar95
!     valid because if we are doing a forecast we may (depending upon   21mar95
!     the chosen forecast start date) be overwriting part of the *.SUM  21mar95
!     file which was based upon real provisional data.  In this case we 21mar95
!     must rewrite those records correctly the next time we update the  21mar95
!     provisional data or create an outlook with a later date.          21mar95
!     The variables lD,lM,lY have not been changed since they were      21mar95
!     calculated from the simulation end date.                          21mar95
!
      File31 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(31, 3105, REC=2, ERR=833) NoSubbasins, PrgD, OverTheLake, Lf
      IF (PrgD .EQ. 9) CALL SequenceDate(lD,lM,lY, Start)
      WRITE(31, 3104, REC=9, ERR=833) lD, lM, lY, Lf
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
  821 ErrorMessage = 'LUMPSTOR: Error opening file '   //TRIM(File21)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'LUMPSTOR: Error reading file '   //TRIM(File21)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'LUMPSTOR: Error opening file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'LUMPSTOR: Error reading file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'LUMPSTOR: Error writing to file '//TRIM(File31)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'LUMPSTOR: Error opening file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'LUMPSTOR: Error reading file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'LUMPSTOR: Error opening file '   //TRIM(File51)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'LUMPSTOR: Error reading file '   //TRIM(File51)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'LUMPSTOR: Error opening file '   //TRIM(File61)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'LUMPSTOR: Error reading file '   //TRIM(File61)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'LUMPSTOR: Error writing to file '//TRIM(File61)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_LumpStor...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup and return
!
  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN
!
!  FORMAT statements
!
 2102 FORMAT(2I2, I4)

 3103 FORMAT(A10)
 3104 FORMAT(2I3, I5, A1)
 3105 FORMAT(2I3, A4, 1X, A1)

 4101 FORMAT(E13.6E2)

 8001 FORMAT(A, A3, I2.2, '.SUM')       !  xxxnn.SUM   (e.g. SUP05.SUM)

 9001 FORMAT(' LUMPSTOR: Invalid sequencing. Must be run immediately after LBRM.')

 9930 FORMAT('Processing piece ', I2, ' of ', I2)
 9931 FORMAT('~^~Processing piece ', I2, ' of ', I2)


      END SUBROUTINE DO_LUMPSTOR

END MODULE LumpStorMain

