!
MODULE NewFcstSubs

      USE FcstUtil

      IMPLICIT   NONE
      INTEGER, PARAMETER :: MaxNoOfLines = 1000
      INTEGER            :: NumSortLines
      CHARACTER (LEN=80) :: Lines1(MaxNoOfLines), Lines2(MaxNoOfLines)


CONTAINS


!------------------------------------------------------------------
      SUBROUTINE Sort(ArrayOfLines, LineCount)

      IMPLICIT NONE

      CHARACTER (LEN=80), INTENT(INOUT) :: ArrayOfLines(MaxNoOfLines)
      INTEGER           , INTENT(IN)    :: LineCount

      INTEGER             :: I, J, K, M, P
      CHARACTER (LEN=80)  :: X

      IF (LineCount .GT. MaxNoOfLines) THEN
         ErrorMessage = 'NEWFRCST: Too many lines in SORT()!'
         CALL PassMsg
         GOTO 898
      END IF

      P = LineCount
      DO WHILE (P .GT. 0)
         K = LineCount - P
         J = 1
         DO WHILE (J .LE. K)
            I = J
            M = I + P
            DO WHILE ((I .GT. 0) .AND. (ArrayOfLines(I) .GT. ArrayOfLines(M)))
               X = ArrayOfLines(I)
               ArrayOfLines(I) = ArrayOfLines(M)
               ArrayOfLines(M) = X
               M = I
               I = I - P
            END DO
            J = J + 1
         END DO
         P = P / 2
      END DO
      RETURN

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : Sort...'
      CALL PassMsg
      RETURN

      END SUBROUTINE Sort

!------------------------------------------------------------------
      LOGICAL FUNCTION CompareStateFiles (FileName1, FileName2)
!
!     Compares two files for equality (to 80 chars/line);
!     returns .TRUE. if same.  
!
!     21jun95, TSH   : The line from the two files which contains the 
!                      "Date of Forecast" (the day upon which the forecast
!                      run is made;  i.e. today's date) is no longer tested.
!     06may97, TSH   : OOPS!  The logic for that change was wrong.  Fixed it.
!     20jul2001, TSH : Hmmm... Something must've been changed since May97.
!                      Looking at the program today I find that the 
!                      "Date of Forecast" is never differentiated at all.
!                      Thus it was still being tested for equivalence just
!                      like all of the other lines from the SIMULATE file.
!                      I know that wasn't intended.  So I fixed it.
!
      IMPLICIT NONE

      CHARACTER (LEN=*), INTENT(IN) :: FileName1, FileName2

      INTEGER            :: I, IOS1, IOS2, LC1, LC2, U1, U2
      LOGICAL            :: Tmp, FilesMatch
      CHARACTER (LEN=80) :: Line1, Line2

      CompareStateFiles = .FALSE.      ! default value in case of early exit
      FilesMatch = .TRUE.              ! default value in case of early exit

!
!     Read the 2 files, storing the lines which are *NOT* part of 
!     the SIMULATE file contents.  So the lines which are stored
!     here are the lines which give a file name, date, size.
!
!     The assumption is made that all of these lines are BEFORE the
!     contents of SIMULATE.
!
!     If either file is not there do NOT spit out an error message.  Just return 
!     to the calling procedure with CompareStateFiles = .FALSE.
!
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(FileName1), STATUS='OLD', IOSTAT=IOS1)
      IF (IOS1 .NE. 0) THEN
         CompareStateFiles = .FALSE.
         RETURN
      END IF
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      LC1 = 0
      READ(U1, 1000, IOSTAT=IOS1) Line1
      Tmp = (Line1(1:44) .NE. '---------- SIMULATE FILE CONTENTS ----------')
      DO WHILE ((IOS1 .EQ. 0) .AND. (Tmp))
         LC1 = LC1 + 1
         Lines1(LC1) = TRIM(Line1)
         READ(U1, 1000, IOSTAT=IOS1) Line1
         Tmp = (Line1(1:44) .NE. '---------- SIMULATE FILE CONTENTS ----------')
      END DO

      U2 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U2, FILE=TRIM(FileName2), STATUS='OLD', IOSTAT=IOS2)
      IF (IOS2 .NE. 0) THEN
         CompareStateFiles = .FALSE.
         CLOSE(U1)
         CALL FileWasClosed(U1);  IF (ErrorLevel .NE. 0) GOTO 899
         RETURN
      END IF
      CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899

      LC2 = 0
      READ(U2, 1000, IOSTAT=IOS2) Line2
      Tmp = (Line2(1:44) .NE. '---------- SIMULATE FILE CONTENTS ----------')
      DO WHILE ((IOS2 .EQ. 0) .AND. (Tmp))
         LC2 = LC2 + 1
         Lines2(LC2) = TRIM(Line2)
         READ(U2, 1000, IOSTAT=IOS2) Line2
         Tmp = (Line2(1:44) .NE. '---------- SIMULATE FILE CONTENTS ----------')
      END DO

!
!     Did each file have the same number of lines not including the
!     contents of SIMULATE?  If not then they obviously did not match.
!
      IF (LC1 .NE. LC2) THEN
         FilesMatch = .FALSE.
         GOTO 888
      END IF

!
!     Sort the non-SIMULATE lines.
!
      CALL Sort(Lines1, LC1); IF (ErrorLevel .NE. 0) GOTO 899
      CALL Sort(Lines2, LC2); IF (ErrorLevel .NE. 0) GOTO 899

      FilesMatch = .TRUE.     ! innocent until proven guilty
      DO I = 1, LC1
         IF (FilesMatch) THEN
           Line1 = TRIM(Lines1(I))
           Line2 = TRIM(Lines2(I))
           CALL Caps(Line1); IF (ErrorLevel .NE. 0) GOTO 899
           CALL Caps(Line2); IF (ErrorLevel .NE. 0) GOTO 899
           IF (TRIM(Line1) .NE. TRIM(Line2)) FilesMatch = .FALSE.
         END IF
      END DO
      IF (.NOT. FilesMatch) GOTO 888

!
!     Now compare contents of SIMULATE file
!
      LC1 = 0
      READ(U1, 1000, IOSTAT=IOS1) Line1
      DO WHILE (IOS1 .EQ. 0)
         IF (Line1(17:32) .NE. 'Date of forecast') THEN
            LC1 = LC1 + 1
            Lines1(LC1) = TRIM(Line1)
         END IF
         READ(U1, 1000, IOSTAT=IOS1) Line1
      END DO

      LC2 = 0
      READ(U2, 1000, IOSTAT=IOS2) Line2
      DO WHILE (IOS2 .EQ. 0)
         IF (Line2(17:32) .NE. 'Date of forecast') THEN
            LC2 = LC2 + 1
            Lines2(LC2) = TRIM(Line2)
         END IF
         READ(U2, 1000, IOSTAT=IOS2) Line2
      END DO

      IF (LC1 .NE. LC2) THEN
         FilesMatch = .FALSE.
         GOTO 888
      END IF

      FilesMatch = .TRUE.     ! innocent until proven guilty
      DO I = 1, LC1
         IF (FilesMatch) THEN
           Line1 = TRIM(Lines1(I))
           Line2 = TRIM(Lines2(I))
           CALL Caps(Line1); IF (ErrorLevel .NE. 0) GOTO 899
           CALL Caps(Line2); IF (ErrorLevel .NE. 0) GOTO 899
           IF (TRIM(Line1) .NE. TRIM(Line2)) FilesMatch = .FALSE.
         END IF
      END DO
            
 888  CLOSE(U1)
      CALL FileWasClosed(U1);  IF (ErrorLevel .NE. 0) RETURN
      CLOSE(U2)
      CALL FileWasClosed(U2);  IF (ErrorLevel .NE. 0) RETURN

      CompareStateFiles = FilesMatch

      RETURN

!
!  Error handling
!
  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : CompareStateFiles...'
      CALL PassMsg
      RETURN
!
 1000 FORMAT(A80)
      END FUNCTION CompareStateFiles

END MODULE NewFcstSubs

!==============================================================================

MODULE NewFcstMain

CONTAINS
!------------------------------------------------------------------------------
!------------------------------------------------------------------------------
!------------------------------------------------------------------------------
!     Compare state information in file named "SIMULTMP.NEW" and "SIMULTMP.OLD".
!     If either file does not exist or the files do not compare exactly then
!     exit with "NO ERROR" condition.  In this case the calling Windows
!     application will continue in the default manner by going ahead with
!     the processing of the entire forecast package.
!
!     If both files exist and they are identical, then we don't need to run
!     the majority of AHPS.  The Windows application will handle the branch
!     appropriately to begin in forecast procedure with the computation of
!     outlook weights and regenerate the probabilistic outlooks.  We will 
!     signal the calling Windows application to do that by returning an 
!     error code of 2.
!
!     Updated 30may95 by TSH to work under Unix variants as well as MS-DOS.
!     Updated 21jun95 by TSH to ignore "Date of Forecast" line in SIMULATE file
!
!     Updated 28sep99 by TSH.  Major changes.
!        1) Converted from Fortran 77 syntax to Fortran 90 syntax
!        2) Reversed the "error condition" flag to mean opposite of previous
!           interpretation.  Previously a "no error" meant to skip to the
!           outlook stuff and an "error" meant to run everything.  I changed
!           that to make it clearer in the Windows app that handles the return 
!           value.
!
      SUBROUTINE DO_NEWFRCST(ErrStatus, HandleIn, Lake)

      USE NewFcstSubs

      IMPLICIT  NONE
      INTEGER, INTENT(INOUT)  ::  ErrStatus
      INTEGER, INTENT(IN)     ::  HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER             :: I, J
      LOGICAL             :: TheyMatch
      CHARACTER (LEN=25)  :: S
      CHARACTER (LEN=99)  :: File1, File2

      WinHandle = HandleIn
      ErrStatus = 0
      ErrorLevel = 0
      CALL ClearFilesOpen()
      
!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Compare the information
!
      File1 = TRIM(ThisLakeDir) // 'SIMULTMP.NEW'
      File2 = TRIM(ThisLakeDir) // 'SIMULTMP.OLD'

      TheyMatch = CompareStateFiles(File1, File2)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Delete SIMULTMP.NEW
!
      CALL DeleteFile('SIMULTMP.NEW')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Take appropriate action based on status
!
      IF (TheyMatch) THEN
         ErrorMessage = 'No new simulation runs needed.'
         CALL PassMsg
         ErrorLevel = 2       ! ErrorLevel=2  => it ran ok, but skip stuff
         GOTO 999
      END IF
      GOTO 999

!
!  Error handling
!
  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_NewFrcst...'
      CALL PassMsg
      GOTO 999

 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

      END SUBROUTINE DO_NEWFRCST

END MODULE NewFcstMain

