!
!     program compute_met_outlooks_for_plots_of_some_runoff_outlooks
!
!     adapted 28oct88 by TEC II (for PC)
!     small corrections made 06jun89 by Tim Hunter at direction of TEC II
!     updated 28oct92, TSH, TEC II for 4-byte dates and move to UNIX
!     Updated by TEC II, 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and for change in use of forecast
!      start date instead of provisional data end date and to suppress writing
!      of status info if file: "QUIET.INT" exists in default directory.
!
!     This routine compiles the simulation/forecast meteorological outlooks
!     used in the simulation/forecast package for purposes of plotting.
!
!     Meteorological data are used from the beginning plot date
!     through the end of the forecast period; the dates are passed through
!     file: BSNNM.
!

MODULE CMOFPOSR_Main

      USE FcstUtil

      INTEGER, PARAMETER :: iNSbsns = 35  ! Maximum number of subbasins.
      INTEGER, PARAMETER :: iLength = 731 ! Buffer length, set as large as possible
                                          ! to reduce processing & i/o time
      INTEGER, PARAMETER :: HdrCLM  = 2   ! number of header records in CLM-type files

CONTAINS

      SUBROUTINE DO_CMOFP(ErrStatus, HandleIn, Lake)

      IMPLICIT NONE
      INTEGER, INTENT(INOUT)        :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: Tmin, Tmax, Precip, BsnID

      INTEGER :: I, iD, iM, iY, sD, sM, sY, jD, jM, jY, hD, hM, hY
      INTEGER :: NoSubbasins, Subbasin, SRNl, SRN, Excess, Length
      INTEGER :: HstStart, HstEnd, PltStart, FcEnd, FcStart
      INTEGER :: Starts, X01Start, AllStart, ClmEnd, PrvEnd
      INTEGER :: NoPieces, Piece, pLength 

      REAL :: qTmin, qTmax, qPrecip

      REAL, DIMENSION(iNSbsns) :: qArea
      REAL, DIMENSION(iLength) :: qMin, qMax, qWater, qMinA, qMaxA, qWaterA

      CHARACTER (LEN=1), PARAMETER :: Lf = CHAR(10)
      CHARACTER (LEN=3)  :: BnS
      CHARACTER (LEN=11) :: Name
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File1, File2, File3, File4, File5, File6, File7

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF


!
!     Get basin name and forecast/simulation dates.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1101, REC=1, ERR=812) Name
      BnS = Name(1:3)
      READ(11, 1102, REC=2, ERR=812) NoSubbasins

!
!     Get earliest changed/added data date since last time this program was
!     run.
!
      READ(11, 1105, REC=10, ERR=812) sD, sM, sY
      CALL DateSequence(sD, sM, sY, Starts); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get forecast start and end dates.  Also set the provisional data end.
!
      READ(11, 1105, REC=5, ERR=812) sD, sM, sY 
      READ(11, 1105, REC=6, ERR=812) iD, iM, iY
      CALL DateSequence(sD, sM, sY, FcStart); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(iD, iM, iY, FcEnd);   IF (ErrorLevel .NE. 0) GOTO 899
      PrvEnd = FcStart - 1                   

!
!     Get plot start date.
!
      READ(11, 1105, REC=7, ERR=812) jD, jM, jY
      CALL DateSequence(jD, jM, jY, PltStart); IF (ErrorLevel .NE. 0) GOTO 899

      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get historical forecast dates (dates at which the "forecast" meteorology
!     is read from the climatic subbasin files: xxx??.CLM).
!
      File2 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, 2101, ERR=822) hD
      READ(21, 2101, ERR=822) hM
      READ(21, 2101, ERR=822) hY
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Sequence number for historical forecast start end end.
!
      CALL DateSequence(hD, hM, hY, HstStart); IF (ErrorLevel .NE. 0) GOTO 899
      HstEnd = HstStart + (FcEnd - FcStart)

!
!     Input subbasin areas.
!
      File3 = TRIM(ThisLakeDir) // 'AREA.' // BnS
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, NoSubbasins
        READ(31, 3101, ERR=832) qArea(I)
      END DO
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Check plot beginning date is within the provisional lumped met data base.
!
      File4 = TRIM(ThisLakeDir) // BnS // 'ALL.MET'
      OPEN(UNIT=41, FILE=TRIM(File4), STATUS='OLD', RECL=6,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899

!
!     J = sequence number for start date of file.
!
      READ(41, REC=1, ERR=842) BsnID, AllStart
      SRNl = HdrCLM + 1 + PltStart - AllStart
      IF (PltStart .LT. AllStart) THEN
        WRITE(ErrorMessage, 9001)
        CALL PassMsg
        WRITE(ErrorMessage, 9002)
        CALL PassMsg
        GOTO 898
      END IF

!
!     Get starting record no. in provisional data base.
!
      File5 = TRIM(ThisLakeDir) // BnS // '01.PRV'
      OPEN(UNIT=51, FILE=TRIM(File5), STATUS='OLD', RECL=6,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899

!
!     I = sequence number for start date of file.
!
      READ(51, REC=1, ERR=852) BsnID, X01Start
      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Difference in start between xxx01.PRV and xxxALL.MET.
!
      Excess = X01Start - AllStart

!
!     Record number of simulation start.
!
      SRN = HdrCLM + 1 + Starts - X01Start

!
!     Update lumped provisional met. data base.
!
      Length = PrvEnd - Starts + 1
      NoPieces = Length / iLength
      IF (NoPieces * iLength .NE. Length) NoPieces = NoPieces + 1
      pLength = MIN(iLength, Length)
      SRN = SRN - pLength

      DO Piece = 1, NoPieces

        SRN = SRN + pLength
        pLength = MIN(iLength, Length - (Piece - 1) * iLength)
       
        qMin   (1:pLength) = 0.
        qMax   (1:pLength) = 0.
        qWater (1:pLength) = 0.
        qMinA  (1:pLength) = 0.
        qMaxA  (1:pLength) = 0.
        qWaterA(1:pLength) = 0.

!
!       For each provisional subbasin data file:
! 
        DO Subbasin = 1, NoSubbasins
           WRITE(File5, 8001) TRIM(ThisLakeDir), BnS, Subbasin
           WRITE(ErrorMessage, 211) Piece, NoPieces, TRIM(File5)
           CALL PassMsg
           OPEN(UNIT=51, FILE=TRIM(File5), STATUS='OLD', RECL=6,         &
     &          ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
           CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
           DO I = 1, pLength
              READ(51, REC = SRN + I - 1, ERR=852) Tmin, Tmax, Precip
              IF(Tmin .GT. -9000) THEN
                qMin(I) = qMin(I) + Tmin * qArea(Subbasin)
                qMinA(I) = qMinA(I) + qArea(Subbasin)
              END IF
              IF (Tmax .GT. -9000) THEN
                qMax(I) = qMax(I) + Tmax * qArea(Subbasin)
                qMaxA(I) = qMaxA(I) + qArea(Subbasin)
              END IF
              IF (Precip .GE. 0) THEN
                qWater(I) = qWater(I) + Precip * qArea(Subbasin)
                qWaterA(I) = qWaterA(I) + qArea(Subbasin)
              END IF
           END DO
           CLOSE (51)
           CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
        END DO    ! {DO Subbasin = 1, NoSubbasins}

        WRITE(ErrorMessage, 211) Piece, NoPieces, TRIM(File4)
        CALL PassMsg
        Tmin = -9999
        Tmax = -9999
        Precip = - 9999
        DO I = 1, pLength
          IF (qMinA(I) .GT. 0.) Tmin = NINT(qMin(I) / qMinA(I))
          IF (qMaxA(I) .GT. 0.) Tmax = NINT(qMax(I) / qMaxA(I))
          IF (qWaterA(I) .GT. 0.) Precip = NINT(qWater(I) / qWaterA(I))
          WRITE(41, REC=SRN+Excess+I-1, ERR=843) Tmin, Tmax, Precip
        END DO

      END DO    ! {DO Piece = 1, NoPieces}

!
!     Update header in xxxALL.MET to reflect added records.
!
      WRITE(41, REC=2, ERR=843) PrvEnd

!
!     Update BSNNM. file to reflect update of xxxALL.MET
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(11, 1105, REC=10, ERR=813) sD, sM, sY, Lf
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read lumped provisional met. file for plotting up to end of provisional
!     data.
!
      File6 = TRIM(ThisLakeDir) // 'SMETOLTK.INT'
      OPEN(UNIT=61, FILE=TRIM(File6), STATUS='OLD', RECL=12,            &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=61)
      CLOSE (61, STATUS = 'DELETE')
 61   OPEN(UNIT=61, FILE=TRIM(File6), STATUS='NEW', RECL=12,            &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Replace missing data at beginning with zeroes for plotting purposes.
!
      qTmin = 0.
      qTmax = 0.
      qPrecip = 0.
      DO I = PltStart, PrvEnd
        READ(41, REC = SRNl + I - PltStart, ERR=842) Tmin, Tmax, Precip
        IF (Tmin .GE. -9000) qTmin = Tmin / 100.
        IF (Tmax .GE. -9000) qTmax = Tmax / 100.
        IF (Precip .GE. 0) qPrecip = Precip / 1000.
        WRITE(61, ERR=863) qPrecip, qTmin, qTmax
      END DO
      CLOSE(41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(ErrorMessage, 212) TRIM(File4)
      CALL PassMsg

!
!     Get starting record number for forecast files.
!
      File7 = TRIM(ThisLakeDir) //BnS//'01.CLM'
      OPEN(UNIT=71, FILE=TRIM(File7), STATUS='OLD', RECL=6,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=871)
      CALL FileWasOpened(71); IF (ErrorLevel .NE. 0) GOTO 899
      READ(71, REC=1, ERR=872) BsnID, I
      READ(71, REC=2, ERR=872) ClmEnd
      SRN = HdrCLM + 1 + HstStart - I
      CLOSE (71)
      CALL FileWasClosed(71); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Check historical forecast dates are within the data base.
!
      IF (HstStart .LT. I .OR. HstStart + FcEnd - FcStart .GT. ClmEnd) THEN
         WRITE(ErrorMessage, 9011)
         CALL PassMsg
         WRITE(ErrorMessage, 9012) TRIM(File4)
         CALL PassMsg
         WRITE(ErrorMessage, 9013)
         CALL PassMsg
         CLOSE (61, STATUS = 'DELETE')
         CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 898
      ENDIF

!
!     Read forecast file for each subbasin and lump for plot file.
!
      Length = HstEnd - HstStart + 1
      NoPieces = Length / iLength
      IF (NoPieces * iLength .NE. Length) NoPieces = NoPieces + 1
      SRN = SRN - MIN(iLength, Length)
      DO Piece = 1, NoPieces
        pLength = MIN(iLength, Length - (Piece - 1) * iLength)
        SRN = SRN + pLength
        qMin   (1:pLength) = 0.
        qMax   (1:pLength) = 0.
        qWater (1:pLength) = 0.
        qMinA  (1:pLength) = 0.
        qMaxA  (1:pLength) = 0.
        qWaterA(1:pLength) = 0.

        DO Subbasin = 1, NoSubbasins
          WRITE(ErrorMessage, 211) Piece, NoPieces, TRIM(File7)
          CALL PassMsg
          WRITE(File7, 8002) TRIM(ThisLakeDir), BnS, Subbasin
          OPEN(UNIT=71, FILE=TRIM(File7), STATUS='OLD', RECL=6,         &
     &         ACCESS='DIRECT', FORM='UNFORMATTED', ERR=871)
          CALL FileWasOpened(71); IF (ErrorLevel .NE. 0) GOTO 899
          DO I = 1, pLength
             READ(71, REC = SRN+I-1, ERR=872) Tmin, Tmax, Precip
             IF (Tmin .GT. -9000) THEN
                qMin(I) = qMin(I) + Tmin * qArea(Subbasin) / 100.
                qMinA(I) = qMinA(I) + qArea(Subbasin)
             END IF
             IF (Tmax .GT. -9000) THEN
                qMax(I) = qMax(I) + Tmax * qArea(Subbasin) / 100.
                qMaxA(I) = qMaxA(I) + qArea(Subbasin)
             END IF
             IF (Precip .GE. 0) THEN
                qWater(I) = qWater(I) + Precip * qArea(Subbasin) / 1000.
                qWaterA(I) = qWaterA(I) + qArea(Subbasin)
             END IF
          END DO
          CLOSE (71)
          CALL FileWasClosed(71); IF (ErrorLevel .NE. 0) GOTO 899
        END DO

        WRITE(ErrorMessage, 211) Piece, NoPieces, 'SMETOTLK.INT'
        CALL PassMsg

        DO I = 1, pLength
          IF (qMinA(I) .GT. 0.) qTmin = qMin(I) / qMinA(I)
          IF (qMaxA(I) .GT. 0.) qTmax = qMax(I) / qMaxA(I)
          IF (qWaterA(I) .GT. 0.) qPrecip = qWater(I) / qWaterA(I)
          WRITE(61, ERR=863) qPrecip, qTmin, qTmax
        END DO
      END DO

      CLOSE (61)
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899

      WRITE(ErrorMessage, 216)
      CALL PassMsg

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'CMOFPOSR: Error opening file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  872 ErrorMessage = 'CMOFPOSR: Error reading file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'CMOFPOSR: Error writing file ' // TRIM(File7)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_CMOFP...'
      CALL PassMsg
      GOTO 999

!  -----------------------

  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

!
!  FORMATs
!
  211 FORMAT('~^~Processing piece ', I4, ' of ', I4, ' (', A, ')   ')
  212 FORMAT('~^~Finished with ', A, '                             ')
  216 FORMAT('~^~Finished ...                                      ')

 1101 FORMAT(A11)
 1102 FORMAT(I3)
 1105 FORMAT(2I3, I5, A1)
 2101 FORMAT(I8)
 3101 FORMAT(E13.6E2)

 8001 FORMAT(A, A3, I2.2, '.PRV')
 8002 FORMAT(A, A3, I2.2, '.CLM')

 9001 FORMAT('CMOFPOSR: Requested plot start date is outside of')
 9002 FORMAT('          provisional lumped meteorological data period.')
 9011 FORMAT('CMOFPOSR: Requested forecast dates are outside of climatic')
 9012 FORMAT('          data period.   ', A)
 9013 FORMAT('          has been updated but the plot file has not.')

      END SUBROUTINE DO_CMOFP

END MODULE CMOFPOSR_Main



