!
!     program Create_parameter_files_for_some_runoff_outlook_model_runs
!
!     Updated by TEC II 28sep88, 26oct88 (for PC)
!     Updated 27oct92, TSH, TEC II for 4-byte dates and move to UNIX
!     Updated by TEC II, 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and allowance of forecast start dates
!      earlier than end of provisional data and to suppress writing of status
!      info if file: "QUIET.INT" exists in default directory.
!
!
!     If RunEvp = 0:
!
!     This routine recreates the overlake parameter file for use with the Lake
!     Evaporation Model to simulate to the end of the forecast data period.
!
!     This routine also checks that the forecast meteorological data file
!     exists and encompasses the desired forecast dates.
!
!
!     If RunEvp = 1:
!
!     This routine recreates subbasin parameter files for use with the Large
!     Basin Runoff Model to generate model storage files to the end of the
!     forecast data period.
!
!     This routine also checks that the forecast meteorological data files
!     exist and encompass the desired forecast dates, by looking at the first
!     subbasin file only.
!
!
!
!     Forecast dates are taken as established in DATEFILE.INT and BSNNM.
!
!     THIS PROGRAM SHOULD BE CALLED AFTER ALL PROVISIONAL DATA CALCULATIONS,
!     INCLUDING PROVISIONAL MODEL RUNS, ARE FINISHED.
!


MODULE CPFSR_subs

      USE FcstUtil

      INTEGER, PARAMETER :: Hdr  =  2   ! Hdr  = header lines in *.SUM files
      INTEGER, PARAMETER :: iNsn = 35   ! iNSn = max. number of subbasins allowed
      INTEGER, PARAMETER :: NoBC = 999  ! NoBC = max. no. of boundary conditions allowed

CONTAINS

      SUBROUTINE DO_EVAP

      IMPLICIT NONE

      INTEGER (KIND=INTEGER2) :: I2

      INTEGER :: I, iD, iM, iY, Start, Last, LfD, LfM, LfY, jDsave
      INTEGER :: jMsave, jYsave, Forecast, DataStart, DataEnd
      INTEGER :: NoOfLayers, iDsave, iMsave, U1, U3, U4, U5, U6

      LOGICAL :: Flag

      CHARACTER (LEN=1), PARAMETER :: Lf = CHAR(10)
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=80) :: Line
      CHARACTER (LEN=99) :: File1, File3, File4, File5, File6

!
!     Input basin name and provisional end date and forecast end date.
!
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,               &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1101, REC=1, ERR=812) BnS
      READ(U1, 1102, REC=2, ERR=812) I
      IF (I .LT. 5 .AND. I .NE. 0) THEN
         WRITE(ErrorMessage, 9001)
         CALL PassMsg
         CLOSE(U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 898
      END IF

!
!     Check that the ZSEVAPPR.xxx file has been updated through the end of the
!     applicable provisional data.  Actually, this is the day after the end of
!     the provisional data  **OR**  it could be the forecast start date from
!     the previous run of the package.  Confusing?  Yep.  Cases in point...                                                         
!        1) provisional data has been added, but no forecast run.  Then
!           this date is the day after the provisional data.           
!        2) no new provisional data has been added since the last fore-
!           cast run, and the forecast start date has been adjusted to 
!           an earlier date than the previous run.  Then this date is  
!           the previous forecast start date.                          
!
      READ(U1, 1105, REC=8, ERR=812) iD, iM, iY
      CALL DateSequence(iD, iM, iY, Last); IF (ErrorLevel .NE. 0) GOTO 899
      iDsave = iD
      iMsave = iM

!
!     Get forecast start date 
!
      READ(U1, 1105, REC=5, ERR=812) iD, iM, iY
      CALL DateSequence(iD, iM, iY, Start); IF (ErrorLevel .NE. 0) GOTO 899
      IF (Start .GT. Last) THEN                
         WRITE(ErrorMessage, 9001)
         CALL PassMsg
         CLOSE(U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 898
      ENDIF

!
!     Get forecast end date.
!
      READ(U1, 1105, REC=6, ERR=812) LfD, LfM, LfY 
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

!
!     End sequence no. for forecast.
!
      CALL DateSequence(LfD, LfM, LfY, Last); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Input user-supplied historical forecast date.
!
      File3 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      U3 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE='DATEFILE.INT', STATUS='OLD', ERR=831)
      CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U3, 3101, ERR=832) jDsave
      READ(U3, 3101, ERR=832) jMsave
      READ(U3, 3101, ERR=832) jYsave
      CLOSE (U3)
      CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Compute the starting record number corresponding to the historical
!     forecast date.
!
      CALL DateSequence(jDsave, jMsave, jYsave, Forecast)
      IF (ErrorLevel .NE. 0) GOTO 899
      File4 = TRIM(ThisLakeDir) // BnS // 'MD.CLM'
      U4 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U4, FILE=TRIM(File4) , STATUS='OLD', RECL=8,               &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
      CALL FileWasOpened(U4); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U4, REC=1, ERR=842) I2, DataStart
      READ(U4, REC=2, ERR=842) DataEnd
      CLOSE (U4)
      CALL FileWasClosed(U4); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Check historical forecast dates are within the data base.
!
      I = Forecast + (Last - Start + 1) - 1
      IF (Forecast .LT. DataStart .OR. I .GT. DataEnd) THEN
         WRITE(ErrorMessage, 9010)
         CALL PassMsg
         WRITE(ErrorMessage, 9011) BnS
         CALL PassMsg
         GOTO 898
      END IF
      CALL SequenceDate(LfD, LfM, LfY, I); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Write a new parameter file for use in continuing simulation.
!
      File5 = TRIM(ThisLakeDir) // 'EVPPARAM.'//BnS
      File6 = TRIM(ThisLakeDir) // 'NEWPARAM.'//BnS

      U5 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U5, FILE=TRIM(File5), STATUS='OLD', ERR=851)
      CALL FileWasOpened(U5); IF (ErrorLevel .NE. 0) GOTO 899

      U6 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD', ERR=61)
      CLOSE (U6, STATUS='DELETE')
  61  OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='NEW', ERR=861)
      CALL FileWasOpened(U6); IF (ErrorLevel .NE. 0) GOTO 899

      READ(U5, 5161, ERR=852) Line
      Ext = Line(1:3)
      IF (Ext .NE. 'CLM') THEN
         WRITE(ErrorMessage, 9020) BnS
         CALL PassMsg
         WRITE(ErrorMessage, 9021)
         CALL PassMsg
         WRITE(ErrorMessage, 9022) Ext
         CALL PassMsg
         CLOSE (U5)
         CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
         CLOSE (U6, STATUS = 'DELETE')
         CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 899
      ENDIF

      WRITE(U6, 5161, ERR=863) TRIM(Line)
      DO Id = 1, 13
         READ (U5, 5161, ERR=852) Line
         WRITE(U6, 5161, ERR=863) TRIM(Line)
      END DO
      READ (U5, 5001, ERR=852)
      WRITE(U6, 5001, ERR=863) jdsave, jmsave, jysave
      READ (U5, 5001, ERR=852)
      WRITE(U6, 5001, ERR=863) Lfd, Lfm, Lfy
      DO Id = 1, 7
         READ (U5, 5161, ERR=852) Line
         WRITE(U6, 5161, ERR=863) TRIM(Line)
      END DO
      READ (U5, 5002, ERR=852) NoOfLayers
      WRITE(U6, 5002, ERR=863) NoOfLayers
      DO Id = 1, NoOfLayers * 3
         READ (U5, 5161, ERR=852) Line
         WRITE(U6, 5161, ERR=863) TRIM(Line)
      END DO
      CLOSE (U5, STATUS = 'DELETE')
      CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (U6)
      CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899

      CALL RenameFile (File6, File5, Flag)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     7 = program designator
!
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,               &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1102, REC=2, ERR=813) 7, Lf
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File6)
      CALL PassMsg; GOTO 898


  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : DO_EVAP (CPFFSROM)...'
      CALL PassMsg
      RETURN

!
!  FORMATs
!
 1101 FORMAT(A3, 8X)
 1102 FORMAT(3X, I3, 5X, A1)
 1105 FORMAT(2I3, I5)
 3101 FORMAT(I8)
 5001 FORMAT(2I3, I5)
 5002 FORMAT(I11)
 5161 FORMAT(A)

 9001 FORMAT('CPFFSROM: Invalid sequencing of programs.')
 9010 FORMAT('CPFFSROM: Requested Forecast Dates are outside of climatic')
 9011 FORMAT('          data period in ', A3, 'MD.CLM.')
 9020 FORMAT('CPFFSROM: Wrong version of EVPPARAM.', A3, ' present for')
 9021 FORMAT('          creation of provisional update parameter file!')
 9022 FORMAT('          Extension given is: ', A3, ', should be CLM.')


      END SUBROUTINE DO_EVAP

!-----------------------------------------------------------------------

      SUBROUTINE DO_RUN
      IMPLICIT NONE

      INTEGER (KIND=INTEGER2) :: I2, ScaleNum, iqSt(5)

      INTEGER :: NoSubbasins, I, iD, iM, iY, J, lfD, lfM, lfY
      INTEGER :: SRNs, iDsave, iMsave, jDsave, jMsave, jYsave
      INTEGER :: Forecast, Subbasin, U1, U2, U3, U4, U5, U6, U7
      INTEGER :: DataStart, DataEnd, FcstStart, FcstEnd, PrvEnd, SumStart
      INTEGER :: Std, Stm, Sty, Lpd, Lpm, Lpy

      REAL    :: qArea(iNSn), qParm(iNSn, 10), qSt(iNSn, 5, 0:0), SubArea

      CHARACTER (LEN=1), PARAMETER :: Lf = CHAR(10)
      CHARACTER (LEN=2)  :: SB
      CHARACTER (LEN=3)  :: BnS
      CHARACTER (LEN=4)  :: OverTheLake
      CHARACTER (LEN=8)  :: iStart, iEnd, jStart
      CHARACTER (LEN=10) :: Heading(iNsn)
      CHARACTER (LEN=13) :: Name
      CHARACTER (LEN=99) :: File1, File2, File3, File4, File5
      CHARACTER (LEN=99) :: File6, File7

!
!     Input basin name and other info.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1101, REC=1, ERR=812) Name
      READ(U1, 1102, REC=2, ERR=812) NoSubbasins, I, OverTheLake
      IF (I .LT. 6 .AND. I .NE. 0) THEN
         WRITE(ErrorMessage, 9001)
         CALL PassMsg
         WRITE(ErrorMessage, 9002) I
         CALL PassMsg
         CLOSE(U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 899
      END IF

!
!     Get provisional model update end date; now equal to day before  
!     start of forecast. (If forecast start has been set to a date    
!     earlier than the true end of the provisional data, this is still
!     valid.  We treat the day before the forecast start as if it were
!     the end of provisional data for purposes of this program, and the
!     SUM files are updated appropriately.)                            
!
      READ(U1, 1105, REC=5, ERR=812) iD, iM, iY
      CALL DateSequence(iD, iM, iY, FcstStart); IF (ErrorLevel .NE. 0) GOTO 899
      PrvEnd = FcstStart - 1

!
!     Check that xxx%%.SUM and xxxALL.SUM files have been updated
!     through the day before the forecast starts.
!
      READ(U1, 1105, REC=8, ERR=812) iD, iM, iY
      CALL DateSequence(iD, iM, iY, I); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1105, REC=9, ERR=812) iD, iM, iY
      CALL DateSequence(iD, iM, iY, J); IF (ErrorLevel .NE. 0) GOTO 899
      IF ((I .NE. J) .OR. (I .LT. FcstStart)) THEN 
         WRITE(ErrorMessage, 9001)
         CALL PassMsg
         CLOSE(U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 899
      ENDIF

!
!     Get forecast end date.
!
      READ(U1, 1105, REC=6, ERR=812) lfD, lfM, lfY
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      BnS = Name(1:3)
      File2 = TRIM(ThisLakeDir) // BnS // '01.SUM'
      U2 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U2, FILE=TRIM(File2), STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899

!
!     SumStart =  starting sequence number in xxx%%.SUM.
!
      READ(U2, REC=1, ERR=822) I2, SumStart        ! I2 = skip 2 bytes
      CLOSE (U2)
      CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899

!
!     SRNs = record number for initial conditions for forecast.  This
!     corresponds to the day before the forecast starts.
!
      SRNs = Hdr + (PrvEnd - SumStart) + 1

!
!     FcstStart = Start date for forecast.
!
      CALL SequenceDate(iD, iM, iY, FcstStart); IF (ErrorLevel .NE. 0) GOTO 899
      iDsave = iD
      iMsave = iM

!
!     FcstEnd = End sequence no. for forecast.
!
      CALL DateSequence(lfD, lfM, lfY, FcstEnd); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Input user-supplied forecast date.
!
      File3 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      U3 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', ERR=831)
      CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U3, 3101, ERR=832) jDsave
      READ(U3, 3101, ERR=832) jMsave
      READ(U3, 3101, ERR=832) jYsave
      CLOSE (U3)
      CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(jDsave, jMsave, jYsave, Forecast)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Compute the starting record number corresponding to the historical
!     forecast date.
!
      File4 = TRIM(ThisLakeDir) // BnS // '01.CLM'
      U4 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U4, FILE=TRIM(File4), STATUS='OLD', RECL=6,                  &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
      CALL FileWasOpened(U4); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U4, REC=1, ERR=842) I2, DataStart
      READ(U4, REC=2, ERR=842) DataEnd

!
!     Check historical forecast dates are within the data base.
!
      I = Forecast + (FcstEnd - FcstStart)
      IF ((Forecast .LT. DataStart)  .OR.  (I .GT. DataEnd)) THEN
         WRITE(ErrorMessage, 9010)
         CALL PassMsg
         WRITE(ErrorMessage, 9011)
         CALL PassMsg
         GOTO 899
      END IF
      CLOSE (U4)
      CALL FileWasClosed(U4); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Determine forecast dates.
!
      WRITE(iStart, 8505) jDsave, jMsave, jYsave
      I = Forecast + (FcstEnd - FcstStart)
      CALL SequenceDate(lfD, lfM, lfY, I); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(iEnd, 8505) lfD, lfM, lfY

!
!     (Date to begin updating of *.SUM files.)
!
      WRITE(jStart, 8505) iD, iM, iY

!
!     Read areas of subbasins.
!
      File5 = TRIM(ThisLakeDir) // 'AREA.' // BnS
      U5 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U5, FILE=TRIM(File5), STATUS='OLD', ERR=851)
      CALL FileWasOpened(U5); IF (ErrorLevel .NE. 0) GOTO 899
      DO Subbasin = 1, NoSubbasins
         READ(U5, 5101, ERR=852) qArea(Subbasin)
      END DO
      CLOSE (U5)
      CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Input subbasin parameters.  Read from file PARAMnn.xxx
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File6 = TRIM(ThisLakeDir) // 'PARAM' // SB // '.' // BnS
         U6 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD',                      &
     &        ACCESS='SEQUENTIAL', FORM='FORMATTED', ERR=861)
         CALL FileWasOpened(U6); IF (ErrorLevel .NE. 0) GOTO 899
         J = 0
         READ(U6, 1501, ERR=862) Heading(Subbasin)
         READ(U6, 1502, ERR=862) SubArea
         IF (SubArea .NE. qArea(Subbasin)) THEN
            WRITE(ErrorMessage, 9051) TRIM(File5), BnS
            CALL PassMsg
            GOTO 898
         END IF
         READ(U6, *, ERR=862)   ! skip date line
         READ(U6, *, ERR=862)   ! skip date line
         DO I = 1, 10
            READ(U6, 1502, ERR=862) qParm(Subbasin, I)
         END DO
         CLOSE (U6)
         CALL FileWasClosed(U5); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Input initial conditions.  These are the values in the xxxnn.SUM file
!     for the day prior to the start of the forecast.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File6 = TRIM(ThisLakeDir) // BnS // SB // '.SUM'
         U5 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD', RECL=12,                  &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
         CALL FileWasOpened(U6); IF (ErrorLevel .NE. 0) GOTO 899
         J = Hdr + (FcstStart - SumStart) + 1
         READ(U6, REC=1, ERR=862) I2, I, ScaleNum       ! throw away first 6 bytes...
         READ(U6, REC=J, ERR=862) I2, (iQst(I), I=1,5)  ! throw away first 2 bytes...
         DO I = 1, 5
           Qst(Subbasin, I, 0) = iQst(I) / 100.
         ENDDO
         Qst(Subbasin, 3, 0) = (iQst(3) / 100.) * ScaleNum
         CLOSE (U6)
         CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Write subbasin parameter files.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File7 = TRIM(ThisLakeDir) // 'PARAM' // SB // '.INT'
         U7 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U7, FILE = TRIM(File7), STATUS = 'OLD', ERR=70)
         CLOSE(U7, STATUS='DELETE')
  70     OPEN(UNIT=U7, FILE = TRIM(File7), STATUS = 'NEW', ERR=871)
         CALL FileWasOpened(U7); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(U7, 1501, ERR=873) Heading(Subbasin)
         WRITE(U7, 1502, ERR=873) qArea(Subbasin)
         CALL SequenceDate(Std, Stm, Sty, FcstStart)
         CALL SequenceDate(Lpd, Lpm, Lpy, FcstEnd)
         WRITE(U7, 1503, ERR=873) Std, Stm, Sty, Std, Stm, Sty
         WRITE(U7, 1503, ERR=873) Lpd, Lpm, Lpy
         DO I = 1, 10
            WRITE(U7, 1502, ERR=873) qParm(Subbasin, I)
         END DO
         CLOSE(U7)
         CALL FileWasClosed(U7); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Write initial condition files.  No other boundary conditions will be
!     written to the output files; just the initial conditions.
!
      DO Subbasin = 1, NoSubbasins
         WRITE(SB, 1305) Subbasin
         File7 = TRIM(ThisLakeDir) // 'BNDCND' // SB // '.INT'
         U7 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U7, FILE=TRIM(File7), STATUS='OLD', ERR=75)
         CLOSE(U7, STATUS='DELETE')
  75     OPEN(UNIT=U7, FILE=TRIM(File7), STATUS='NEW', ERR=871)
         CALL FileWasOpened(U7); IF (ErrorLevel .NE. 0) GOTO 899
         CALL SequenceDate(Std, Stm, Sty, FcstStart)
         WRITE(U7, 1602, ERR=873) Std, Stm, Sty, (Qst(Subbasin, I, 0), I=1,5)
         CLOSE(U7)
         CALL FileWasClosed(U7); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Update sequence file; 7 = program designator.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE='BSNNM', STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U1, 1102, REC=2, ERR=813) NoSubbasins, 7, OverTheLake, Lf
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'CPFFSROM: Error reading file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'CPFFSROM: Error opening file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'CPFFSROM: Error writing file ' // TRIM(File7)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : DO_RUN (CPFFSROM)...'
      CALL PassMsg
      RETURN

!
!  FORMATs
!
 1101 FORMAT(A11)
 1102 FORMAT(2I3, A4, 1X, A1)
 1105 FORMAT(2I3, I5)
 1305 FORMAT(I2.2)
 1501 FORMAT(A)
 1502 FORMAT(E13.6E2)
 1503 FORMAT(2I2, I4, 2I2, I4)
 1601 FORMAT(2I2, I4, 5E13.6E2)
 1602 FORMAT(2I2.2, I4, 5E13.6E2)
 3101 FORMAT (I8)
 5101 FORMAT(E13.6E2)
 6101 FORMAT(A10, /, 9(E13.6E2, /), E13.6E2)
 6102 FORMAT(I2, 6X, 5E13.6E2)
 7100 FORMAT(A10, A3, /, E13.6E2)
 7101 FORMAT(I8, /, A8, /, A8)
 7102 FORMAT(E13.6E2)
 7103 FORMAT(A8, 5E13.6E2)

 8501 FORMAT(A3, I2.2, '.SUM')
 8502 FORMAT('SUPARM', I2.2, '.INT')
 8505 FORMAT(I2.2, I2.2, I4)

 9001 FORMAT(' CPFFSROM: Invalid sequencing of programs.')
 9002 FORMAT('           Program Descriptor number is ', I0)
 9010 FORMAT(' CPFFSROM: Requested Forecast Dates are outside of')
 9011 FORMAT('           the climatic data period.')
 9051 FORMAT(' CPFFSROM: Subbasin area mismatch in ', A, ' and AREA.', A3)

      END SUBROUTINE DO_RUN


END MODULE CPFSR_Subs


!========================================================================

MODULE CPFSR_Main

CONTAINS

      SUBROUTINE DO_CPFFSROM(RunEvp, ErrStatus, HandleIn, Lake)

      USE CPFSR_Subs

      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: RunEvp, ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER  :: I
      CHARACTER (LEN=20) :: S

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

      IF (RunEvp .EQ. 0) THEN
         CALL DO_Evap()
      ELSE
   !      CALL DO_Run()
      END IF
      IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Error Handling
! 
 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_CPFFSROM...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

      END SUBROUTINE DO_CPFFSROM

END MODULE CPFSR_Main



