!----------------------------------------------------------------------------
!
!     program compute_storage_quantiles_for_plots_of_some_runoff_outlooks
!
!     Updated by TEC II 28sep88, 26oct88 (for PC)
!     Updated by TEC II 17jan90 to allow for variable (long) plotting period
!     Updated 27oct92, TSH, TEC II for 4-byte dates and move to UNIX
!     Updated by TEC II 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and to suppress writing of status info
!      if file: "QUIET.INT" exists in default directory.
!
!     This routine reads the file 'STOQUANT.xxx' and creates the file
!     'STOQUANT.INT', containing storage quantiles (20%, 50%, & 80%)
!     for every day of the plotting period.  xxx = three-
!     byte basin designator assigned by calling procedure.
!
!----------------------------------------------------------------------------

MODULE CSQFPOSR_Main

      USE FcstUtil

      INTEGER, PARAMETER :: BufLen = 400

CONTAINS
      SUBROUTINE DO_CSQF(ErrStatus, HandleIn, Lake)
      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER :: I, J, K, L, T, D, M, Y, iQ, iQq
      INTEGER :: Kkk, Lwe, Rwe, FcEnd, PStart, PieceNo, MidNo

      REAL :: qKkk, qSum

      REAL, DIMENSION(   366, 5, 9) :: Quantile
      REAL, DIMENSION(Buflen, 5, 9) :: Quant
      REAL, DIMENSION(        5, 3) :: qTemp

      CHARACTER (LEN=3)  :: BnS
      CHARACTER (LEN=10) :: LakeName
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File1, File2, File3

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Input basin name and other info.
!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,               &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1101, REC=1, ERR=812) LakeName
      BnS = Lake(1:3)

!
!     Get forecast end date.
!
      READ(11, 1105, REC=6, ERR=812) D, M, Y
      CALL DateSequence(D, M, Y, FcEnd); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get plot start date.
!
      READ(11, 1105, REC=7, ERR=812) D, M, Y
      CALL DateSequence(D, M, Y, PStart); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Set the length of the moving-average window for smoothing.
!
!        kkk = moving average window length, must be odd number
!
      Kkk = 9
      Lwe = -Kkk / 2     !  left window edge
      Rwe = -Lwe          !  right window edge

!
!     Check buffer is big enough.
!
      IF (Rwe - Lwe + 1 .GE. BufLen) THEN
         ErrorMessage = 'CSQFPOSR: Internal buffer length is too small.'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Input Quantile(day_of_the_year, storage_number, quantile_number),
!     where day_of_the_year = 1, ..., 366, storage_number = 1 (USZM + LSZM),
!     2 (GZM), 3 (SS), 4 (SNW), and 5 (TOTAL), and quantile_number = 1 (10%
!     non-exceedance), 2 (20% non-exceedance), ..., 9 (90% non-exceedance).
!
      File2 = TRIM(ThisLakeDir) // 'STOQUANT.' // BnS
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', RECL=4,               &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      I = 0
      DO Y = 1, 9
        DO M = 1, 5
          DO D = 1, 366
            I = I + 1
            READ(21, REC=I, ERR=822) Quantile(D, M, Y)
          ENDDO
        ENDDO
      ENDDO
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!      Write smoothed quantiles to plot file: STOQUANT.INT.
!
      File3 = TRIM(ThisLakeDir) // 'STOQUANT.INT'
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', RECL=60,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=31)
      CLOSE(31, STATUS='DELETE')
 31   OPEN(UNIT=31, FILE=TRIM(File3), STATUS='NEW', RECL=60,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899

      L = 0
      qKkk = Kkk

!
!     Process quantiles in time segments.
!
      PieceNo = 0
      DO WHILE (PStart .LE. FcEnd)
        PieceNo = PieceNo + 1
!        WRITE(ErrorMessage, 9009) PieceNo; CALL PassMsg
        MidNo = PStart + Lwe + BufLen - Rwe - 1
        IF (MidNo .GT. FcEnd) MidNo = FcEnd

!
!       Select quantiles of interest.
!
        DO I = PStart + Lwe, MidNo + Rwe
          CALL SequenceDate(D, M, Y, I);   IF (ErrorLevel .NE. 0) GOTO 899
          CALL DateSequence(01, 01, Y, J); IF (ErrorLevel .NE. 0) GOTO 899
          K = I - J + 1
          J = I - PStart - Lwe + 1
          iQq = 0
          DO iQ = 2, 8, 3
            iQq = iQq + 1
            DO T = 1, 5
              Quant(J, T, iQq) = Quantile(K, T, iQ)
            END DO
          END DO
        END DO

!
!       Smooth quantiles by using moving average.
!
        DO I = PStart, MidNo
          J = I - PStart - Lwe + 1
          DO T = 1, 5
            DO iQ = 1, 3
              qSum = 0.
              DO K = Lwe, Rwe
                qSum = qSum + Quant(J + K, T, iQ)
              END DO
              Qtemp(T, iQ) = qSum / qKkk
            END DO
          END DO
          L = L + 1
          WRITE(31, REC=L, ERR=833) ((qTemp(T, iQ), iQ=1,3), T=1,5)
        END DO
        PStart = MidNo + 1
      ENDDO

      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

!      WRITE(ErrorMessage, 9010) PieceNo; CALL PassMsg

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'CSQFPOSR: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'CSQFPOSR: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'CSQFPOSR: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'CSQFPOSR: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'CSQFPOSR: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'CSQFPOSR: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'CSQFPOSR: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'CSQFPOSR: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'CSQFPOSR: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898


  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_CSQF...'
      CALL PassMsg
      GOTO 999

  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

!
!  FORMATs
!
 1101 FORMAT(A)
 1105 FORMAT(2I3, I5)

 9009 FORMAT('~^~Processing piece number: ', I4)
 9010 FORMAT('~^~', I4, ' pieces processed.      ')

      END SUBROUTINE DO_CSQF

END MODULE CSQFPOSR_Main


