!
!     This program takes the first file named in file DATENAME.LIS and renames
!     it to DATEFILE.INT.  If DATENAME.LIS cannot be opened successfully,
!     cannot be read successfully, or has no filenames, an error code is
!     returned.
!
!     updated 23oct92, TEC II, for 4-byte dates and move to UNIX
!     updated 31mar94, TSH, added documentation only
!     updated 21mar95, TEC II, eliminated unnecessary check of same
!      provisional data end dates in BSNNM & EBSNNM files.
!

MODULE DateNameMain

      USE FcstUtil

CONTAINS
      SUBROUTINE DO_DATENAME(ErrStatus, HandleIn, Lake)
      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER   :: I, L, iM
      LOGICAL   :: F1, F2
      CHARACTER (LEN=4)  :: YearName
      CHARACTER (LEN=10) :: Line
      CHARACTER (LEN=12) :: NameS, Name2
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File1, File2, File3, File4

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!  Open the DATENAME.LIS file and read the first valid DAT?????.INT file name,
!  then delete DATENAME.LIS
!
      File1 = TRIM(ThisLakeDir) // 'DATENAME.LIS'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899

      L = 0
      DO WHILE ((L .EQ. 0)  .OR.  F1  .OR.  F2)
         READ(11, 1102, ERR=812) NameS
         L = LEN_TRIM(NameS)
         F1 = NameS(1:3) .NE. 'DAT'
         F2 = NameS(10:12) .NE. 'INT'
      END DO

      CLOSE (11, STATUS='DELETE')
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Open the DAT?????.INT file (old) and a DATEFILE.INT file (new)
!
      File2 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', ERR=20)
      CLOSE (21, STATUS='DELETE', ERR=821)
  20  OPEN(UNIT=21, FILE=TRIM(File2), STATUS='NEW', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899

      File3 = TRIM(ThisLakeDir) // NameS
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Copy the first 7 lines from DAT?????.INT to 
!  DATEFILE.INT then close DATEFILE.INT
!
      DO I = 1, 7
         READ(31, 1301, ERR=832) Line
         WRITE(21, 1205, ERR=823) TRIM(Line)
      ENDDO
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!  REWIND DAT?????.INT.  If it is a DAT1????.INT file then
!  copy it (first 7 lines only) to DAT2????.INT.  Then delete the input
!  DAT?????.INT file, whichever it was called.
!
      REWIND(31)
      IF (NameS(1:4) .EQ. 'DAT1') THEN
         Name2 = 'DAT2'//NameS(5:12)
         File4 = TRIM(ThisLakeDir) // Name2
         OPEN(UNIT=41, FILE=TRIM(File4), STATUS='OLD', ERR=25)
         CLOSE (41, STATUS='DELETE')
   25    OPEN(UNIT=41, FILE=TRIM(File4), STATUS='NEW', ERR=841)
         CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
         DO I = 1, 7
           READ(31, 1301, ERR=832) Line
           WRITE(41, 1205, ERR=843) TRIM(Line)
         END DO
         CLOSE (41)
         CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
      ELSE
        YearName = NameS(5:8)
        ErrorMessage = 'Processing year '//YearName;  CALL PassMsg
      END IF
      CLOSE (31, STATUS='DELETE')
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Open new version of DATENAME.LIS and write into it the 4-digit year 
!  from the DAT?????.INT file name.
!
      File1 = TRIM(ThisLakeDir) // 'DATENAME.LIS'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='NEW', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(11, 1104, ERR=813) NameS(5:8)
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!-----------------------
!
!     I/O error handling
!
 811  ErrorMessage = 'DATENAME: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'DATENAME: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 813  ErrorMessage = 'DATENAME: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'DATENAME: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 822  ErrorMessage = 'DATENAME: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 823  ErrorMessage = 'DATENAME: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'DATENAME: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 832  ErrorMessage = 'DATENAME: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 833  ErrorMessage = 'DATENAME: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 841  ErrorMessage = 'DATENAME: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
 842  ErrorMessage = 'DATENAME: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
 843  ErrorMessage = 'DATENAME: Error writing file ' // TRIM(File4)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_DateName...'
      CALL PassMsg
      GOTO 999

!----------------------------

 999  CALL CloseAllFiles()          ! in case any files are still open
      ErrStatus = ErrorLevel
      RETURN


!
!  FORMATs
!
 1102 FORMAT(A12)
 1104 FORMAT('FOR OUTLOOK YEAR ', A4, ':')
 1205 FORMAT(A)
 1301 FORMAT(A10)

      END SUBROUTINE DO_DATENAME

END MODULE DateNameMain


