!
!     This routine reads all eSTOTLyyyy.INT files and determines the average
!     daily value for every variable in eSTOTLyyyy.INT.  It places this
!     information into eSTOTLKS.INT and it places simulated results
!     preceding the forecast period into eSMULSTO.INT.
!
!     Adapted from LUMPOUT2.NI4, TEC II, Sep88, 26oct88 (for PC)
!     Updated by TEC II 17jan90 to allow for variable (long) plotting period
!        Note that there is no limit to simulation length.
!     Updated apr92, YMC to allow processing of "missing" data in evap. output
!        files (some special applications of the forecast package replace model
!        generated output with "historical" series which may be incomplete).
!     Updated 26oct92, TSH, TEC II for 4-byte dates and move to UNIX
!     Updated 01apr93, TEC II for expansion of ESTOTLyy.INT file.
!     Updated by TEC II 21mar95 for change in EBSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and to suppress writing of status info
!      if file: "QUIET.INT" exists in default directory.
!
MODULE ELMPOUT2_main

      USE FcstUtil
      INTEGER, PARAMETER :: ZHdr = 1            ! header records in ZS*.*
      INTEGER, PARAMETER :: BufLen = 500

CONTAINS

      SUBROUTINE DO_ELMP2(ErrStatus, HandleIn, Lake)
      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: I2
      INTEGER (KIND=INTEGER2), DIMENSION(BufLen) :: Cntr1, Cntr2, Cntr3
      INTEGER (KIND=INTEGER2), DIMENSION(BufLen) :: Cntr4, Cntr5, Cntr6

      INTEGER :: NumberOfScenarios, I, J, sD, sM, sY, FcEnd, FcStrt
      INTEGER :: ForecastPeriodLength, StartNo, Srn, K, iD, iM, iY
      INTEGER :: St, En, PieceNo, IOS
      REAL    :: qE, qTw, qTa, qVa, qW, qC, Q1, Q2, Q3, Q4, Q5, Q6
      REAL, DIMENSION(BufLen) :: q1Mean, q2Mean, q3Mean, q4Mean, q5Mean, q6Mean

      CHARACTER (LEN=1), PARAMETER :: Lf = CHAR(10)
      CHARACTER (LEN=4)  :: Yyyy
      CHARACTER (LEN=6)  :: Skip6
      CHARACTER (LEN=10) :: LakeName
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File1, File2, File3, File4, File5, File6, File7

      CHARACTER (LEN=4), DIMENSION(100) :: YearFile

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Determine application-specific information
!
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,     &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1101, REC=1, ERR=812) LakeName
      READ(11, 1102, REC=2, ERR=812) I
      IF (I .NE. 10) THEN
         WRITE(ErrorMessage, 9001) I
         CALL PassMsg
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 898
      END IF

      File2 = TRIM(ThisLakeDir) // 'YEARFILE.INT'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      NumberOfScenarios = 0
      READ(21, 2101, IOSTAT=IOS) I
      DO WHILE (IOS .EQ. 0)
         NumberOfScenarios = NumberOfScenarios + 1
         WRITE(Yyyy, 1002) I 
         YearFile(NumberOfScenarios) = Yyyy
         READ(21, 2101, IOSTAT=IOS) I
      END DO
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find out when forecast begins.  
!
      READ(11, 1105, REC=5, ERR=812) sD, sM, sY 
      CALL DateSequence(sD, sM, sY, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find out when model output files end (all are assumed
!     to end on the same date).
!
      READ(11, 1105, REC=6, ERR=812) sD, sM, sY  
      CALL DateSequence(sD, sM, sY, FcEnd); IF (ErrorLevel .NE. 0) GOTO 899
      ForecastPeriodLength = FcEnd - FcStrt + 1

!
!     Determine starting date for model statistics output.
!
      READ(11, 1105, REC=7, ERR=812) sD, sM, sY  
      CALL DateSequence(sD, sM, sY, StartNo); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Open Lake Evaporation model output file: ZSEVAPPR.xxx.
!
      File3 = TRIM(ThisLakeDir) // 'ZSEVAPPR.' // LakeName(1:3)
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', RECL=64,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Sequence number for first day in file.
!
      READ(31, REC=1, ERR=832) Srn

!
!     Record number for first day of output minus one.
!
      Srn = StartNo - Srn + 1 + zHdr - 1

!
!     Input and output evaporation model variables for simulation period.
!
      File4 = TRIM(ThisLakeDir) // 'ESMULSTO.INT'
      OPEN(UNIT=41, FILE=TRIM(File4), STATUS='OLD', ERR=41)
      CLOSE (41, STATUS = 'DELETE')
  41  OPEN(UNIT=41, FILE=TRIM(File4), STATUS='NEW', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899

      WRITE(41, 4101, ERR=843) ForecastPeriodLength
      WRITE(41, 4101, ERR=843) sD, sM, sY
      DO K = StartNo, FcStrt - 1
         CALL SequenceDate(iD, iM, iY, K); IF (ErrorLevel .NE. 0) GOTO 899
         I = Srn + K - StartNo + 1
         READ(31, REC=I, ERR=832) qE, qTw, qTa, qVa, qW, qC
         WRITE(41, 4101, ERR=843) iD, iM, iY, qE, qTw, qTa, qVa, qW, qC
      END DO

      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Determine average storages for the forecast period.
!
      File5 = TRIM(ThisLakeDir) // 'ESTOTLKS.INT'
      OPEN(UNIT=51, FILE=TRIM(File5), STATUS='OLD', ERR=51)
      CLOSE (51, STATUS = 'DELETE')
  51  OPEN(UNIT=51, FILE=TRIM(File5), STATUS='NEW', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899

      PieceNo = 0
      St = 1
      DO WHILE (St .LE. ForecastPeriodLength)
       PieceNo = PieceNo + 1
!       WRITE(ErrorMessage, 1009) PieceNo; CALL PassMsg
       En = St + BufLen - 1
       IF (En .GT. ForecastPeriodLength) En = ForecastPeriodLength

       q1Mean = 0.
       q2Mean = 0.
       q3Mean = 0.
       q4Mean = 0.
       q5Mean = 0.
       q6Mean = 0.

!
!      The following use of "counters" allows processing of "missing" data
!      which never occurs during normal use of the forecast package.  However,
!      under specialized uses, such as forecast evaluation simulations where
!      model components in the package are replaced with "historical"
!      estimates, missing data can be processed.
!
       Cntr1 = 0
       Cntr2 = 0
       Cntr3 = 0
       Cntr4 = 0
       Cntr5 = 0
       Cntr6 = 0

       DO I = 1, NumberOfScenarios

         Yyyy = YearFile(I)
         File6 = TRIM(ThisLakeDir) // 'ESTOTL' // Yyyy // '.INT'
         OPEN(UNIT=61, FILE=TRIM(File6), STATUS='OLD', RECL=70,           &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
         CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
         DO K = St, En
           READ(61, REC=K, ERR=862) Skip6, Q1, Q2, Q3, Q4, Q5, Q6
           J = K - St + 1
           IF (Q1 .GT. -99) THEN
             q1Mean(J) = q1Mean(J) + Q1
             Cntr1(J) = Cntr1(J) + 1
           END IF
           IF (Q2 .GT. -99) THEN
             q2Mean(J) = q2Mean(J) + Q2
             Cntr2(J) = Cntr2(J) + 1
           END IF
           IF (Q3 .GT. -99) THEN
             q3Mean(J) = q3Mean(J) + Q3
             Cntr3(J) = Cntr3(J) + 1
           END IF
           IF (Q4 .GT. -99) THEN
             q4Mean(J) = q4Mean(J) + Q4
             Cntr4(J) = Cntr4(J) + 1
           END IF
           IF (Q5 .GT. -99) THEN
             q5Mean(J) = q5Mean(J) + Q5
             Cntr5(J) = Cntr5(J) + 1
           END IF
           IF (Q6 .GT. -99) THEN
             q6Mean(J) = q6Mean(J) + Q6
             Cntr6(J) = Cntr6(J) + 1
           END IF
         END DO
         CLOSE (61)
         CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
       END DO

       DO J = 1, BufLen
         IF (Cntr1(J) .EQ. 0) THEN
           q1Mean(J) = -99.9
         ELSE
           q1Mean(J) = q1Mean(J) / Cntr1(J)
         END IF
         IF (Cntr2(J) .EQ. 0) THEN
           q2Mean(J) = -99.9
         ELSE
           q2Mean(J) = q2Mean(J) / Cntr2(J)
         END IF
         IF (Cntr3(J) .EQ. 0) THEN
           q3Mean(J) = -99.9
         ELSE
           q3Mean(J) = q3Mean(J) / Cntr3(J)
         END IF
         IF (Cntr4(J) .EQ. 0) THEN
           q4Mean(J) = -99.9
         ELSE
           q4Mean(J) = q4Mean(J) / Cntr4(J)
         END IF
         IF (Cntr5(J) .EQ. 0) THEN
           q5Mean(J) = -99.9
         ELSE
           q5Mean(J) = q5Mean(J) / Cntr5(J)
         END IF
         IF (Cntr6(J) .EQ. 0) THEN
           q6Mean(J) = -99.9
         ELSE
           q6Mean(J) = q6Mean(J) / Cntr6(J)
         END IF
       END DO

       DO K = St, En
         CALL SequenceDate(iD, iM, iY, FcStrt+K-1); IF (ErrorLevel .NE. 0) GOTO 899
         J = K - St + 1
         WRITE(51, 5101, ERR=853) iD, iM, iY, q1Mean(J), q2Mean(J),       &
     &                  q3Mean(J), q4Mean(J), q5Mean(J), q6Mean(J)
       END DO

       St = En + 1
      END DO    ! {DO WHILE (St .LE. ForecastPeriodLength)}

      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899

!
!      Update 'eBSNNM' file with new program designator.
!
      File7 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(11, 1102, REC=2, ERR=813) 0, Lf
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!      WRITE(ErrorMessage, 1010) PieceNo;  CALL PassMsg

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File5)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File6)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'ELMPOUT2: Error opening file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  872 ErrorMessage = 'ELMPOUT2: Error reading file ' // TRIM(File7)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'ELMPOUT2: Error writing file ' // TRIM(File7)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_ELMP2...'
      CALL PassMsg
      GOTO 999

!  -----------------------

  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN


!
!  FORMATs
!
 1002 FORMAT(I4)
 1009 FORMAT('~^~Processing piece number: ', I4)
 1010 FORMAT('~^~', I4, ' pieces processed.               ')
 1101 FORMAT(A10)
 1102 FORMAT(3X, I3, 5X, A1)
 1105 FORMAT(2I3, I5)
 2101 FORMAT(I4)
 4101 FORMAT(2I3, I5, 6F8.4)
 5101 FORMAT(2I3, I5, 6F8.4)
 9001 FORMAT('ELMPOUT2: Invalid sequencing of forecast programs. Pd=', I3)

      END SUBROUTINE DO_ELMP2

END MODULE ELMPOUT2_main



