! *****************************************************************************
! *****************************************************************************
!
!  Interval is an integer code for the time interval (timestep) to use:
!       1 = DAILY
!       2 = WEEKLY
!       3 = QUARTER-MONTHLY
!       4 = MONTHLY
!       5 = ANNUAL
!  We're using an integer code because it is easier to pass integers than 
!  character strings from Delphi to Fortran.

!
SUBROUTINE PHISTDAT (ErrStatus, WHandle, Interval, ReqVals, Lake)
   USE PROVHIST_Main, ONLY : DO_PROVHIST

   IMPLICIT    NONE
   INTEGER, INTENT(INOUT)  :: ErrStatus              ! 0=OK,  otherwise an error
   INTEGER, INTENT(INOUT)  :: WHandle
   INTEGER, INTENT(INOUT)  :: Interval
   INTEGER, INTENT(INOUT)  :: ReqVals                ! # of months to process
   CHARACTER (LEN=*), INTENT(IN) :: Lake             ! 3-character lake code (SUP,MIC,etc)

   CHARACTER (LEN=15) :: TStep, StringArray(2)

 !GCC$ ATTRIBUTES CDECL, DLLEXPORT ::  PHISTDAT

   IF (Interval .EQ. 1) TStep = 'DAILY'
   IF (Interval .EQ. 2) TStep = 'WEEKLY'
   IF (Interval .EQ. 3) TStep = 'QTRMONTHLY'
   IF (Interval .EQ. 4) TStep = 'MONTHLY'
   IF (Interval .EQ. 5) TStep = 'ANNUAL'

   IF (ReqVals .LE. 0) THEN
      IF (Interval .EQ. 1) ReqVals = 90
      IF (Interval .EQ. 2) ReqVals = 12
      IF (Interval .EQ. 3) ReqVals = 12
      IF (Interval .EQ. 4) ReqVals = 36
      IF (Interval .EQ. 5) ReqVals = 5
   END IF

   StringArray(1) = TStep
   StringArray(2) = Lake

   CALL DO_PROVHIST(ErrStatus, WHandle, ReqVals, StringArray)

END SUBROUTINE PHISTDAT

! *****************************************************************************
! *****************************************************************************

SUBROUTINE POUTLOOK (ErrStatus, WHandle, BigString)
   USE POutlook_Main, ONLY : DO_POutlook

   IMPLICIT    NONE
   INTEGER, INTENT(INOUT)  :: ErrStatus              ! 0=OK,  otherwise an error
   INTEGER, INTENT(INOUT)  :: WHandle
   CHARACTER (LEN=*), INTENT(IN) :: BigString

   INTEGER :: I, J, K, L
   CHARACTER (LEN=30), DIMENSION(4) :: StringArray

 !GCC$ ATTRIBUTES CDECL, DLLEXPORT ::  POUTLOOK

!
!  Parse BigString into 4 entries.  The entries are separated by 
!  the '|' character (ASCII 124)
!
   J = 1        ! index into BigString
   L = LEN_TRIM(BigString)
   DO I = 1, 4
      StringArray(I) = ''      ! initialize to empty string
      K = 1     ! index into StringArray(I)
      DO WHILE ((J .LE. L) .AND. (BigString(J:J) .NE. '|'))
         IF (K .LT. 30) StringArray(I)(K:K) = BigString(J:J)
         J = J + 1
         K = K + 1
      END DO
      J = J + 1      ! skip over the '|'
   END DO

   CALL DO_POutlook(ErrStatus, WHandle, StringArray)

END SUBROUTINE POUTLOOK

! *****************************************************************************
! *****************************************************************************



