!============================================================
!
!  This function calls the Windows API function SetWindowText,
!  which sends a WM_SETTEXT message to the window identified by the
!  specified window handle (hWnd).
!
!  Intended for use with AHPS, where the Fortran code calls
!  the MyMsgAPI() function, passing in a window handle and the
!  text to be passed to that window.
!
!  Current usage is that the Windows GUI form created with Delphi
!  has an invisible edit box. The handle of that edit box is passed 
!  to the Fortran code by the Delphi app. The Fortran code then
!  calls this MyMsgAPI() function, with that window hendle, to pass
!  back status and error messages to the GUI. The GUI app contains
!  an event handler that is triggered when it receives the WM_SETTEXT
!  message. It does some pre-processing parsing of the text string 
!  (app-specific behavior in order to decode what type of status or
!   error message it is), then does the appropriate thing with the text.
!
!  Took a lot of digging and experimentation to finally get this right.
!  The previous one worked with Lahey Fortran, but getting this to work
!  with gfortran was a pain. The problems were mainly in the 
!  typecasting, etc. Piecing together info from MANY user forum posts
!  and other internet sources, I finally figured it out. And now it looks
!  so simple. Typical for these kinds of issues. :P
!
!  Tim Hunter - 02 May 2013
!============================================================

MODULE MyWIN32
   USE ISO_C_BINDING
   IMPLICIT NONE
   PRIVATE
   PUBLIC SetWindowText
   INTERFACE
      FUNCTION SetWindowText(hWnd, lpString)      &
            BIND(C, NAME='SetWindowTextA')
         USE ISO_C_BINDING     ! gfortran
         IMPLICIT NONE
         !GCC$ ATTRIBUTES STDCALL :: SetWindowText
         LOGICAL(C_BOOL)        :: SetWindowText
         INTEGER(C_INT), value  :: hWnd
         CHARACTER(KIND=C_CHAR) :: lpString
      END FUNCTION SetWindowText
   END INTERFACE
END MODULE MyWIN32

!========================================
      SUBROUTINE MyMsg_API(Handle, TString)

      USE MyWIN32
      USE ISO_C_BINDING
      IMPLICIT NONE
      
      INTEGER             ::  Handle
      INTEGER(C_INT)      ::  hWnd
      LOGICAL             ::  F
      CHARACTER (LEN=*)   ::  TString
      CHARACTER(LEN=255)  ::  lpString

      hWnd = Handle
      lpString = TRIM(TString) // C_NULL_CHAR

      F = SetWindowText(hWnd, lpString)

      END SUBROUTINE MyMsg_API
