!---------------------------------------------------------------------------------
!      PROGRAM DIStributedAVerageMETeorology
!
!       Updated by TEC II 28sep88, 26oct88 (for PC).
!       Updated by TSH 14aug89 (modified handling of 1-station networks).
!       Updated by TEC II 17jan90 (for variable length), 28feb90 (added
!        informational message on computation of Thiessen weights for use of
!        same weights as last time), 02mar90.
!       Updated by TSH 08jun90 (added handling of Ural & Emba rivers).
!       Updated by TEC II 21jun90 (corrected for non-unity cell size), 15oct90 (amended for
!        arbitrary inclusion of subbasins), 23apr91 (added handling of Volga
!        River).
!       Updated by TSH 07sep90 (corrected informational message, on
!        computation of Thiessen weights & added history log of Thiessen
!        weights computation, as per changes made to DISAVMET.SRC by TEC II on
!        09aug90).
!       Updated by TEC II 24sep91 (corrected assessment of presence of correct
!        Thiessen tables), 16jul92 (updated for UNIX & for 4-byte dates).
!       Updated by TSH 11may93 (to conform to F77 standard for handling output
!        of arithmetic results to unformatted file (specifically required
!        by the USACE "Clipper" Fortran compiler).
!       Updated by TEC II 22mar94 (changed to save number of stations in
!        network in history log).
!       Updated by TSH 29mar94 (changed references to unit 5 to unit 8).
!       Updated by TEC II, 21mar95 to suppress writing of status info if
!         file: "QUIET.INT" exists in default directory; to make IDString a
!         function call, after TSH 18jan94 in PROVSNAL; for change in eBSNNM
!         structure (addition of forecast start date in record 5 and
!         displacement of later records by one to accomodate the addition).
!       Updated by TSH, April 1999, to handle both evap and runoff model
!         inputs.  Also restructured to be included as a DLL in the Windows
!         version of the AHPS Forecast Package.
!       Updated by TSH, May 1999.  Significant changes -- see long description below.
!       Updated by TSH in April, 2014 to account for new provisional database
!         structure. Structure is actually quite similar, but with the following
!         changes.
!          1) Combined the PRVSNAL and EPRVSNAL files into a single file
!          2) A longer station ID field (40 chars instead of 10)
!          3) I moved the MaxStationsInOneDay field into record 1
!
!       -----  Significant changes made by TSH, May 1999. -----
!       We have restructured the EPRVSNAL.xxx/PROVSNAL.xxx files (see 
!       documentation in PRVSNAL.F90) and also have changed our approach regarding 
!       2+ stations at the same location.  We now allow multiple stations to 
!       co-exist at the same location (within 1/100 degree lat/long).  This 
!       program will "see" that the 2+ stations are in the same place, which 
!       would cause problems for the thiessen-weighting if we used them individually.  
!       Instead, we will combine their values by averaging and use the averaged 
!       value as if it were a single station at that location.
!       
!       Another significant change I made at this time is the elimination of the
!       combined Huron/Georgian Bay basin.  The use of "NHU" has been
!       something of a thorn in the side, so we are now switching our
!       designator for the Lake Huron basin (w/o Georgian Bay) to HUR.
!       This is done with some fear and trepidation, since HUR is the
!       same designator that we had been using for NHU/GEO combined
!       way back in the past before they were modeled separately.  We
!       now judge the benefits to outweigh the risks, though.  Enough
!       people have mistakenly interpreted "NHU" to mean "NORTHERN
!       Huron" (instead of "NEW Huron") to make us worried as this
!       goes to a wider audience (hopefully).  A lot of ancillary programs
!       will have to be changed, too, but it's time to "bite the bullet".
!       I also changed the BasinDesignator numbers to reflect the revised
!       ordering.  This number is used in the headers of the CLM/PRV files
!       and maybe other places (I don't know yet - I imagine I will find
!       out as I modify the rest of the programs.)  The new values are:
!
!          SUP = 1   MIC = 2   HUR = 3   GEO = 4   STC = 5   ERI = 6
!          ONT = 7   CHA = 8   URA = 9   EMB = 10  VOL = 11
!
!
!       This routine updates the provisional subbasin/overlake data files from 
!       the provisional data files: EPRVSNAL.xxx/PROVSNAL.xxx.  It computes 
!       Thiessen-weighted areal averages for the meteorological variables if 
!       they are flagged as new data in the provisional data files and removes 
!       the flags.  Otherwise old data values, computed previously, are left 
!       unchanged in the provisional overlake or subbasin data files.  
!       The format of the input data files are:
!
!       This program reads the file: EBSNNM/BSNNM and updates it; all forecast
!       programs do the same with the exception of optional modules.
!       This file is not to be disturbed by the user as it exists only for
!       the purpose of each program keeping track of what other programs have
!       been run as well as to reference specific basin information.
!       It is a formatted direct access file with 12-byte records (the last
!       byte of each record is a line feed) and its contents are as follows:
!
!       -------------------
!       (EBSNNM)
!
!       1st record:   (FORMAT: A10, 1X)
!
!         Name  -  basin name, left adjusted, in all upper case.
!
!       2nd record:   (FORMAT: 3X, I3, 5X)
!
!         Program designator number.
!
!       The remaining records are filled in during a forecast session and are
!       rewritten by the programs only.  They have no meaning after the
!       session is over.  Each record is a date (day, month, year) in the
!       format: 2I3, I5.
!
!       3rd through 8th records:   (FORMAT: 2I3, I5)
!
!         3rd record:  base date (see PRVSNAL.f90)
!         4th record:  provisional data end date
!         5th record:  forecast start date
!         6th record:  forecast end date
!         7th record:  plot start date
!         8th record:  date of earliest new data since XEVAPOUT.f90 last run
!       -------------------
!
!       (BSNNM)
!
!       1st record:   (FORMAT: A10, 1X)
!
!         Name  -  basin name, left adjusted, in all upper case.
!
!       2nd record:   (FORMAT: 2I3, A4, 1X)
!
!         # # switch
!         | | |    |
!         | | |____|____lake precipitation computation switch:
!         | |           "BASN" => estimate over-the-lake precipitation with
!         | |                     land depths
!         | |           "LAKE" => estimate over-the-lake precipitation
!         | |                     directly from nearby gages (subbasin "0"
!         | |                     file)
!         | |_________program designator number; each program has its own and
!         |           places it in BSNNM so programs can check for proper
!         |           sequence
!         |_________number of subbasins in basin
!
!       The remaining records are filled in during a forecast session and are
!       rewritten by the programs only.  They have no meaning after the
!       session is over.  Each record is a date (day, month, year) in the
!       format: 2I3, I5.
!
!       3rd through 9th records:   (FORMAT: 2I3, I5)
!
!         3rd record:  base date (see PROVSNAL.f and REPACK.f)
!         4th record:  provisional data end date
!         5th record:  forecast start date
!         6th record:  forecast end date
!         7th record:  plot start date
!         8th record:  date of earliest new data since WATOUT.f last run
!         9th record:  date of earliest new data since LUMPSTOR.f last run
!        10th record:  date of earliest new data since CMOFPOSR.f last run
!
!  -------------------- begin ProvMetData.xxx ----------------------------
!
!  ProvMetData.xxx is composed of "blocks", one for each day of record, arranged in 
!  a linked sequential order with every day represented by one block.  
!  Every day between the beginning and end dates must be represented even if 
!  there are no data for that day.  Each block must have the individual station 
!  data arranged in an order that is unique for that group of stations; 
!  i. e., every time the same set of stations reoccurs, they must be listed 
!  in exactly the same order. In order to accomplish this, the entries are
!  sorted by Latitude, then Longitude; both in ascending order.
!
!  The file is a direct access file with a fixed record length of 62 bytes; 
!  therefore, it must not be edited normally since an editor will create 
!  sequential access files which are unusable in DISAVMET.  Besides that, 
!  all records are unformatted, so it would be pretty difficult to do 
!  anything meaningful within a text editor.  Each block has the following 
!  structure:
!
!  First record of the file:
!
!    date linkNH PP MaxCount unused link (unformatted: 4, 4, 4, 4, 42, 4)
!    |  | |    | || |      | |    | |  |
!    |  | |    | || |      | |    | |__|__Pointer to next logical record (header or data)
!    |  | |    | || |      | |____|_______This field is filled with nulls. It is used simply
!    |  | |    | || |      |              to fill all the records to 62 bytes.
!    |  | |    | || |______|_______This is a special field in record 1. It contains the
!    |  | |    | ||                maximum number of stations reporting on any day.
!    |  | |    | ||_____Special interpretation for record 1...
!    |  | |    |        The absolute value of this field is a pointer to the last 
!    |  | |    |        physical data record. The positive/negative state is still 
!    |  | |    |        interpreted as in the other block headers.
!    |  | |____|_____pointer to header record of next logical data block
!    |  |            In the final block, this value is 0, indicating no more data blocks
!    |__|_________date sequence number
!
!  First record of each "block":
!
!    date linkNH PP unused link (unformatted: 4, 4, 4, 46, 4)
!    |  | |    | || |    | |  |
!    |  | |    | || |    | |__|__Pointer to next logical record (header or data)
!    |  | |    | || |____|_______This field is filled with nulls. It is used simply
!    |  | |    | ||              to fill all the records to 62 bytes.
!    |  | |    | ||_____This field is just used as an indicator of whether or not 
!    |  | |    |        the block has been processed by DISAVMET.  
!    |  | |    |          Negative = the block has been processed by DISAVMET
!    |  | |    |          Positive = new data has been put into the block since the 
!    |  | |    |                     last time DISAVMET was run and DISAVMET needs to 
!    |  | |    |                     process/reprocess this block.  The actual value 
!    |  | |    |                     is irrelevant.
!    |  | |____|_____pointer to header record of next logical data block
!    |  |            In the final block, this value is 0, indicating no more data blocks
!    |__|_________date sequence number
!
!   Other records in the block  (i.e. the data records):
!
!    Lt Ln Tmx Tmn Prc Air Dew Wnd Cld ID link (unformatted: 2,2,2,2,2,2,2,2,2,40,4)
!    || || | | | | | | | | | | | | | | || |  |
!    || || | | | | | | | | | | | | | | || |__|__pointer to next logical record. In the
!    || || | | | | | | | | | | | | | | ||       final record, this is set to zero.
!    || || | | | | | | | | | | | | | | ||_______station ID (40 characters, left-justified)
!    || || | | | | | | | | | | | | |_|_____cloudcover in percent (0-100)
!    || || | | | | | | | | | | |_|_________windspeed in hundredths of meters / second
!    || || | | | | | | | | |_|_____________daily dewpoint air temperature in
!    || || | | | | | | | |                 hundredths of degrees Centigrade
!    || || | | | | | | |_|_________________daily dry-bulb temperature in 
!    || || | | | | | |                     hundredths of degrees Centigrade
!    || || | | | | |_|_____________________daily total precipitation in 
!    || || | | | |                         hundredths of millimeters
!    || || | | |_|_________________________daily minimum air temperature in 
!    || || | |                             hundredths of degrees Centigrade
!    || || |_|_____________________________daily maximum air temperature in 
!    || ||                                 hundredths of degrees Centigrade
!    || ||_________________________________longitude in hundredths of decimal degrees
!    ||____________________________________latitude in hundredths of decimal degrees
!
!
!    Note that the very first record of the file is special.  It is a block header, 
!    as would be expected, but the third field indicates the record number of the 
!    last data record in the file and the 4th field indicates the maximum number of 
!    stations reporting in a single day.
!
!    The last block header record has some special values as well.  Field
!    number 2 (pointer to the next block header) is set to zero.
!    Field number 5 (pointer to the next logical record) in the last data
!    record (which could possibly be the last block header record as well)
!    is also set to zero.  Both of these zero "flag values" are used to 
!    indicate the end of logical records.
!
!  ------------------ end ProvMetData.xxx format --------------------------
!
!
!    These are the output files from this program.  The xxxnn.PRV files contain
!    the daily thiessen-weighted values for each subbasin or overlake area.
!
!               xxxnn.PRV
!               | |||
!               | |||____two character subbasin ID.  For the evap model inputs
!               | |      (in EPRVSNAL.xxx) this ID is "MD".  For the runoff
!               | |      model inputs (in PROVSNAL.xxx) this is the 2-digit
!               | |      subbasin number "00", "01", etc.
!               | |
!               |_|______three-character Great Lakes Basin identifier:
!                             SUP  (Superior)
!                             MIC  (Michigan)
!                             HUR  (Huron only)
!                             GEO  (Georgian Bay)
!                             STC  (St. Clair)
!                             ERI  (Erie)
!                             ONT  (Ontario)
!                             CHA  (Champlain)
!                             URA  (Ural River)
!                             EMB  (Embe River)
!                             VOL  (Volga River)
!
!       Each provisional overlake data file is an unformatted, direct-access
!       file with a record length of: (xxxMD.PRV = 8 bytes)/(xxxnn.PRV = 6 bytes).
!       The first record contains the basin numerical identifier (e. g. 1 corresponds 
!       to basin 1, Superior) and the starting date sequence number in that order as 
!       a 2-byte integer and as a 4-byte integer, respectively.  The second record 
!       contains the end date sequence number as a leading 4-byte number.  The
!       format of these files is documented below.
!
!
!  ------------- begin xxx??.PRV format ---------------------
!
!     The data files, xxx??.PRV are direct access, unformatted (binary) files.
!     The record length for xxxMD.PRV is 8 bytes. The record length for 
!     xxxnn.PRV is 6 bytes. When less than that number of bytes is specified
!     in the format description below then the rest of the record is ignored.
!     e.g. Record 1 uses 6 bytes in both cases.  In the xxxMD.PRV file the
!          last 2 bytes of record 1 are ignored.
!
!
!     1st record:     Id StartSeqNum   (2, 4)
!                     || |         |
!                     || |_________|__ 4-byte sequence number corresponding
!                     ||               to date of first data record.
!                     ||____________ Lake identifier number 
!                                    (1=SUP, 2=MIC, 3=HUR, 4=GEO, 5=STC, 6=ERI, 
!                                     7=ONT, 8=CHA, 9=URA, 10=EMB, 11=VOL).
!
!     2nd record:     EndSeqNum   (4)
!                     |       |
!                     |_______|_____ 4-byte sequence number corresponding to
!                                    the date of the last record.
!
!     Format of remaining records depends on which file it is:
!
!     xxxnn.PRV format:
!     records 3-n:    TMin TMax Prec   (2, 2, 2)
!                     |  | |  | |  |
!                     |  | |  | |__|___daily precipitation volume (mm/100)
!                     |  | |__|________daily maximum air temperature (deg. C/100)
!                     |__|_____________daily minimum air temperature (deg. C/100)
!
!     xxxMD.PRV format:
!     records 3-n:    AirTemp DewPt Wind Cloud   (2, 2, 2, 2)
!                     |     | |   | |  | |   |
!                     |     | |   | |  | |___|__ Total cloud cover percentage
!                     |     | |   | |  |         (15 = 15.00%)
!                     |     | |   | |__|________ Wind speed in meters per sec
!                     |     | |   |              times 100 (1500 = 15.00 m/s)
!                     |     | |___|_____________ Dew point temperature in deg C
!                     |     |                    times 100 (1500 = 15.00 deg C)
!                     |_____|___________________ Air temperature in degrees C
!                                                times 100 (1500 = 15.00 deg C)
!
!  ------------- end xxx??.PRV format ---------------------
!
!       This program also notes the earliest "new" provisional data processed
!       and writes the corresponding date to file: EBSNNM/BSNNM, for use in
!       subsequent processing by other programs.
!
!       'MapWidth' is the number of the right column of the map (0 is the
!       number of the left column of the map).  'MapHeight' is the number of
!       the top row of the map (0 is the number of the bottom row of the map).
!
!
!
!  ---------------------------------------------------------------------------
!   Structure of the HSTRYLOG.xxx file:
!
!     The history log file is an unformatted, direct access file read 
!     and written to sequentially.  Each record has the following structure:
!
!         ddddccnnii      (unformatted, stored as 4, 2, 2, 2 bytes)
!         |  |||||||
!         |  |||||||_________________network index (network number)
!         |  |||||_________________Number of stations in named network
!         |  |||_________________Hundreds place is code:
!         |  |                   0: network added
!         |  |                   1: network deleted
!         |  |                   2: 1-station "network" added & dropped
!         |  |                   3: network recalled
!         |  |                   4: network repeated
!         |  |                   Ones place is DataNo
!         |__|_________________Date
!
!
!  ---------------------------------------------------------------------------
!      Explanation of how Hashing is implemented for storage of old weights:
!                (THSN_DAT.xxx, THSN_SYM.xxx, THSN_HSH.xxx)
!      Written by Tim Hunter 26 October 2001.
!
!      Ihave just determined that we've been violating the Fortran standard in
!      our implementation of this process in that we were assigning values 
!      greater than 127 to CHARACTER variables.  While I think it may have
!      been working, the standard does not allow this.  In fact, I know that
!      we got bit by the rule as it relates to ICHAR() and I had to change the
!      routine IDString a few months back to get it to work correctly.
!
!      Today I've been testing this program and it seems to be misbehaving.
!      In the process of trying to determine what's wrong I had to dig into
!      the storage/retrieval system for Thiessen weights.  It took me quite
!      some time to decipher how this was implemented, so I'll take a little 
!      bit more time and document it so we don't have to dig so much next time.
!      I'll also document here the changes I had to make in order to bring the
!      program into conformance with the standard.
!
!      THSN_DAT.xxx is a direct-access, unformatted file where each record
!      contains a bunch (MaxNoSta * (NoSubbasins+1)) of 4-byte REAL values
!      representing the weights for each station on each subbasin (including the
!      0 subbasin - the lake).  There is no information in this file on which
!      particular stations are there -- keeping track of that information is
!      the job of the other 2 THSN_???.xxx files.
!
!      THSN_SYM.xxx is a direct-access, unformatted file that contains the
!      network identifiers corresponding to the records in THSN_DAT.xxx.
!      By that I mean that record 1 of THSN_SYM.xxx has the network ID and
!      THSN_DAT.xxx has the weights for that network.  Record 2 of each file
!      matches up. Record 3 matches up. etc.  So these 2 files have the same
!      number of records, though the record lengths are not the same.  Each 
!      record in THSN_SYM.xxx is essentially a string of the Lat/Long values
!      of each station (4 bytes per station). See the routine IDString to see
!      how the 4-byte ID is constructed for each station from its lat/long.
!
!      THSN_HSH.xxx is a sequential-access, unformatted file that contains a
!      basic description of the THSN_???.xxx files (ID lengths, hash size,
!      max number of networks, etc) that is used in determining if the existing
!      THSN_???.xxx files can be used with the current program setup.  It then
!      has the hash table information (more on this shortly) and the last-used
!      information as well as the number of stations in each stored network.
!
!      IMPORTANT NOTE:  It is assumed for purposes of finding 'already-computed'
!                       weights that the stations have been sorted by the program
!                       that wrote the PROVSNAL.xxx or EPRVSNAL.xxx file.  If that
!                       was not done then we may unnecessarily compute weights
!                       when the correct set was already sitting in the file/array.
!                       Results will be correct, but processing time will be increased.
!
!      So, how does this hashing/storing/etc stuff work?
!
!         1) Construct a "Network ID".
!                The lat and long for each station, as read from PROVSNAL.xxx
!                (or EPRVSNAL.xxx) is a 2-byte integer in degrees times 100.
!                That is, if the latitude is 82.71 degrees then we will read it
!                as 8271.  We concatenate the 2 bytes of latitude and 2 bytes
!                of longitude together (bitwise) to create a 4-byte integer.
!                We then store this 4 byte integer into the SymbolArray, which
!                is just an array of MaxNoSta 4-byte integers.
!         2) Are the weights for this network already stored in THSN_DAT.xxx?
!            If so, load them.
!                To determine this we compare the network ID to the set of 
!                stored network IDs.  See process of searching symbol table
!                below.
!         3) If the weights were not just loaded then compute them.
!         4) Save the new weights.
!                If the weights were already saved (i.e. just loaded) then all
!                we have to update is the LastUsed variable.  Otherwise we do
!                a full save.  See the process for saving weights below.
!       
!
!         Process for searching symbol table and loading weights:
!
!           1) Compute a hash code (i.e. bin number) for the network.  
!                See the HashNum() routine for how this is done.
!           2) Are there any entries stored in that "bin"?
!                No  = We're done -- weights not there.
!                Yes = Continue with step 3.
!           3) Use the bin number as a record number and read that record
!              from THSN_SYM to get a network ID.
!           4) Is this network ID a match to the one we constructed?
!                Yes = Use the corresponding entry in the weights file/array.  Done!
!                No  = Use the entry in ChainTable(RecNum) as the new record number
!                      and continue.
!           5) Is the new record number zero?
!                Yes = We're done -- weights not there.
!                No  = Get RecNum entry from symbol file/table (another old ID)
!           6) Go to Step 4.
!
!
!
!        Structure of the Symbol Table (THSN_SYM.xxx) and Theissen weights (THSN_DAT.xxx):
!
!           Each array is dimensioned (1:MaxNoNetworks) and there is an exact 1-to-1
!           correspondence between them.  Record 1 of the THSN_SYM file contains the
!           network ID corresponding to the network for which the weights are stored
!           in record 1 of the THSN_DAT file.  Record 2 of each file has the paired-up
!           entries.  Record 3 of each file are a pair, etc.
!
!
!        Structure of the Theissen weights (THSN_DAT.xxx):
!
!           This is an array of theiessen weights for each network.
!
!
!        Structure of the Hash Table (THSN_HSH.xxx):
!
!           Actually, the file THSN_HSH.xxx contains information about the saved
!           networks, then it has the Hash Table, then the Chain Table, then the
!           LastUsed Table, then the Network Symbol Size Table.
!
!           Rec 1: Length of network ID values
!           Rec 2: Hash Size (i.e. number of hash bins)
!           Rec 3: Max number of networks that can be stored
!           Rec 4: Number of subbasins in this lake
!           Rec 5: Max number of stations allowed in a network
!
!           Rec 6-(HashSize+5) : The Hash Table
!                   This is just an array (1:HashSize) of pointers (record numbers) 
!                   into the THSN_SYM.xxx file.  A value of zero indicates that the 
!                   bin is empty.
!                   Entry 1 : record number in THSN_SYM.xxx for first network in bin 1.
!                   Entry 2 : record number in THSN_SYM.xxx for first network in bin 2.
!                   Entry 3 : record number in THSN_SYM.xxx for first network in bin 3.
!                   Entry 4 : record number in THSN_SYM.xxx for first network in bin 4.
!                           .... etc ....
!           Next MaxNoNetworks entries : Chain Table
!                   This is an array of record numbers (array indices) in the Symbol Table.
!                   If you start where the Hash Table tells you to for a particular bin then
!                   this also forms a chain of entries ending with zero.
!
!           Next MaxNoNetworks entries : LastUse Table
!                   Each entry is a sequence number for the latest date the corresponding
!                   network was used.
!
!           Next MaxNoNetworks entries : Network Size Table
!                   Each entry is the number of stations in the corresponding network.
!                   
!
!        Simple example of these structures:
!           Assume the following:
!              HashSize      =  3 (3 bins) 
!              MaxNoNetworks = 10
!              MaxNoStations =  5
!
!           For simplicity here, we'll refer to the stations by letter and construct the
!           network ID as a string.  In reality it is an array of integers.
!           We'll also assign weights from 0-10 for simplicity.
!           Also for simplicity, we'll assume that the hashing function returns a value
!           of 1 for any network ID starting with A, 2 for anything starting with B, and
!           3 for any other network ID.
!
!           We could have the following info:
!              HashTable      ChainTable    NetwkSymSz     Symbol      Weights
!                  1              7             3           ABC          4 4 2
!                  3              0             4           ABCD         3 3 2 2 
!                  4              0             2           BC           7 3
!                                 6             3           CDE          1 2 7
!                                 0             5           CDFGH        2 2 1 1 4
!                                 5             1           D            10
!                                 2             4           ABDE         1 1 7 1
!                                 0             0           ---          -
!                                 0             0           ---          -
!                                 0             0           ---          -
!
!          There are 3 networks starting with A (Hash=1).  They are in records 1,2,7
!          of THSN_SYM and THSN_DAT.  Only 1 network starts with B (Hash=2) and it is
!          in record 3.  3 networks start with something else (Hash=3) and they are 
!          in records 4,5,6.
!
!          Suppose our new network was ABDE...  The hash function returns 1.  
!          HashTable(1) is 1, so we look at Symbol(1) [ABC] and compare to our 
!          new ID [ABDE].  They do not match, so we check ChainTable(1) [7].  
!          It is non-zero so we use it as the new index.  Symbol(7) [ABDE] matches
!          our new ID, so we will use Weights(7) [1 1 7 1] as our weights.
!
!          Suppose our new network was BDE...  The hash function returns 2.  
!          HashTable(2) is 3, so we look at Symbol(3) [BC] and compare to our 
!          new ID [BDE].  They don't match, so we check ChainTable(3). It is
!          zero, so we are done and have failed to find a set of stored weights
!          for this network.
!
!

MODULE DISAVSUBS

      USE FCSTUTIL                

      IMPLICIT NONE

      INTEGER, PARAMETER :: MaxNoNetworks = 75
      INTEGER, PARAMETER :: HashSize = 25
      INTEGER, PARAMETER :: ThsnDatFileSize = 8000000    ! max file size in bytes

      LOGICAL, PARAMETER :: DoDebug = .TRUE.

      INTEGER :: NoSubbasins, MapWidth, MapHeight, CellSize
      INTEGER :: MaxNoSta                ! set in SetParams() based on ThsnDatFileSize
      INTEGER :: BridgeCode              ! map code for "bridges" to islands


      !
      !  This variable holds the highest subbasin number used.  When RunEvp=0 (evap)
      !  it will be set to 0 (using subbasins 0 to 0).  When RunEvp=1 (runoff) it
      !  will be set to the number of subbasins for that lake (0..n)
      !
      INTEGER :: SubbasinsUsed

      REAL    :: LakeArea, XOffset, YOffset, Radius, RM
      CHARACTER (LEN=10)  :: LakeName

      !
      !  These variables are used for the thiessen file and hashing control and reporting
      !
      INTEGER (KIND=INTEGER2) :: NoNetworks
      INTEGER                 :: ThsnDatRecl
      CHARACTER (LEN=15)      :: ThsnDat, ThsnSym, ThsnHsh, LogFile

      !
      !  OldWeights and NewWeights are used to keep track of how many times 
      !  did we use already-computed weights and newly-computed weights.
      !  LogRec is the record number in the HSTRYLOG file.
      !
      INTEGER  :: OldWeights, NewWeights, LogRec

      !
      !  TmpSymbol is used in the ___Symbol() routines as a temporary
      !  symbol or network ID.  Allocated and freed in the main routine.
      !
      INTEGER, DIMENSION(:), ALLOCATABLE :: TmpSymbol

      INTEGER, DIMENSION(:), ALLOCATABLE   :: LastUse                     ! (1:MaxNoNetworks)

!
!  Global variables from old COMMON /HshTbls/
!      COMMON /HshTbls/ HashTable, ChainTable, NtWrkSymSz
!
!  NtWrkSymSz is no longer used.
!
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE :: HashTable   ! (1:HashSize)
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE :: ChainTable  ! (1:MaxNoNetworks)
!      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE :: NtwrkSymSz  ! (1:MaxNoNetworks)

!
!  Global variables from old COMMON /Weights/
!      COMMON /Weights/ qThiessenWeights, qCount                  15oct90
!
      REAL (KIND=REAL4), DIMENSION(:), ALLOCATABLE :: qCount              ! (0:NoSubbasins+2)
      REAL (KIND=REAL4), DIMENSION(:,:), ALLOCATABLE :: qThiessenWeights  ! (1:MaxNoSta,0:NoSubbasins+1)

!
!  Global variables from old COMMON /Digital/
!      COMMON /Digital/ zMap
!
      CHARACTER (LEN=1), DIMENSION(:,:), ALLOCATABLE  :: zMap           ! (0:MapWidth, 0:MapHeight)

!
!  Global variables from old COMMON /StaLoc/
!      COMMON /StaLoc/ Qx, Qy, SrtX, SrtY, SrtN, StaLocN
!
      REAL (REAL4), DIMENSION(:), ALLOCATABLE :: Qx, Qy, SrtX, SrtY     ! (1:MaxNoSta)
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE  :: SrtN       ! (1:MaxNoSta)
      INTEGER  :: StaLocN

!
!  Global variables from old COMMON /Polygon/
!      COMMON /Polygon/ PStaNo, PNoSta, Yl, Yu, Lrow, Rrow
!
      INTEGER :: PStaNo, PNoSta, Ylower, Yupper
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE :: Lrow, Rrow  ! (0:MapHeight)


!
!  Unit numbers for file operations. These were originally hardcoded as unit 11, 16, etc,
!  but I wanted them to be more flexible and "trackable" using my utility functions like
!  FileWasOpened(), FileWasClosed(), etc.  To avoid passing unit numbers right now (it
!  is probably the RIGHT way to do it, but I'm trying to get this to work quickly right
!  now), I am making these global.
!
      INTEGER  :: U11, U16, U21, U26, U31, U36, U41, U46, U51, U56
      
CONTAINS

!-------------------------------------------------------------
        INTEGER FUNCTION SetParams(Bsn)
        IMPLICIT   NONE
        CHARACTER (LEN=3), INTENT(IN)   :: Bsn
        CHARACTER (LEN=3)   :: LkCode
        INTEGER             :: I

        INTEGER ::  NoSubsArr(11)
        INTEGER ::  MapWideArr(11), MapHighArr(11), CellSizeArr(11)
        REAL    ::  XOffsetArr(11), YOffsetArr(11), RadiusArr(11), RMArr(11)
        CHARACTER (LEN=10) :: LakeNameArr(11)

!
!                          SUP  MIC  NHU  GEO  STC  ERI  ONT  CHA
!                          URA  EMB  VOL
!
        DATA NoSubsArr   /  22,  27,  16,  13,   7,  21,  15,   7,    &
     &                       1,   1,   1 /
        DATA MapWideArr  / 759, 471, 410, 551, 239, 583, 455, 183,    &
     &                     391, 420, 554 /
        DATA MapHighArr  / 515, 607, 435, 425, 183, 423, 391, 311,    &
     &                     433, 340, 631 /
        DATA CellSizeArr /   1,   1,   1,   1,   1,   1,   1,   1,    &
     &                       2,   1,   3 /

        DATA LakeNameArr  / 'SUPERIOR  ', 'MICHIGAN  ', 'HURON     ',      &
     &                      'GEORGIAN  ', 'STCLAIR   ', 'ERIE      ',      &
     &                      'ONTARIO   ', 'CHAMPLAIN ', 'URAL      ',      &
     &                      'EMBE      ', 'VOLGA     '/

        DATA XOffsetArr   /   406.039150,   227.973530,   279.796362,      &
     &                        279.796362,    64.526407,   298.661220,      &
     &                        262.877366,    89.026790,   204.165222,      &
     &                        214.582169,   274.266012 /

        DATA YOffsetArr   / -5121.245120, -4586.637700, -4709.846822,      &
     &                      -4859.846822, -4648.658407, -4473.268070,      &
     &                      -4632.988223, -4784.424320, -2450.377930,      &
     &                      -4896.804688, -1606.893229 /

        DATA RMArr        /       -87.85,       -86.87,       -81.79,      &
     &                            -81.79,       -82.71,       -81.74,      &
     &                            -76.90,       -73.34,        54.99,      &
     &                             55.85,        46.46 /
!
!       Map calibrations yielded rmse = 1.63861 km (HURON), 1.03296 km (STCLAIR),
!       and 0.456165 km (ONTARIO).
!
        DATA RadiusArr    /       6365.2,       6363.7,       6346.2,      &
     &                            6346.2,       6328.6,       6338.3,      &
     &                            6341.8,       6339.6,       3014.5,      &
     &                            6034.0,       2061.6 /


        I = -99
        LkCode = Bsn
        CALL Caps(LkCode)
        IF (LkCode .EQ. 'SUP') I = 1
        IF (LkCode .EQ. 'MIC') I = 2
        IF (LkCode .EQ. 'HUR') I = 3
        IF (LkCode .EQ. 'GEO') I = 4
        IF (LkCode .EQ. 'STC') I = 5
        IF (LkCode .EQ. 'ERI') I = 6
        IF (LkCode .EQ. 'ONT') I = 7
        IF (LkCode .EQ. 'CHA') I = 8
        IF (LkCode .EQ. 'URA') I = 9
        IF (LkCode .EQ. 'EMB') I = 10
        IF (LkCode .EQ. 'VOL') I = 11

        IF (I .LT. 1) THEN
           SetParams = -1
           ErrorMessage = 'DISAVMET: Bad lake code in SetParams = '//LkCode
           CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
           ErrorLevel = 1
           RETURN
        END IF

        NoSubbasins = NoSubsArr(I)
        MapWidth    = MapWideArr(I)
        MapHeight   = MapHighArr(I)
        CellSize    = CellSizeArr(I)
        XOffset     = XOffsetArr(I)
        YOffset     = YOffsetArr(I)
        Radius      = RadiusArr(I)
        RM          = RMArr(I)
        LakeName    = LakeNameArr(I)

        BridgeCode  = NoSubbasins + 1

        I           = (NoSubbasins+1) * MaxNoNetworks
        MaxNoSta    = INT(ThsnDatFileSize / I / 4)         ! truncated integer value

        SetParams = 0
        RETURN

        END FUNCTION SetParams


!----------------------------------------------------------------------------
      SUBROUTINE MainRoutine(Bsn)

      IMPLICIT NONE

      CHARACTER (LEN=3)  :: Bsn
      INTEGER            :: RunEvp

      INTEGER :: WriteFlagInterval = 15   ! Day/data iterations between writes

      INTEGER (KIND=1)  :: Int1, Int1Arr(4)
      CHARACTER (LEN=1) :: Char1

      INTEGER (KIND=INTEGER2) :: Q, Qwmet(0:30,1:7)
      INTEGER (KIND=INTEGER2) :: BasinDesignator, I2, J2, K2, StaNo
      INTEGER (KIND=INTEGER2) :: I2Lat, I2Long, PrvLat, PrvLong
      INTEGER (KIND=INTEGER2) :: TotalNoNetworks, Ibn, DataNo, NoSta
      INTEGER (KIND=INTEGER2) :: Dat(7), DatCount(7)
      INTEGER (KIND=INTEGER2) :: Zero2, zOne2
      INTEGER (KIND=INTEGER2) :: z100, z200, z300, z400, TempI2Var
      INTEGER (KIND=INTEGER2) :: MetForOutput(7)

      INTEGER (KIND=INTEGER2), DIMENSION(:,:), ALLOCATABLE :: Qqmet, TDat ! (1:MaxNoSta, 1:7)
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE   :: QMet        ! (1:MaxNoSta)
      INTEGER (KIND=INTEGER2), DIMENSION(:), ALLOCATABLE   :: TLat, TLon  ! (1:MaxNoSta)

      INTEGER (KIND=INTEGER4) :: Zero, zOne, B2, C2, R2, L2, Rcp
      INTEGER (KIND=INTEGER4) :: I, J, K, SeqNum, NoRec, Rcs, NoDays
      INTEGER (KIND=INTEGER4) :: Date, SDate, EDate, NumStaLoc, TSub, TSta
      INTEGER (KIND=INTEGER4) :: DateOfData, EarliestLastUse, NumStaToday
      INTEGER (KIND=INTEGER4) :: Day, Month, Year
      INTEGER (KIND=INTEGER4) :: FirstDay, FirstMonth, FirstYear
      INTEGER (KIND=INTEGER4) :: LastDay, LastMonth, LastYear, Starts
      INTEGER (KIND=INTEGER4) :: DaysDone, DaysToDo, PctDone

      INTEGER  :: FSubRecl, DataTypesToProcess, IOS, LLS, MaxNoStaFromPRV
      INTEGER  :: Subbasin, NumberOfRecords, ScrRec, WriteFlagNum
      INTEGER  :: SymbolSize, NetworkNo, NetworkToDelete, NetworkInMemory

      INTEGER, DIMENSION(:), ALLOCATABLE   :: NetworkID, OldNetworkID     ! (1:MaxNoSta)
      INTEGER, DIMENSION(:), ALLOCATABLE   :: xS, yS, xS_Tmp              ! (1:MaxNoSta)

      REAL (KIND=REAL4) :: Qq, Lt, Ln, DatAccum(7)

      REAL (KIND=REAL4), DIMENSION(:), ALLOCATABLE :: Qqx, Qqy           ! (1:MaxNoSta)

      LOGICAL           :: Saved, FirstNewData, Found, Always
      LOGICAL           :: WriteToBsnFile, IsOpen

      CHARACTER (LEN=1)   :: Lf
      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=3)   :: BnS, Str3
      CHARACTER (LEN=4)   :: OTLCode, Nulls4
      CHARACTER (LEN=5)   :: Str5
      CHARACTER (LEN=6)   :: Str6
      CHARACTER (LEN=15)  :: FSubPrv, BsnFile, FileNamePRV
      CHARACTER (LEN=40)  :: Skip40, Null40
      CHARACTER (LEN=42)  :: Skip42, Null42
      CHARACTER (LEN=46)  :: Skip46, Null46
      CHARACTER (LEN=80)  :: MessageString
      CHARACTER (LEN=100) :: EMsg

      CHARACTER (LEN=1), DIMENSION(:), ALLOCATABLE :: ZMapRow            ! (0:MapWidth)


!
!  initialization (some replacing DATA statements)
!
      Lf = CHAR(10)

      Zero  = 0
      Zero2 = 0
      zOne  = 1
      zOne2 = 1
      z100  = 100
      z200  = 200
      z300  = 300
      z400  = 400
      Nulls4 = CHAR(0)//CHAR(0)//CHAR(0)//CHAR(0)

      OldWeights = 0
      NewWeights = 0

      DO I = 1, MaxOpenedFiles
         OpenedFileUnits(I) = -1
      END DO
      OpenedFiles = 0

      IF (RunEvp .EQ. 0) SubbasinsUsed = 0            ! (0..0)
      IF (RunEvp .EQ. 1) SubbasinsUsed = NoSubbasins  ! (0..NoSubbbasins)

      DO I = 1, 46
         Null46(I:I) = CHAR(0)
      END DO
      Null42 = Null46(1:42)
      Null40 = Null46(1:40)

      WriteToBsnFile = .TRUE.

!
!     Does the user want to cancel already?
!
      IF (UserWantsToCancel()) GOTO 999

      IF (DoDebug) THEN
         OPEN(UNIT=1001, FILE='DISAV_DBG.TXT', STATUS='REPLACE', IOSTAT=IOS)
         IF (IOS .NE. 0) THEN
            ErrorMessage = 'DISAVMET: Error creating debug output file.'
            CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
            GOTO 898
         END IF
         CALL FileWasOpened(1001)
      END IF

!
!  Set local variables BnS and BasinDesignator based on dummy variable Bsn
!
      BnS = Bsn
      IF (BnS .EQ. 'SUP') BasinDesignator =  1
      IF (BnS .EQ. 'MIC') BasinDesignator =  2
      IF (BnS .EQ. 'HUR') BasinDesignator =  3
      IF (BnS .EQ. 'GEO') BasinDesignator =  4
      IF (BnS .EQ. 'STC') BasinDesignator =  5
      IF (BnS .EQ. 'ERI') BasinDesignator =  6
      IF (BnS .EQ. 'ONT') BasinDesignator =  7
      IF (BnS .EQ. 'CHA') BasinDesignator =  8
      IF (BnS .EQ. 'URA') BasinDesignator =  9
      IF (BnS .EQ. 'EMB') BasinDesignator = 10
      IF (BnS .EQ. 'VOL') BasinDesignator = 11

!
!     Set file names
!
      BsnFile = 'BSNNM'
      FileNamePRV = 'ProvMetData.'//BnS
      FSubPRV = BnS//'01.PRV'

!
!     Get the maximum number of stations reporting for a single day.  This number
!     applies to the entire file, so the portion we are processing may not have
!     nearly as many stations, but this is the upper limit.
!
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=FileNamePRV, STATUS='OLD', RECL=62,               &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(U11)
      READ(U11, REC=1, ERR=812) SeqNum, B2, R2, MaxNoStaFromPRV, Skip42, L2
      CLOSE(U11)
      CALL FileWasClosed(U11)
      IF (MaxNoStaFromPRV .GT. MaxNoSta) THEN
         ErrorMessage = 'Number of stations reporting > MaxNoSta.  MaxNoSta adjusted.'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         WRITE(ErrorMessage, 9268) MaxNoSta, MaxNoStaFromPRV
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
 9268    FORMAT('MaxNoSta: OldValue=', I0, '   NewValue=',I0)
         MaxNoSta = MaxNoStaFromPRV
      END IF

!
!  Determine how many bytes of RAM we are going to try to allocate.  This
!  allows us to produce a more informative error message in case of problems.
!
      I = 0
      I = I + MaxNoSta * 4                              ! TmpSymbol array
      I = I + HashSize * 2                              ! HashTable array
      I = I + MaxNoNetworks * 2                         ! ChainTable array
      I = I + (MaxNoSta * (NoSubbasins+2)) * 4          ! qThiessenWeights
      I = I + (NoSubbasins+3) * 4                       ! qCount
      I = I + MaxNoNetworks * 4                         ! LastUse
      I = I + ((MapWidth+1) * (MapHeight+1)) * 1        ! zMap
      I = I + MaxNoSta * 4 * 4                          ! Qx, Qy, SrtX, SrtY
      I = I + MaxNoSta * 2                              ! SrtN
      I = I + (MapHeight+1) * 2 * 2                     ! LRow, RRow

!
!  Allocate the GLOBAL dynamic arrays
!
      ALLOCATE(TmpSymbol(1:MaxNoSta),                                     &
     &         HashTable(1:HashSize),                                     &
     &         ChainTable(1:MaxNoNetworks),                               &
     &         qThiessenWeights(1:MaxNoSta,0:NoSubbasins+1),              &
     &         qCount(0:NoSubbasins+2),                                   &
     &         LastUse(1:MaxNoNetworks),                                  &
     &         zMap(0:MapWidth, 0:MapHeight),                             &
     &         Qx(1:MaxNoSta),                                            &
     &         Qy(1:MaxNoSta),                                            &
     &         SrtX(1:MaxNoSta),                                          &
     &         SrtY(1:MaxNoSta),                                          &
     &         SrtN(1:MaxNoSta),                                          &
     &         Lrow(0:MapHeight),                                         &
     &         Rrow(0:MapHeight),                                         &
     &         STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'DISAVMET: Error allocating RAM for global dynamic arrays.'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         WRITE(ErrorMessage, 9020) I
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF

!
!  Determine how many bytes of RAM we are going to try to allocate.  This
!  allows us to produce a more informative error message in case of problems.
!
      J = 0
      J = J + MaxNoSta * 2                          ! QMet
      J = J + MaxNoSta * 7 * 2                      ! Qqmet
      J = J + (MaxNoSta * 4) * 4                    ! Qqx, Qqy, xS, yS
      J = J + MaxNoSta * 4                          ! xS_Tmp
      J = J + (MaxNoSta * 2) * 2                    ! TLat, TLon
      J = J + MaxNoSta * 7 * 2                      ! TDat
      J = J + MaxNoSta * 4                          ! NetworkID
      J = J + MaxNoSta * 4                          ! OldNetworkID
      J = J + MapWidth+1                            ! zMapRow

!
!  Allocate the local dynamic arrays
!
      ALLOCATE(QMet(1:MaxNoSta),                                          &
     &         Qqmet(1:MaxNoSta, 7),                                      &
     &         Qqx(1:MaxNoSta),                                           &
     &         Qqy(1:MaxNoSta),                                           &
     &         xS(1:MaxNoSta),                                            &
     &         yS(1:MaxNoSta),                                            &
     &         xS_Tmp(1:MaxNoSta),                                        &
     &         TLat(1:MaxNoSta),                                          &
     &         TLon(1:MaxNoSta),                                          &
     &         TDat(1:MaxNoSta, 7),                                       &
     &         NetworkID(1:MaxNoSta),                                     &
     &         OldNetworkID(1:MaxNoSta),                                  &
     &         zMapRow(0:MapWidth),                                       &
     &         STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'DISAVMET: Error allocating RAM for local dynamic arrays.'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         K = I + J
         WRITE(ErrorMessage, 9021) J, K
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF

!
!     Initializate the Thiessen weights array (just allocated) to zeros.  If you
!     don't do this you end up with miscellaneous "garbage" in the THSN_DAT.xxx
!     file.  It doesn't hurt anything, but if the user happens to load the file
!     into a text editor (as I did) it can be a bit disconcerting to see perfectly
!     valid ASCII text (e.g. data from a station file) in the supposedly binary
!     direct access file.  By initializing the allocated RAM, you eliminate this.
!
      qThiessenWeights(:,:) = 0.0
 
!
!     Initialize a bunch of other arrays just for safety's sake.  When I turn on
!     full checking/debug I get a number of errors appearing (seemingly random in
!     that it's different every time) complaining of "undefined value" for various
!     variables.  Because of the "randomness" of the events I can't really track
!     down what is happening.  I strongly suspect, though, that it is related to
!     the allocated variables never being initialized.  In the previous version
!     of this code they were all statically declared (no ALLOCATE was available in
!     F77) and the compiler would initialize them all. (Yes, this is a compiler-
!     dependant behavior, but it's what our compiler did.)  Now that they are not
!     automatically initialized I have to do it manually if I want it done.
!      Tim Hunter - January 8, 2001
!
      HashTable(:)  = 0
      ChainTable(:) = 0
      qCount(:)     = 0.0
      zMap(:,:)     = CHAR(0)
      Qx(:)         = 0.0
      Qy(:)         = 0.0
      SrtX(:)       = 0.0
      SrtY(:)       = 0.0
      SrtN(:)       = 0
      Lrow(:)       = 0
      Rrow(:)       = 0
      QMet(:)       = 0
      Qqmet(:,:)    = 0
      Qqx(:)        = 0
      Qqy(:)        = 0
      xS(:)         = 0
      yS(:)         = 0
      xS_Tmp(:)     = 0
      LastUse(:)    = 0
      TLat(:)       = 0
      TLon(:)       = 0
      TDat(:,:)     = 0

!
!     Set variables which control THSN_???.xxx and HSTRYLOG.xxx file usage.
!
      ThsnDat = 'THSN_DAT.'//BnS
      ThsnHsh = 'THSN_HSH.'//BnS
      ThsnSym = 'THSN_SYM.'//BnS
      LogFile = 'HSTRYLOG.' // BnS
      ThsnDatRecl = MaxNoSta * (NoSubbasins+2) * 4       ! MaxNoSta * (NoSubbasins+2) * 4  

      IF (UserWantsToCancel()) GOTO 999

!
!     Get starting date from ProvMetData.xxx and
!     check number of stations.
!
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=FileNamePRV, STATUS='OLD', RECL=62,               &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(U11)

!
!     Inspect database to see if all are already processed data; exit normally
!     if no new data processing is required.
!
      Rcp = zOne
      Found = .FALSE.
      DO WHILE (Rcp .NE. Zero)
        READ(U11, REC=Rcp, ERR=812) SeqNum, B2, R2, Skip46, L2
        Found = Found .OR. (R2 .GT. Zero)
        Rcp = B2
      END DO

!
!     No new provisional data to process, so just deallocate the RAM and return
!
      IF (.NOT. Found) THEN
        CLOSE(U11)
        CALL FileWasClosed(U11)
        GOTO 875          ! deallocate the RAM and exit successfully
      END IF

      READ(U11, REC=zOne, ERR=812) SeqNum, B2, R2, NoSta, Skip42, L2    ! SeqNum = start of ProvMetData.xxx

      IF (NoSta .GT. MaxNoSta) THEN
        ErrorMessage = 'DISAVMET: Number of stations exceeds program limits!'
        CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
        CLOSE (U11)
        CALL FileWasClosed(U11)
        GOTO 898
      END IF
      Rcp = zOne

!
!     Input cell count in map.
!
      U16 = GetFreeUnitNumber()
      OPEN(UNIT=U16, FILE='AREA.'//BnS, STATUS='OLD', ERR=816)
      CALL FileWasOpened(U16)
      READ(U16, 2201, ERR=817) (qCount(I), I=1, NoSubbasins)
      READ(U16, 2201, ERR=817) qCount(0)
      CLOSE(U16)
      CALL FileWasClosed(U16)
      DO I = 0, NoSubbasins
         qCount(I) = qCount(I) / 1000000. / CellSize**2
      END DO

!
!     Initialize the number of networks computed thus far in this routine.
!     Set "Saved" TRUE to avoid saving uninitialized weights.
!
      TotalNoNetworks = 0
      Saved = .TRUE.

!
!     Input the code map for the subbasins within the basin.
!     Read one row at a time into zMapRow, then assign the row
!     into the global zMap array.
!
      J = MapWidth+1
      U21 = GetFreeUnitNumber()
      OPEN(UNIT=U21, FILE=BnS//'BYTCD.MAP', STATUS='OLD', RECL=J,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(U21)
      DO J = 0, MapHeight
        READ(U21, REC=J+1, ERR=822) zMapRow
        DO I = 0, MapWidth
           zMap(I,J) = zMapRow(I)
        END DO
      END DO
      CLOSE (U21)
      CALL FileWasClosed(U21)

      IF (UserWantsToCancel()) GOTO 999

!
!     Get number of records and starting date from provisional
!     data file xxx01.PRV 
!
      U26 = GetFreeUnitNumber()
      OPEN(UNIT=U26, FILE=TRIM(FSubPRV), STATUS='OLD', RECL=6,      &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=826)
      CALL FileWasOpened(U26)
      READ(U26, REC=1, ERR=827) Ibn, K                  ! K = start seq num
      READ(U26, REC=2, ERR=827) NoRec                   ! NoRec = end seq num
      CLOSE(U26)
      CALL FileWasClosed(U26)

      NoRec = NoRec - K + 1                     ! NoRec = # of records in xxx??.PRV file
      Always = (NoRec .EQ. 0) .OR. (K .NE. SeqNum)
      IF (Always) THEN
        K = SeqNum
        NoRec = 0
      END IF
      NoRec = NoRec + 2
      Rcs = 3 + SeqNum - K                ! rec num for first data to add in FSubPRV file
      FirstNewData = .FALSE.
      NoDays = SeqNum - K
      NumberOfRecords = 0

!
!     OPEN intermediate provisional subbasin (or overlake) data file
!     Unnamed because it is just a scratch file for internal use.
!     It will be set up as 14-byte records to allow storage of all
!     seven types of met data.
!
!     Structure of the file is just a bunch of data blocks.  Each block is:
!
!     record 1 :  RecNum *    (unformatted: 4, 10)
!                 |    | |
!                 |    | |__ unused 10 bytes
!                 |____|____ record number in FSubPRV file that this block corresponds to.
!                            i.e. this block data is for some particular date, and that
!                            date corresponds to a record number in the FSubPRV file(s).
!
!     record 2-?: D1 D2 D3 D4 D5 D6 D7   (unformatted: 2,2,2,2,2,2,2)
!                 ||________________||_ data values
!
!                 There are NoSubbasins+2 of these records in each block - one 
!                 record for each subbasin (0..n+1).  The evap model inputs are
!                 treated as subbasin number NoSubbasins+1.
!
!
      OPEN(UNIT=777, STATUS='SCRATCH', RECL=14, ACCESS='DIRECT',           &
     &     FORM='UNFORMATTED', ERR=881)
      CALL FileWasOpened(777)
      ScrRec = 0

!
!     Set file names and record lengths
!
!
!     Open the intermediate Thiessen weight table file.
!
      U31 = GetFreeUnitNumber()
      Found = .FALSE.
      OPEN(UNIT=U31, FILE=ThsnDat, STATUS='OLD', RECL=ThsnDatRecl,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', IOSTAT = IOS)
      IF (IOS .EQ. 0) THEN
         CALL FileWasOpened(U31)
         Found = .TRUE.
      END IF

!
!     Old weights available, check for hash tables...
!
      IF (Found) THEN
         U36 = GetFreeUnitNumber()
         OPEN(UNIT=U36, FILE=ThsnHsh, STATUS='OLD',                        &
     &        ACCESS='SEQUENTIAL', FORM='UNFORMATTED', IOSTAT=IOS)
         IF (IOS .EQ. 0) THEN
            CALL FileWasOpened(U36)
         ELSE
            Found = .FALSE.
            CLOSE(U31, STATUS='DELETE')
            CALL FileWasClosed(U31)
         END IF
      END IF

!
!     Old weights and hash tables available, check for symbol table...
!
      IF (Found) THEN
         U41 = GetFreeUnitNumber()
         OPEN(UNIT=U41, FILE=ThsnSym, STATUS='OLD', RECL=MaxNoSta*4,          &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', IOSTAT = IOS)
         IF (IOS .EQ. 0) THEN
            CALL FileWasOpened(U41)
         ELSE
            Found = .FALSE.
            CLOSE(U31, STATUS='DELETE')
            CALL FileWasClosed(U31)
            CLOSE(U36, STATUS='DELETE')
            CALL FileWasClosed(U36)
         END IF
      END IF

!
!     Old weights and hash tables exist, check dimensions...
!
!     If at any time we get an error while trying to read the file, skip
!     the rest of the attempt.  This prevents trying to compare an
!     undefined value in the IF (A .NE. B) section of the test.
!
      IF (Found) THEN
        READ(U36,IOSTAT=IOS) I                 ! max num stations (used for network ID - THSN_SYM)
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) J                 ! hash size (i.e. number of bins - ?)
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) K                 ! max number of networks that can be stored
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) TSub              ! actually the number of subbasins
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) TSta              ! max number of stations allowed
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) Ibn
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101
        READ(U36,IOSTAT=IOS) NoNetworks
        Found = Found .AND. (IOS .EQ. 0)
        IF (.NOT. Found) GOTO 101

        IF (I    .NE. MaxNoSta)        Found = .FALSE.
        IF (J    .NE. HashSize)        Found = .FALSE.
        IF (K    .NE. MaxNoNetworks)   Found = .FALSE.
        IF (TSub .NE. NoSubbasins)     Found = .FALSE.
        IF (TSta .NE. MaxNoSta)        Found = .FALSE.
        IF (Ibn  .NE. BasinDesignator) Found = .FALSE.
      END IF
 101  CONTINUE

      IF (UserWantsToCancel()) GOTO 999

!
!     Either initialize or read hash tables...
!
      IF (Found) THEN
        DO I = 1, HashSize
          READ(U36, ERR=837) HashTable(I)
        END DO
        DO I = 1, MaxNoNetworks
          READ(U36, ERR=837) ChainTable(I)
        END DO
        DO I = 1, MaxNoNetworks
          READ(U36, ERR=837) LastUse(I)
        END DO
        CLOSE (U36)
        CALL FileWasClosed(U36)
        NetworkInMemory = 0
      ELSE
        CALL ReinitThiessen()
        IF (ErrorLevel .EQ. 50) GOTO 898
      END IF

!                                                                    
!     Open file for saving Thiessen-weight-computation history log.  
!                           
      U46 = GetFreeUnitNumber()                                         
      OPEN(UNIT = U46, FILE = LogFile, STATUS = 'REPLACE',                 &
     &     ACCESS='DIRECT', RECL=10, FORM='UNFORMATTED', ERR=846)
      CALL FileWasOpened(U46)
      LogRec = 0

      IF (UserWantsToCancel()) GOTO 999

!
!     Write a blank line to the interface.  This will then be overwritten 
!     by the status writes.
!
      ErrorMessage = '  '
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      WriteFlagNum = WriteFlagInterval   ! force a write on the first iteration

!
!     How many days of data will we be reading from ProvMetData.xxx?
!
!     J=0   => at end-of-file
!
      J = Rcp
      DaysToDo = 0
      DO WHILE (J .NE. Zero)
        DaysToDo = DaysToDo + 1
        READ(U11, REC=J, ERR=812) Date, B2, R2, Skip46, L2
        J = B2
      END DO        

!
!     Read a block of data from ProvMetData.xxx.
!
!     Rcp = 0 => at end-of-file
!
      DaysDone = 0
      DO WHILE (Rcp .NE. Zero)
        READ(U11, REC=Rcp, ERR=812) Date, B2, R2, Skip46, L2
        NoDays = NoDays + 1

        DaysDone = DaysDone + 1
        WRITE(Str3, '(I3)') (DaysDone*100) / DaysToDo
        ErrorMessage = '~%~'//Str3
        CALL WriteErrorMessage(Str3, '~%~', -1, .TRUE.)

        IF ((R2 .LT. Zero) .AND. (.NOT. Always)) THEN
!
!         Flag = (old); skip block.
!
          Rcp = B2

!
!         Preserve old values in provisional subbasin data files by skipping
!         record.  (They must be there from a previous update!).
!         Rcs = record number in the *.PRV file where we will be adding this
!         day's data.
!
          IF (Rcs .GT. NoRec) THEN
             WRITE(ErrorMessage, 9787) Rcs, NoRec
  9787       FORMAT('Rcs > NoRec!  Rcs,NoRec=', 2I7)
             CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
             GOTO 827
          END IF
          Rcs = Rcs + 1
          CALL SequenceDate(Day, Month, Year, Date)

          WriteFlagNum = WriteFlagNum + 1
          IF (WriteFlagNum .GE. WriteFlagInterval) THEN
             WRITE(MessageString, 9010) Day, MonCode3(Month), Year
             ErrorMessage = TRIM(MessageString)    ! overwrite previous line
             CALL WriteErrorMessage(ErrorMessage, '~^~', -1, .TRUE.)
             WriteFlagNum = 0
          END IF

        ELSE

!
!         Flag = "new", must calculate weighted data.
!
          Rcp = L2
          DataNo = 0
          IF (.NOT. FirstNewData) THEN
            CALL SequenceDate(FirstDay, FirstMonth, FirstYear, Date)
            IF (ErrorLevel .NE. 0) GOTO 898
            FirstNewData = .TRUE.
          END IF
          NumStaToday = 0
          DO WHILE (Rcp .NE. B2)    ! at end of block if Rcp=B2

            IF (UserWantsToCancel()) GOTO 999

!
!              Dat(1) = maximum air temp    (hundredths of degrees Celsius)
!              Dat(2) = minimum air temp    (hundredths of degrees Celsius)
!              Dat(3) = precipitation       (hundredths of millimeters)
!              Dat(4) = dry bulb air temp   (hundredths of degrees Celsius)
!              Dat(5) = dewpoint temp       (hundredths of degrees Celsius)
!              Dat(6) = wind speed          (hundredths of meters per second)
!              Dat(7) = cloud cover         (hundredths,  i.e. percent)
!
             READ(U11, REC=Rcp, ERR=812) I2Lat,I2Long, (Dat(I),I=1,7), Skip40, Rcp
!
!            Save the data just read into temporary arrays.  After the entire
!            block is read we will check for duplicate locations.
!
             J = NumStaToday+1
             TLat(J) = I2Lat
             TLon(J) = I2Long
             xS_Tmp(J) = IDStringNum(I2Lat, I2Long)    ! concatenated binary Lat/Long
             TDat(J,:) = Dat(:)
             NumStaToday = NumStaToday + 1
          END DO

!
!         Now check for duplicate locations.  If any are found then average 
!         the station data values together.  One thing that makes this easy
!         is that each block of the ProvMetData file is required to be 
!         sorted in order by Lat/Long so any duplicate stations will be 
!         adjacent to each other.
!
!         StaNo is used to keep track of the revised count of *locations* 
!         for which we have data.  It's just a temporary variable and will
!         be used later.  This section of code was added on 26may99 by TSH,
!         and in order to minimize the changes I use this variable and then
!         assign its final value back to "J", which is the variable that
!         contained the station count before these changes.
!
          IF (NumStaToday .GT. 0) THEN
             LLS = xS_Tmp(1)         ! Lat/Long image
             DO I = 1, 7
                DatAccum(I) = 0.0
                DatCount(I) = 0
                IF (TDat(1,I) .GT. -9998) THEN            
                   DatAccum(I) = DatAccum(I) + TDat(1,I)
                   DatCount(I) = DatCount(I) + 1
                END IF
             END DO
             StaNo = 0

             DO K = 2, NumStaToday
               IF (xS_Tmp(K) .NE. LLS) THEN 
                  StaNo = StaNo + 1
                  xS(StaNo) = LLS                 ! location of previous station, since that
                  Qqy(StaNo) = TLat(K-1) / 100.0  ! is the location corresponding to the
                  Qqx(StaNo) = TLon(K-1) / 100.0  ! data in DatAccum & DatCount
                  DO I = 1, 7
                    IF (DatCount(I) .GT. 0) THEN
                      Qqmet(StaNo,I) = NINT(DatAccum(I) / DatCount(I))
                    ELSE
                      Qqmet(StaNo,I) = -9999
                    END IF
                  END DO

                  LLS = xS_Tmp(K)
                  DO I = 1, 7
                    DatAccum(I) = 0.0
                    DatCount(I) = 0
                  END DO
               END IF

               LLS = xS_Tmp(K)
               DO I = 1, 7
                 IF (TDat(K,I) .GT. -9998) THEN
                   DatAccum(I) = DatAccum(I) + TDat(K,I)
                   DatCount(I) = DatCount(I) + 1
                 END IF
               END DO
             END DO

!
!            Save data from final station(s) at that location
!
             StaNo = StaNo + 1
             xS(StaNo) = xS_Tmp(J)          ! location of final station, since that
             Qqy(StaNo) = TLat(J) / 100.0   ! is the location corresponding to the
             Qqx(StaNo) = TLon(J) / 100.0   ! data in DatAccum & DatCount
             DO I = 1, 7
                IF (DatCount(I) .GT. 0) THEN
                   Qqmet(StaNo,I) = NINT(DatAccum(I) / DatCount(I))
                ELSE
                   Qqmet(StaNo,I) = -9999
                END IF
             END DO
             NumStaToday = StaNo       ! assign revised count (# of unique locations) 
          END IF
          NumStaLoc = NumStaToday

!
!         Convert station locations to map coordinates; not executed if NumStaLoc = 0.
!         The values of Qqx & Qqy passed to the procedure are irrelevant.  Lt & Ln
!         are used to calculate new values for Qqx and Qqy (Lt -> Qqy, Ln -> Qqx)
!
          DO StaNo = 1, NumStaLoc
             Lt = Qqy(StaNo)
             Ln = Qqx(StaNo)
             CALL ComputeMapCoordinates(Lt, Ln, RM, XOffset, YOffset,      &
     &                                  Radius, Qqx(StaNo), Qqy(StaNo))
             IF (ErrorLevel .NE. 0) GOTO 898
          END DO

!
!         DataNo =  1 => maximum air temp
!                   2 => minimum air temp
!                   3 => precipitation
!                   4 => dry-bulb air temperature   
!                   5 => dewpoint temperature       
!                   6 => windspeed                  
!                   7 => cloud cover                
!
          OldNetworkID(:) = 0
          DataNo = 1
          DO WHILE (DataNo .LE. 7)
            CALL SequenceDate(Day, Month, Year, Date)

            WriteFlagNum = WriteFlagNum + 1
            WRITE(MessageString, 9010) Day, MonCode3(Month), Year     ! always build string for future use
            IF (WriteFlagNum .GE. WriteFlagInterval) THEN
               ErrorMessage = TRIM(MessageString)      ! overwrite previous line
               CALL WriteErrorMessage(ErrorMessage, '~^~', -1, .TRUE.)
               WriteFlagNum = 0
            END IF

            I = 1
            K = 0
            DO WHILE (K .NE. NumStaLoc)
              K = K + 1
              Q = Qqmet(K, DataNo)
              IF (Q .GT. -9000) THEN
                yS(I) = xS(K)
                Qx(I) = Qqx(K)
                Qy(I) = Qqy(K)
                Qmet(I) = Q
                I = I + 1
              END IF
            END DO
            NoSta = I - 1

            IF (NoSta .EQ. 0) THEN
              !
              !  If all is missing for this data item, skip weighting
              !  and fill in as missing.
              !
              DO Subbasin = 0, NoSubbasins+1
                 Qwmet(Subbasin,DataNo) = -9999
              END DO
              IF (DoDebug) THEN
                 WRITE(1001, 1081) Date, DataNo, 0
              END IF
            ELSE IF (NoSta .EQ. 1) THEN
              !
              !  If this is a 1-station network then do not compute new
              !  Thiessen weights, since it is unnecessary to do so.  Simply
              !  use the value at the station as the weighted value.
              !
              LogRec = LogRec + 1
              TempI2Var = 200 + DataNo
              J2 = TotalNoNetworks
              WRITE(U46, REC=LogRec, ERR=848) Date, TempI2Var, J2, Zero2
              DO Subbasin = 0, NoSubbasins+1
                Qwmet(Subbasin,DataNo) = Qmet(1)
              END DO
              IF (DoDebug) THEN
                 WRITE(1001, 1081) Date, DataNo, 1
              END IF
            ELSE
              !
              !  Network contains 2+ stations.
              !  Construct network identifier.
              !
              NetworkID(:) = 0
              DO StaNo = 1, NoSta
                NetworkID(StaNo) = yS(StaNo)
              END DO
              SymbolSize = NoSta * 4
              IF (DoDebug) THEN
                 WRITE(1001, 1081) Date, DataNo, NoSta
                 WRITE(1001, 1084, ADVANCE='NO') (NetworkID(I), I=1,NoSta)
              END IF

!
!             Is this the same network as the last one we used?
!
!             If it is then all we have to do is write an entry in the Log File
!             and increment the OldWeights counter.
!
!             If not, then get the new weights (which may be stored in the file
!             or may need to be newly computed).
!
              IF (SymbolsMatch(NetworkID, OldNetworkID)) THEN
                 CALL FindSymIndex(NetworkID, Found, NetworkNo)
                 LogRec = LogRec + 1
                 I2 = 400 + DataNo
                 J2 = NoSta
                 K2 = NetworkNo
                 WRITE(U46, REC=LogRec, ERR=848) Date, I2, J2, K2
                 OldWeights = OldWeights + 1
              ELSE
                 I = DataNo
                 J = NoSta
                 CALL GetWeights(NetworkID, Date, I, J)
              END IF

!
!             Use weights to calculate subbasin values
!
              DO Subbasin=0, NoSubbasins+1
                Qq = 0.
                DO StaNo = 1, NoSta
                  Qq = Qq + Qmet(StaNo) * qThiessenWeights(StaNo,Subbasin)
                END DO
                Qwmet(Subbasin,DataNo) = NINT(Qq)
              END DO
            END IF
            DataNo = DataNo + 1
          END DO

          IF (UserWantsToCancel()) GOTO 999

!
!         Write updated values to temporary provisional data file.
!
!         Note that when processing the temperature data inputs to the runoff model
!         we must reverse the order of data elements 1 and 2.  In the data array they are:
!                           
!         DataNo =  1 => maximum air temp   ****
!                   2 => minimum air temp   ****
!                   3 => precipitation
!
!         In the subbasin provisional data files (xxx??.PRV) though, the order is:
!         DataNo =  1 => minimum air temp   ****
!                   2 => maximum air temp   ****
!                   3 => precipitation
!             
!         They get stored into the scratch file in the same order as the PRV file (min then max)
!
          NumberOfRecords = NumberOfRecords + 1
          ScrRec = ScrRec + 1
          WRITE(777, REC=ScrRec, ERR=883) Rcs
          DO Subbasin = 0, NoSubbasins+1
             ScrRec = ScrRec + 1
             MetForOutput(1) = Qwmet(Subbasin,2)
             MetForOutput(2) = Qwmet(Subbasin,1)
             DO DataNo = 3, 7
                MetForOutput(DataNo) = Qwmet(Subbasin,DataNo)
             END DO
             WRITE(777, REC=ScrRec, ERR=883) (MetForOutput(DataNo), DataNo=1,7)
          END DO
          IF (Rcs .GT. NoRec) NoRec = Rcs
          Rcs = Rcs + 1
        END IF
      END DO

      IF (UserWantsToCancel()) GOTO 999

!
!     Write final value to the interface so user doesn't think the program
!     quit prior to the end of the new provisional data.  Because we always
!     built MessageString, even if we didn't write it out, we don't have to
!     fool around with the dates and data number at this point.  MessageString
!     will already have the correct value for the last date & data type.
!
      ErrorMessage = TRIM(MessageString)
      CALL WriteErrorMessage(ErrorMessage, '~^~', -1, .TRUE.)  ! overwrite previous line

!
!     End of ProvMetData.xxx.
!
      CALL SequenceDate(LastDay, LastMonth, LastYear, Date)
      IF (ErrorLevel .NE. 0) GOTO 898

!
!     Save Thiessen weights and network hash tables.
!
      IF (.NOT. Saved) THEN
        CALL InvalidateOldWeights(ThsnHsh, IOS)
        IF (ErrorLevel .NE. 0) GOTO 898
        WRITE(U31, REC=NetworkInMemory, ERR=832) qThiessenWeights
      END IF
      CLOSE (U31)
      CALL FileWasClosed(U31)
      CALL InvalidateOldWeights(ThsnHsh, IOS)
      IF (ErrorLevel .NE. 0) GOTO 898

      OPEN(UNIT = U36, FILE = ThsnHsh, STATUS = 'REPLACE',                 &
     &     ACCESS = 'SEQUENTIAL', FORM = 'UNFORMATTED', ERR=836)
      CALL FileWasOpened(U36)
      WRITE(U36, ERR=838) MaxNoSta
      WRITE(U36, ERR=838) HashSize
      WRITE(U36, ERR=838) MaxNoNetworks
      WRITE(U36, ERR=838) NoSubbasins
      WRITE(U36, ERR=838) MaxNoSta
      WRITE(U36, ERR=838) BasinDesignator
      WRITE(U36, ERR=838) NoNetworks
      DO I = 1, HashSize
        WRITE(U36, ERR=838) HashTable(I)
      END DO
      DO I = 1, MaxNoNetworks
        WRITE(U36, ERR=838) ChainTable(I)
      END DO
      DO I = 1, MaxNoNetworks
        WRITE(U36, ERR=838) LastUse(I)
      END DO
      CLOSE (U36)
      CALL FileWasClosed(U36)
      CLOSE (U41)
      CALL FileWasClosed(U41)
      CLOSE (U46)
      CALL FileWasClosed(U46)

      IF (UserWantsToCancel()) GOTO 999

!
!     Update the permanent subbasin weighted data files (xxxnn.PRV) with the
!     data stored in the scratch file.
!

      DO Subbasin = 0, NoSubbasins
         WRITE(SB, '(I2.2)') Subbasin
         FSubPrv = BnS//SB//'.PRV'
         U26 = GetFreeUnitNumber()
         OPEN(UNIT=U26, FILE=FSubPrv, STATUS='OLD', RECL=6,         &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=826)
         CALL FileWasOpened(U26)

         READ(U26, REC=1, ERR=827) Ibn, SDate          ! basin ID, start date
         READ(U26, REC=2, ERR=827) EDate               ! end date

         !
         ! Read the data for this subbasin from the scratch file and write 
         ! it into the xxx??.PRV file.
         !
         !
         ! This is a little messy, so let me describe the variables...
         !
         !  K   = number of data records in each "block"
         !  J   = record number in the scratch file where we stored the record
         !        number in the PRV file for this data.  Essentially it is the
         !        date for the data in this block.  See the description of the
         !        scratch file.
         !  B2  = record number in the scratch file where we stored the data
         !        for this particular subbasin on this particular date.
         !  R2  = record number in the PRV file where this data will be written
         !  
         !
         K = 1 + (NoSubbasins+2)      ! one record_number_rec + 0..n subbasin_recs
         DO I = 1, NumberOfRecords
            J = (I-1)*K + 1
            B2 = J + Subbasin + 1
            READ(777, REC= J, ERR=882) R2
            READ(777, REC=B2, ERR=882) (MetForOutput(C2), C2=1,3)
            WRITE(U26, REC=R2, ERR=828) (MetForOutput(C2), C2=1,3)
         END DO

         DateOfData = SDate + R2 - 3    ! records 3 .. n
         IF (DateOfData .GT. EDate) EDate = DateOfData

         WRITE(U26, REC=1, ERR=828) Ibn, SDate
         WRITE(U26, REC=2, ERR=828) EDate
         CLOSE(U26)
         CALL FileWasClosed(U26)

      END DO
  
!
!     Update the permanent lake-surface weighted data file (xxxMD.PRV) with the
!     data stored in the scratch file.
!
      U26 = GetFreeUnitNumber()
      FSubPrv = BnS//'MD.PRV'
      OPEN(UNIT=U26, FILE=FSubPrv, STATUS='OLD', RECL=8,            &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=826)
      CALL FileWasOpened(U26)

      READ(U26, REC=1, ERR=827) Ibn, SDate          ! basin ID, start date
      READ(U26, REC=2, ERR=827) EDate               ! end date

      !
      ! Read the data for this "subbasin" from the scratch file and write 
      ! it into the xxxMD.PRV file.
      !
      K = 1 + (NoSubbasins+2)      ! one record_number_rec + 0..n subbasin_recs
      DO I = 1, NumberOfRecords
         J = (I-1)*K + 1
         B2 = J + NoSubbasins + 2
         READ(777, REC= J, ERR=882) R2
         READ(777, REC=B2, ERR=882) (MetForOutput(C2), C2=4,7)
         WRITE(U26, REC=R2, ERR=828) (MetForOutput(C2), C2=4,7)
      END DO

      DateOfData = SDate + R2 - 3    ! records 3 .. n
      IF (DateOfData .GT. EDate) EDate = DateOfData

      WRITE(U26, REC=1, ERR=828) Ibn, SDate
      WRITE(U26, REC=2, ERR=828) EDate
      CLOSE(U26)
      CALL FileWasClosed(U26)

      CLOSE(777)
      CALL FileWasClosed(777)

!
!     Set flags to "old" for processed data.
!
      Rcp = zOne
      DO WHILE (Rcp .NE. Zero)
        READ(U11, REC = Rcp, ERR=812) Date, B2, R2, J, Skip42, L2
        IF (R2 .GT. Zero) WRITE(U11, REC=Rcp, ERR=813) Date, B2, -R2, J, Null42, L2
        Rcp = B2
      END DO
      CLOSE (U11)
      CALL FileWasClosed(U11)

!
!     Store earliest changed date in file BSNNM so that rest of
!     forecast package knows when to begin updating of other files.
!
      U56 = GetFreeUnitNumber()
      WriteToBsnFile = .FALSE.
      OPEN(UNIT=U56, FILE=BsnFile, STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=856)
      CALL FileWasOpened(U56)
      WriteToBsnFile = .TRUE.

!
!     3 = designator for this program.
!
      READ(U56, 1006, REC=2, ERR=857)  I, J, OTLCode     
      WRITE(U56, 1006, REC=2, ERR=858) I, 3, OTLCode, Lf 
      CALL DateSequence(LastDay, LastMonth, LastYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898
      K = K + 1                 ! K = Sequence number of 1st day of forecast.
      IF (.NOT. FirstNewData)                                             &
     &    CALL SequenceDate(FirstDay, FirstMonth, FirstYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898

!
!     Provisional data end date
!
      WRITE(U56, 1002, REC=4, ERR=858) LastDay, LastMonth, LastYear, Lf

!
!     Forecast start date is "normally" the day after the end of the
!     provisional data; however, it will be reassigned in the forecast
!     procedure by program FRCST.
!
      CALL SequenceDate(LastDay, LastMonth, LastYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=5, ERR=858) LastDay, LastMonth, LastYear, Lf

!
!     Determine starting date for model statistics output (1st of month
!     of forecast beginning minus 18 months and 1 day) and save in
!     BSNNM; however, it will be reassigned in the forecast procedure
!     by program FRCST.
!
      CALL SequenceDate(LastDay, LastMonth, LastYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898
      LastMonth = LastMonth + 18
      LastYear = LastYear - 3 + (LastMonth - 1) / 12
      LastMonth = MOD(LastMonth - 1, 12) + 1
      LastDay = 1
      CALL DateSequence(LastDay, LastMonth, LastYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898
      K = K - 1
      CALL SequenceDate(LastDay, LastMonth, LastYear, K)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=7, ERR=858) LastDay, LastMonth, LastYear, Lf

!
!     Write earliest date sequence number of added/changed data for other
!     files, comparing it to the date already there for them, and writing the
!     earliest back.  This number is used by other programs that update those
!     files as the beginning of the update; at that time, the other programs
!     correct this number to reflect the update.
!
      CALL DateSequence(FirstDay, FirstMonth, FirstYear, Starts)
      IF (ErrorLevel .NE. 0) GOTO 898

!
!     Date of earliest new data put by this program since
!      WATOUT was last run to update xxxnn.SUM.
!
      READ(U56, 1002, REC=8, IOSTAT=IOS) LastDay, LastMonth, LastYear 
      IF (IOS .EQ. 0) THEN
         CALL DateSequence(LastDay, LastMonth, LastYear, Date)
      ELSE
         CALL DateSequence(31, 12, 2999, Date)
      END IF
      IF (ErrorLevel .NE. 0) GOTO 898
      IF (Date .GT. Starts) Date = Starts
      CALL SequenceDate(LastDay, LastMonth, LastYear, Date)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=8, ERR=858) LastDay, LastMonth, LastYear, Lf   ! 21mar95

!
!     Date of earliest new data put by this program since
!      LUMPOUT was last run to update xxxALL.SUM.
!
      READ(U56, 1002, REC=9, IOSTAT=IOS) LastDay, LastMonth, LastYear 
      IF (IOS .EQ. 0) THEN
         CALL DateSequence(LastDay, LastMonth, LastYear, Date)
      ELSE
         CALL DateSequence(31, 12, 2999, Date)
      END IF
      IF (ErrorLevel .NE. 0) GOTO 898
      IF (Date .GT. Starts) Date = Starts
      CALL SequenceDate(LastDay, LastMonth, LastYear, Date)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=9, ERR=858) LastDay, LastMonth, LastYear, Lf   ! 21mar95

!
!     Date of earliest new data put by this program since
!      CMOFPOSR was last run to update xxxALL.MET.
!
      READ(U56, 1002, REC=10, IOSTAT=IOS) LastDay, LastMonth, LastYear 
      IF (IOS .EQ. 0) THEN
         CALL DateSequence(LastDay, LastMonth, LastYear, Date)
      ELSE
         CALL DateSequence(31, 12, 2999, Date)
      END IF
      IF (ErrorLevel .NE. 0) GOTO 898
      IF (Date .GT. Starts) Date = Starts
      CALL SequenceDate(LastDay, LastMonth, LastYear, Date)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=10, ERR=858) LastDay, LastMonth, LastYear, Lf   ! 21mar95

!
!     Date of earliest new data put by this program since
!      EVAPOUT was last run to update ZSEVAPPR.xxx
!
      READ(U56, 1002, REC=11, IOSTAT=IOS) LastDay, LastMonth, LastYear 
      IF (IOS .EQ. 0) THEN
         CALL DateSequence(LastDay, LastMonth, LastYear, Date)
      ELSE
         CALL DateSequence(31, 12, 2999, Date)
      END IF
      IF (ErrorLevel .NE. 0) GOTO 898
      IF (Date .GT. Starts) Date = Starts
      CALL SequenceDate(LastDay, LastMonth, LastYear, Date)
      IF (ErrorLevel .NE. 0) GOTO 898
      WRITE(U56, 1002, REC=11, ERR=858) LastDay, LastMonth, LastYear, Lf   ! 21mar95

      CLOSE (U56)
      CALL FileWasClosed(U56)

      IF (UserWantsToCancel()) GOTO 999

 875  J = IOS
      ErrorLevel = 0
      ErrorMessage = 'Thiessen weighting of station data completed for '//TRIM(LakeName)
      CALL WriteErrorMessage(ErrorMessage, '~^~', -1, .TRUE.)  ! overwrite previous line
      WRITE(ErrorMessage, 9031) NewWeights, OldWeights
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      GOTO 999

!
!     Error handling...
!
 811  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(FileNamePRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 812  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(FileNamePRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 813  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(FileNamePRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 816  ErrorMessage = 'DISAVMET: Error opening file ' // 'AREA.' // BnS
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 817  ErrorMessage = 'DISAVMET: Error reading file ' // 'AREA.' // BnS
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 818  ErrorMessage = 'DISAVMET: Error writing file ' // 'AREA.' // BnS
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 821  ErrorMessage = 'DISAVMET: Error opening file ' // BnS // 'BYTCD.MAP'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 822  ErrorMessage = 'DISAVMET: Error reading file ' // BnS // 'BYTCD.MAP'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 823  ErrorMessage = 'DISAVMET: Error writing file ' // BnS // 'BYTCD.MAP'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 826  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(FSubPRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 827  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(FSubPRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 828  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(FSubPRV)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 831  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(ThsnDat)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 832  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(ThsnDat)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 833  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(ThsnDat)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 836  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(ThsnHsh)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 837  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(ThsnHsh)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 838  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(ThsnHsh)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 841  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(ThsnSym)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 842  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(ThsnSym)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 843  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(ThsnSym)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 846  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(LogFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 847  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(LogFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 848  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(LogFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 856  ErrorMessage = 'DISAVMET: Error opening file ' // TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 857  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 858  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 881  ErrorMessage = 'DISAVMET: Error opening scratch file'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 882  ErrorMessage = 'DISAVMET: Error reading scratch file'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 883  ErrorMessage = 'DISAVMET: Error writing scratch file'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898


 898  ErrorLevel = 1

!
!     Invalidate use of any programs by writing a -1 in the Pd field of BSNNM
!
      IF (WriteToBsnFile) THEN
        WriteToBsnFile = .FALSE.
        U56 = GetFreeUnitNumber()
        OPEN(UNIT=U56, FILE=BsnFile, STATUS='OLD', RECL=12,                 &
     &       ACCESS='DIRECT', FORM='FORMATTED', ERR=856)
        CALL FileWasOpened(U56)
        WriteToBsnFile = .TRUE.
        READ (U56, 1007, REC=2, ERR=857) Str3,  I, Str5
        WRITE(U56, 1007, REC=2, ERR=858) Str3, -1, Str5, Lf
        CLOSE (U56)
        CALL FileWasClosed(U56)
      END IF

 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DISAVMET_MAIN...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      GOTO 999

!
!     Return to calling procedure.  I used a GOTO 999 throughout the program
!     instead of a RETURN everywhere so that I can do any "cleanup" here.
!
 999  CONTINUE

!
!    Deallocate the dynamic arrays as part of termination regardless of error
!    status.  If the arrays were never allocated then the IOS variable will be
!    set to a non-zero value indicating an error, but it will not actually
!    cause any problems.  This is the cleanest way of handling the deallocation
!    that I could devise, given the multiple exit points (errors or no error).
!
      DEALLOCATE(QMet, Qqmet, Qqx, Qqy, xS, yS, LastUse,                  &
     &           xS_Tmp, TLat, TLon, TDat, STAT=IOS)
      DEALLOCATE(NetworkID, OldNetworkID, zMapRow, STAT=IOS)
      DEALLOCATE(TmpSymbol, HashTable, ChainTable, qThiessenWeights,      &
     &           qCount, zMap, Qx, Qy, SrtX, SrtY, SrtN,                  &
     &           Lrow, Rrow, STAT=IOS)

!
!     The rest of the normal cleanup
!
      CALL CloseAllFiles()     ! guarantee that all files are closed
      CALL WriteErrorMessage('0', '~%~', -1, .TRUE.)
      RETURN

!
!  FORMATs
!
 1000 FORMAT(45X, I5)
 1001 FORMAT(A10)
 1002 FORMAT(2I3, I5, A1)
 1005 FORMAT(3X, I3, 5X, A1)
 1006 FORMAT(2I3, A4, 1X, A1)
 1007 FORMAT(A3, I3, A5, A1)

 1081 FORMAT('Date=',I0, '  DataNo=',I1, '  NoSta=',I0)
 1082 FORMAT('Date=',I0, '  DataNo=',I1)
 1084 FORMAT('ID=', 1000(1X,Z0))

 2201 FORMAT(E13.6E3)

 9010 FORMAT('Processing:', I3, ' ', A3, I5)
 9020 FORMAT('Attempted to allocate ', I0, ' bytes')
 9021 FORMAT('Attempted to allocate additional', I0, ' bytes of ', I0, ' total')
 9031 FORMAT('New weights computed ', I0, ' times and old weights used ', I0, ' times.')

      END SUBROUTINE MainRoutine


!---------------------------------------------------------------------------
      SUBROUTINE InvalidateOldWeights (ThsnHsh, IOS)

      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: IOS
      CHARACTER (LEN=*), INTENT(INOUT) :: ThsnHsh
      INTEGER :: I
      LOGICAL :: IsOpen

      IF (FileIsOpen(U36)) THEN
         CLOSE(U36, STATUS='DELETE')
         CALL FileWasClosed(36)
      END IF

      INQUIRE(FILE=TRIM(ThsnHsh), NUMBER=I, OPENED=IsOpen)
      IF (IsOpen) THEN
         CLOSE(I, STATUS='DELETE')
         CALL FileWasClosed(I)
      END IF

      OPEN(UNIT=U36, FILE=TRIM(ThsnHsh), STATUS='REPLACE',                 &
     &     ACCESS='SEQUENTIAL', FORM='UNFORMATTED', IOSTAT=IOS)
      CALL FileWasOpened(U36)
      CLOSE (U36, IOSTAT=IOS)
      CALL FileWasClosed(U36)

      RETURN

      END SUBROUTINE InvalidateOldWeights

!----------------------------------------------------------------------------
      INTEGER FUNCTION IDStringNum (Lt, Ln)
!
!     This routine returns the concatenated binary image of the two 2-byte
!     integers as a 4-byte integer.
!     An effort is made to enable subsequent ordering by latitude.
!
!     Note that these bitwise operators number the bits in descending order.
!     e.g.  15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0
!
!     Also remember that Intel is "little-endian". i.e. the bytes are stored
!     in reverse order.
!
!     Bytes of the resultant value are arranged like this:
!     (low-order byte of long) (high-order byte of long) (low of lat) (high of lat)
!
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2), INTENT(IN) :: Lt, Ln
      INTEGER :: NewNum, I

      !
      !  We need to assign the 2-byte latitude value into the 2 highest
      !  order bytes of NewNum.  
      !
      NewNum = 0
      DO I = 15, 0, -1               
         IF (BTEST(Lt, I)) THEN
            NewNum = IBSET(NewNum, I)           ! 3rd + 4th bytes of NewNum
         END IF
      END DO

      !
      !  Bit-wise assignment of the 2-byte longitude value into NewNum  
      !
      DO I = 15, 0, -1               
         IF (BTEST(Ln, I)) THEN
            NewNum = IBSET(NewNum, I+16)           ! first 2 bytes of NewNum
         END IF
      END DO

      IDStringNum = NewNum
      RETURN
      END FUNCTION IDStringNum

!---------------------------------------------------------------------------
      INTEGER FUNCTION HashNum (SymArray, HSize)
!
!     This function returns the hash code (i.e. "bin" number) for a particular 
!     network given the NetworkID (SymArray), number of stations (i.e. MaxNoSta),
!     and the hash size (i.e. number of hashing "bins").
!
!     Result is in the range 1..HSize
!
      IMPLICIT NONE
      INTEGER, DIMENSION(:), INTENT(IN) :: SymArray
      INTEGER,               INTENT(IN) :: HSize
      INTEGER  ::  H, I, J, Bt, BitNo, Ndx, IntVal, TotalBits

      !
      !  Construct a 32-bit integer by splicing together evenly spaced bits from
      !  the network symbol array.  Keep the sign bit zero (positive).  That is
      !  the leftmost bit and is bit number 31 as bitwise operators treat things.
      !
      IntVal = 0
      TotalBits = MaxNoSta * 32
      DO I = 0, 30
        BitNo = I * (TotalBits / 31)     ! integer division, remember  (0..x)
        Ndx   = (BitNo / 32) + 1         ! which entry in array?     (1..MaxNoSta)
        Bt    = MOD(BitNo, 32)           ! which bit in that entry?  (0..31)
        Bt    = 31 - Bt                  ! for bitwise operators bits are numbered 31, 30, 29,...,0
        J     = SymArray(Ndx)
        IF (BTEST(J,Bt)) IntVal = IBSET(IntVal,30-I)    ! set the bit (leftmost+1 first)
      END DO

      H = MOD(IntVal, HSize) + 1
      IF ((H .LE. 0) .OR. (H .GT. HSize)) THEN
         WRITE(ErrorMessage, 9001) H, IntVal
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorMessage = ' '; 
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         H = -1
      END IF

      HashNum = H
      RETURN

 9001 FORMAT('HashNum() malfunctioned!  Produced the value ', I0, ' from IntVal ', I0)

      END FUNCTION HashNum

!---------------------------------------------------------------------------
      SUBROUTINE FindSymIndex(NetSym, Found, Indecs)
!
!     This routine searches for a given symbol in the hashed symbol table
!     and, if it is found, returns Found = .TRUE. and its index in the symbol
!     table.  NetSym is the symbol to find.  HashTable is the table (i.e.
!     hashing "bin") to hash to; its contents point to positions in the 
!     SymbolTable (on disk) and in the ChainTable.  If a symbol hashes to the 
!     same location in the HashTable, then it will exist in the symbol table 
!     along the links established by the ChainTable.
!
      IMPLICIT NONE
      INTEGER, DIMENSION(:), INTENT(IN)    :: NetSym
      INTEGER,               INTENT(INOUT) :: Indecs
      LOGICAL,               INTENT(INOUT) :: Found

      INTEGER :: ValFromHash, J
      LOGICAL :: Quit

      ValFromHash = HashNum(NetSym, HashSize)
      IF (ValFromHash .LE. 0) THEN
         Found = .FALSE.
         RETURN
      END IF

      Indecs = HashTable(ValFromHash)
      IF (Indecs .EQ. 0) THEN
        Found = .FALSE.
      ELSE
        READ(U41, REC=Indecs, ERR=800) TmpSymbol(:)
        Found = SymbolsMatch(NetSym, TmpSymbol)
        Quit = Found .OR. (Indecs .LE. 0) .OR. (ChainTable(Indecs) .EQ. 0)
        DO WHILE (.NOT. Quit)
          Indecs = ChainTable(Indecs)
          IF (Indecs .GT. 0) THEN
            READ(U41, REC=Indecs, ERR=800) TmpSymbol(:)
            Found = SymbolsMatch(NetSym, TmpSymbol)
          END IF
          Quit = Found .OR. (Indecs .LE. 0)
          IF (Indecs .GT. 0) Quit = Quit .OR. (ChainTable(Indecs) .EQ. 0)
        END DO
      END IF
      RETURN

 800  WRITE(ErrorMessage, 8001) Indecs
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      CALL WriteErrorMessage(' ', '', -1, .TRUE.)
      CALL ReinitThiessen()
      ErrorLevel = 50
      Found = .FALSE.
      RETURN

 8001 FORMAT('Error in FindSymIndex attempting to read THSN_SYM record number ', I0)

      END SUBROUTINE FindSymIndex

!---------------------------------------------------------------------------
      SUBROUTINE StoreSymbol(NetSym, Indecs)
!
!     This routine stores a symbol in the given hashed symbol table.  See
!     comments in the procedure: FindSymbol.  Indecs is the place in the
!     tables to store the symbol.  THIS ROUTINE DOES NOT CHECK TO SEE IF
!     SYMBOL IS ALREADY IN THE TABLES NOR IF THE TABLES OVERFLOW!
!
      IMPLICIT NONE
      INTEGER, DIMENSION(:), INTENT(IN)    :: NetSym
      INTEGER,               INTENT(INOUT) :: Indecs
      INTEGER :: I

      IF (Indecs .LE. 0) THEN
         CALL ReinitThiessen()
         Indecs = 1
      END IF
      WRITE(U41, REC=Indecs, ERR=800) NetSym(:)          ! Store in symbol table
      I = HashNum(NetSym, HashSize)
      IF (I .LE. 0) THEN
         CALL ReinitThiessen()
         RETURN
      END IF

      IF (HashTable(I) .EQ. 0) THEN
         ChainTable(Indecs) = 0                         ! Start new chain
      ELSE
         ChainTable(Indecs) = HashTable(I)              ! Insert in chain
      END IF
      HashTable(I) = Indecs                             ! Store in hash table
      RETURN

 800  WRITE(ErrorMessage, 8001) Indecs
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      CALL WriteErrorMessage(' ', '', -1, .TRUE.)
      CALL ReinitThiessen()
      ErrorLevel = 50
      RETURN

 8001 FORMAT('Error in StoreSymbol attempting to write THSN_SYM record number ', I0)

      END SUBROUTINE StoreSymbol

!---------------------------------------------------------------------------
      SUBROUTINE DeleteSymbol (NetSym, Indecs)
!
!     This routine finds and deletes a given symbol from the hashed symbol
!     table and returns the index in the symbol table of the new vacancy.  See
!     comments in the procedure: FindSymbol.  ASSUMES THAT SYMBOL EXISTS IN
!     THE HASHED SYMBOL TABLE; IT IS UNCHECKED HERE!!!
!
      IMPLICIT NONE
      INTEGER, DIMENSION(:), INTENT(IN)    :: NetSym
      INTEGER,               INTENT(INOUT) :: Indecs
      INTEGER :: IndexSave
      LOGICAL :: Found

      IndexSave = HashNum(NetSym, HashSize)
      IF (IndexSave .LE. 0) GOTO 700
      Indecs = HashTable(IndexSave)
      IF (Indecs .EQ. 0) GOTO 700
      READ(U41, REC=Indecs, ERR=800) TmpSymbol(:)
      IF (SymbolsMatch(NetSym, TmpSymbol)) THEN
         HashTable(IndexSave) = ChainTable(Indecs)
      ELSE
         Found = SymbolsMatch(NetSym, TmpSymbol)
         DO WHILE (.NOT. Found)
           IndexSave = Indecs
           Indecs = ChainTable(Indecs)
           IF (Indecs .EQ. 0) GOTO 700
           READ(U41, REC=Indecs, ERR=800) TmpSymbol(:)
         END DO
         ChainTable(IndexSave) = ChainTable(Indecs)
      END IF

      RETURN

 700  CALL ReinitThiessen()
      Indecs = 1
      RETURN

 800  WRITE(ErrorMessage, 8001) Indecs
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      CALL WriteErrorMessage(' ', '', -1, .TRUE.)
      CALL ReinitThiessen()
      ErrorLevel = 50
      RETURN
 8001 FORMAT('Error in DeleteSymbol attempting to read THSN_SYM record number ', I0)

      END SUBROUTINE DeleteSymbol

!---------------------------------------------------------------------------
      LOGICAL FUNCTION SymbolsMatch(Sym1, Sym2)
      IMPLICIT   NONE
      INTEGER, DIMENSION(:), INTENT(IN) :: Sym1, Sym2
      INTEGER :: I, L
      LOGICAL :: Match

      L = UBOUND(Sym1, 1)     ! Get size of Sym1 array. Sym2 is supposed to be same size
      Match = .TRUE.
      DO I = 1, L
         IF (Sym1(I) .NE. Sym2(I)) Match = .FALSE.
      END DO
      SymbolsMatch = Match

      END FUNCTION SymbolsMatch

!---------------------------------------------------------------------------
      INTEGER FUNCTION NumStaInNetwork(NetID)
      IMPLICIT  NONE
      INTEGER, DIMENSION(:), INTENT(IN) :: NetID
      INTEGER :: I, L, NS

      L = UBOUND(NetID, 1)     ! Get size of network array
      DO I = 1, L
         IF (NetID(I) .NE. 0) NS = I
      END DO
      NumStaInNetwork = NS

      END FUNCTION NumStaInNetwork

!---------------------------------------------------------------------------
      SUBROUTINE ReinitThiessen()
!
!     This routine deletes (if needed) and creates new thiessen files.
!
      IMPLICIT NONE
      INTEGER            :: I, J, IOS
      CHARACTER (LEN=80) :: S

      INTEGER (KIND=INTEGER2) :: Zero2
      INTEGER                 :: Zero4
      CHARACTER (LEN=1)  :: Null

      Zero2 = 0
      Zero4 = 0
      Null = CHAR(0)

!
!  Close open files
!
      IF (FileIsOpen(U41)) THEN 
         CLOSE(U41, IOSTAT=IOS)    !  close symbol file
         CALL FileWasClosed(U41)
      END IF
      IF (FileIsOpen(U31)) THEN
         CLOSE(U31, IOSTAT=IOS)    !  close dat file
         CALL FileWasClosed(U31)
      END IF
      IF (FileIsOpen(U36)) THEN
         CLOSE(U36, IOSTAT=IOS)    !  close hash file
         CALL FileWasClosed(U36)
      END IF

!
!  Open the new files, replacing old ones if they exist
!
      OPEN(UNIT=U41, FILE=ThsnSym, STATUS='REPLACE', ACCESS='DIRECT',          &
     &     RECL=MaxNoSta*4, FORM='UNFORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'DISAVMET: Error opening new file '//TRIM(ThsnSym)
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF
      CALL FileWasOpened(U41)
      DO I=1, MaxNoNetworks
         WRITE(U41, REC=I) (Zero4, J=1,MaxNoSta)   ! fill record with zero
      END DO

      OPEN(UNIT=U31, FILE=ThsnDat, STATUS='REPLACE', ACCESS='DIRECT',          &
     &     RECL=ThsnDatRecl, FORM='UNFORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'DISAVMET: Error opening new file '//TRIM(ThsnDat)
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF
      CALL FileWasOpened(U31)
      DO I=1, MaxNoNetworks
         WRITE(U31, REC=I) (Null, J=1,ThsnDatRecl)   ! fill record with nulls
      END DO


      OPEN(UNIT=U36, FILE=ThsnHsh, STATUS='REPLACE', ACCESS='SEQUENTIAL',      &
     &     FORM='UNFORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'DISAVMET: Error opening new file '//TRIM(ThsnHsh)
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF
      CALL FileWasOpened(U36)
      WRITE(U36) MaxNoSta, HashSize, MaxNoNetworks, NoSubbasins, MaxNoSta, Zero2, Zero4
      DO I=1, HashSize
         WRITE(U36) Zero2        ! HashTable is an array of 2-byte integers
      END DO
      DO I=1, MaxNoNetworks
         WRITE(U36) Zero2        ! ChainTable is an array of 2-byte integers
      END DO
      DO I = 1, MaxNoNetworks
        WRITE(U36) Zero4         ! LastUse is an array of 4-byte integers
      END DO
      CLOSE (U36)                                                         
      CALL FileWasClosed(U36)

      NoNetworks = 0
      DO I = 1, HashSize
        HashTable(I) = 0
      END DO
      DO I=1, MaxNoNetworks
        ChainTable(I) = 0
        LastUse(I) = 0
      END DO


      WRITE(ErrorMessage, *) 'New Thiessen files created'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      RETURN

 898  ErrorLevel = 50
      RETURN

      END SUBROUTINE ReinitThiessen

!---------------------------------------------------------------------------
      SUBROUTINE GetWeights(NetworkID, Date, DatNum, NoSta)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: Date, DatNum, NoSta
      INTEGER, DIMENSION(:), INTENT(IN) :: NetworkID     ! array of 4-byte integers (1:MaxNoSta)

      INTEGER (KIND=INTEGER2) :: I2, J2, K2
      INTEGER :: I, Indecs, OldestAge
      LOGICAL :: Found, DoDelete

      !
      !  Does the network exist in the saved entries?
      !  If so, then just read that record from the THSN_DAT.xxx file,
      !  update a few things, and return.
      !
      CALL FindSymIndex(NetworkID, Found, Indecs)
      IF (Found) THEN
         READ(U31, REC=Indecs, ERR=832) qThiessenWeights
         LastUse(Indecs) = Date
         OldWeights = OldWeights + 1

         LogRec = LogRec + 1
         I2 = 300 + DatNum
         J2 = NoSta
         K2 = Indecs
         WRITE(U46, REC=LogRec, ERR=848) Date, I2, J2, K2

         RETURN
      END IF

      !
      !  OK, we didn't find this network in the saved set.  
      !  The first thing we do is compute the thiessen weights.
      !  
      I2 = NoSta
      CALL ThiessenWeights(I2)
      IF (ErrorLevel .NE. 0) GOTO 898

      !
      !  Now we want to save these new weights in the files.  That may require
      !  that we delete an entry.  If any entry of LastUse() is zero then that 
      !  slot is empty and we don't have to delete anything.  Otherwise we will have
      !  to delete the oldest entry.  Once we have an index number (i.e. record
      !  number) we can save the symbol and weights.
      !
      DoDelete = .TRUE.
      Indecs = -1
      DO I = MaxNoNetworks, 1, -1
         IF (LastUse(I) .EQ. 0) THEN
            DoDelete = .FALSE.
            Indecs = I
         END IF
      END DO

      IF (DoDelete) THEN
         OldestAge = LastUse(1)
         Indecs = 1
         DO I = 2, MaxNoNetworks
            IF (LastUse(I) .LT. OldestAge) THEN
               OldestAge = LastUse(I)
               Indecs = I
            END IF
         END DO
         READ(U41, REC=Indecs) TmpSymbol
         CALL DeleteSymbol(TmpSymbol, Indecs)     ! Delete TmpSymbol and adjust HashTable, ChainTable
         IF (ErrorLevel .NE. 0) GOTO 898
         LogRec = LogRec + 1
         I2 = 100 + DatNum
         J2 = NumStaInNetwork(TmpSymbol)          ! num stations that were in deleted network
         K2 = Indecs
         WRITE(U46, REC=LogRec, ERR=848) Date, I2, J2, K2
      END IF

      IF (Indecs .GT. 0) THEN
         CALL StoreSymbol(NetworkID, Indecs)      ! store symbol in record# Indecs
         IF (ErrorLevel .NE. 0) GOTO 898
         WRITE(U31, REC=Indecs, ERR=833) qThiessenWeights
         LastUse(Indecs) = Date
         NewWeights = NewWeights + 1
         LogRec = LogRec + 1
         I2 = 0 + DatNum
         J2 = NoSta
         K2 = Indecs
         WRITE(U46, REC=LogRec, ERR=848) Date, I2, J2, K2
      END IF

      RETURN

 832  ErrorMessage = 'DISAVMET: Error reading file ' // TRIM(ThsnDat)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 833  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(ThsnDat)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898
 848  ErrorMessage = 'DISAVMET: Error writing file ' // TRIM(LogFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.);  GOTO 898

 898  ErrorLevel = 1

      END SUBROUTINE GetWeights



!---------------------------------------------------------------------------
      LOGICAL FUNCTION ClosestStation (I, J)
!
!     This routine returns .TRUE. if the closest station to point (I, J) is
!     StaNo and .FALSE. if not; assumes the StaNo is the first station
!     represented in the arrays: SrtX, SrtY, and SrtN, defined in the calling
!     routine.  If the arrays represent all stations in the network sorted in
!     order of distance from StaNo, then this routine will find .FALSE.'s
!     faster than if not ordered.  The structure of the IF statement herein is
!     designed to guarantee this routine always gives unique assignment of
!     closest station for any (and all) points no matter what the value of
!     StaNo; i. e., if this routine returns .TRUE. for one station, then it
!     will return .FALSE. for all other stations for the same point (I, J).
!     This is a consideration only when a point is equidistant from two
!     stations, a rare but possible problem.
!
      IMPLICIT NONE
      REAL     ::  W, V
      INTEGER , INTENT(IN) ::  I, J
      INTEGER  ::  StaNo

      W = (SrtX(1) - I) ** 2 + (SrtY(1) - J) ** 2
      ClosestStation = .FALSE.
      DO StaNo = 2, StaLocN
        V = (SrtX(StaNo) - I) ** 2 + (SrtY(StaNo) - J) ** 2
        IF (V .LT. W) RETURN
        IF ((ABS(V-W) .LT. 0.000001) .AND. (SrtN(StaNo) .LT. SrtN(1))) RETURN
      END DO
      ClosestStation = .TRUE.

      RETURN

      END FUNCTION ClosestStation


!---------------------------------------------------------------------------
      SUBROUTINE ThiessenWeights (NoStations)
!
!       This routine calculates Thiessen weights for all meteorological
!       stations, for each subbasin, for all subbasins.
!
!       Inputs are: basin map, zMap(0:MapWidth, 0:MapHeight)
!                         where 'zMap(I, J)' is array of codes,
!                                  one code for each subbasin
!                               'I' is the x-coordinate of an area on the map
!                               'J' is the y-coordinate of an area on the map
!                    number of meteorological stations in network, NoSta
!                    station coordinates, Qx(1:NoSta), Qy(1:NoSta)
!                         where 'Qx(StaNo)' is the x-coordinate of station
!                               'Qy(StaNo)' is the y-coordinate of station
!                               'StaNo' is station index between 1 and NoSta
!
!       Outputs are: weight table, qThiessenWeights(1:NoSta)
!                        where 'qThiessenWeights(StaNo)'
!                                  is the Thiessen weight for indicated station
!
!
!       Thiessen weights are determined by computing polygon maps (one at a
!       time and disposing of each before getting the next) and counting the
!       codes of each subbasin in the  polygon  row 'windows' (between left
!       and right columns in each row).
!
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2), INTENT(IN) :: NoStations
      INTEGER (KIND=INTEGER4), DIMENSION(:,:), ALLOCATABLE :: Qtw    ! (1:MaxNoSta,0:NoSubbasins+2)
      INTEGER (KIND=INTEGER4) :: Qk, Qkk
      INTEGER (KIND=INTEGER2) :: Subbasin, NoStaMinusOne, L, R, I, J
      INTEGER  :: IOS

!
!     The following statement in the original program was simply
!     a memory-saving device.  There was no *requirement* that they be
!     equivalenced.  The programmer (TEC?) just recognized that the same
!     memory block could be used for both arrays.
!
!        EQUIVALENCE (qThiessenWeights, Qtw)
!
!     Fortran90 programming style strongly discourages the EQUIVALENCE
!     statement (it has been designated an "obsolete" feature), so I have
!     removed it.  We also no longer need it since we're not constrained
!     by the 640K DOS limit on RAM.  Note that we don't want to go "wild"
!     in our RAM usage, but this is a relatively small block of RAM we
!     are using (MaxNoSta*4 bytes, so with MaxNoSta=500, RAM usage=2000 bytes).
!     The trade-off of using <2k extra RAM vs EQUIVALENCE is worth it, IMHO.
!
      IF (NoStations .GT. MaxNoSta) THEN
         ErrorMessage = 'Too many stations : ThiessenWeights'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorLevel = 1
         RETURN
      END IF

      ALLOCATE(Qtw(1:MaxNoSta,0:NoSubbasins+2), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'Error allocating RAM : ThiessenWeights'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorLevel = 1
         RETURN
      END IF
      Qtw(:,:) = 0

      DO I = 0, NoSubbasins+2
        DO J = 1, MaxNoSta
           Qtw(J,I) = 0.0
        END DO
      END DO

      PNoSta = NoStations
      IF (PNoSta .EQ. 1) THEN
        DO Subbasin = 0, NoSubbasins
          qThiessenWeights(1,Subbasin) = 1.0
        END DO
      ELSE
        NoStaMinusOne = PNoSta - 1
        DO PStaNo = 1, NoStaMinusOne
          DO Subbasin = 0, NoSubbasins+2
            Qtw(PStaNo,Subbasin) = 0
          END DO
          CALL ThiessenPolygon
          DO J = Ylower, Yupper
            L = Lrow(J)
            R = Rrow(J)
            DO I = L, R
              Subbasin = ICHAR(zMap(I, J))
              Qtw(PStaNo,Subbasin) = Qtw(PStaNo,Subbasin) + 1
            END DO
          END DO
        END DO

        DO PStaNo = 1, NoStaMinusOne
           Qtw(PStaNo, 0) = Qtw(PStaNo, 0) + Qtw(PStaNo,BridgeCode)
        END DO

        DO Subbasin = 0, NoSubbasins
          Qk = 0
          DO PStaNo = 1, NoStaMinusOne
           Qkk = Qtw(PStaNo,Subbasin)
           Qk = Qk + Qkk
           qThiessenWeights(PStaNo,Subbasin) = Qkk / qCount(Subbasin)
          END DO
          qThiessenWeights(PNoSta,Subbasin) = 1. - Qk / qCount(Subbasin)
        END DO
      END IF

      DEALLOCATE(Qtw)

      RETURN

      END SUBROUTINE ThiessenWeights

!---------------------------------------------------------------------------
      SUBROUTINE ThiessenPolygon
!
!     This routine generates a map of the Thiessen polygon for the
!     given station.  Only the polygon edges are identified.
!
!     Inputs are:  station index, StaNo
!                  total number of stations, NoSta
!                  map coordinates of each station (expressed in map units),
!                       Qx(1:NoSta) and Qy(1:NoSta)
!
!     Outputs are: polygon map expressed as left and right abcissa for each
!                       ordinate, Lrow(0:MapHeight) and Rrow(0:MapHeight)
!                  map upper most and lower most ordinate values, Yu and Yl
!
!
!     Polygon edges are found by first locating an initial point on the edge
!     of the polygon, or in the polygon but on the edge of the map, and then
!     searching along the edge (of the polygon or map, whichever dominates)
!     to establish other edge points.  The search pattern is initially north,
!     east, south, and west (NESW) until the next point is found in both the
!     map and polygon.  The search pattern is rotated (ESWN, SWNE, WNES, etc.)
!     whenever the search returns to the previous point found (no next point
!     found) and the search continues until the edge is completely traversed.
!     A point is identified as being in the polygon if the polygon station is
!     the closest of all stations to it.
!
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2), DIMENSION(1:4,0:5) :: SearchPatternX
      INTEGER (KIND=INTEGER2), DIMENSION(1:4,0:5) :: SearchPatternY
      INTEGER (KIND=INTEGER2) :: NoPatternRotations, K, Search
      INTEGER (KIND=INTEGER2) :: iStart, jStart, iLast, jLast
      INTEGER (KIND=INTEGER2) :: iFirst, jFirst
      INTEGER :: I, J, Ii, Jj
      LOGICAL :: FirstCell             !  , ClosestStation
      REAL    :: V

!
!     Initialize search pattern and number of pattern rotations
!
      DATA SearchPatternX /  0,  1,  0, -1,                               &
     &                       1,  0, -1,  0,                               &
     &                       0, -1,  0,  1,                               &
     &                      -1,  0,  1,  0,                               &
     &                       0,  1,  0, -1,                               &
     &                       1,  0, -1,  0/
      DATA SearchPatternY /  1,  0, -1,  0,                               &
     &                       0, -1,  0,  1,                               &
     &                      -1,  0,  1,  0,                               &
     &                       0,  1,  0, -1,                               &
     &                       1,  0, -1,  0,                               &
     &                       0, -1,  0,  1/


      NoPatternRotations = 0
      StaLocN = PNoSta                      ! PNoSta is a global variable
!
!     Initialize upper most and lower most ordinates to null map.
!
      Ylower = MapHeight
      Yupper = 0
!
!     Initialize left and right row limits to null map.
!
      DO J = 0, MapHeight
       Lrow(J) = MapWidth
       Rrow(J) = 0
      END DO
!
!     Store station coordinates and numbers in order of closest to StaNo.
!
      DO I = 1, PNoSta
       SrtX(I) = (Qx(I) - Qx(PStaNo)) ** 2 + (Qy(I) - Qy(PStaNo)) ** 2
       SrtN(I) = I
      END DO
      DO I = 1, PNoSta - 1
       K = SrtN(I)
       V = SrtX(I)
       DO J = I + 1, PNoSta
        IF (SrtX(J) .LT. V) THEN
         SrtX(I) = SrtX(J)
         SrtX(J) = V
         V = SrtX(I)
         SrtN(I) = SrtN(J)
         SrtN(J) = K
         K = SrtN(I)
        END IF
       END DO
      END DO
      DO I = 1, PNoSta
       SrtX(I) = Qx(SrtN(I))
       SrtY(I) = Qy(SrtN(I))
      END DO
!
!     Find map or off-map location of station and find an initial
!     point on the map, on the polygon edge with nothing to the north.
!
!     NOTE: Since station coordinates are already shifted to economize on
!     distance calculations, we must "unshift" them to determine which cell
!     that the station is in.  Alternatively, we could use the shifted
!     coordinates and routine ClosestStation to see which cell is closest to
!     the station (which would be the cell that the station is in); but that is
!     too much work.
!
      I = Qx(PStaNo) + 0.5
      J = Qy(PStaNo) + 0.5

      IF (I .GT. MapWidth) THEN
       IF (J .GT. MapHeight) THEN
!
!       Station is northeast of map...
!
!       Search from northwest corner east and continue if nothing found.
!
        I = 0
        J = MapHeight
        DO WHILE (I .LE. MapWidth .AND. .NOT. ClosestStation(I, J))
         I = I + 1
        END DO
        IF (I .GT. MapWidth) THEN
!
!        Search from northeast corner south and exit if nothing found.
!
         I = MapWidth
         DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
          J = J - 1
         END DO
         IF (J .LT. 0) RETURN
        END IF
       ELSE
        IF (J .GE. 0) THEN
!
!        Station is east of map...
!
!        Search from northeast corner south and exit if nothing found.
!
         I = MapWidth
         J = MapHeight
         DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
          J = J - 1
         END DO
         IF (J .LT. 0) RETURN
        ELSE
!
!        Station is southeast of map...
!
!        Search from northeast corner south and continue if nothing found.
!
         I = MapWidth
         J = MapHeight
         DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
          J = J - 1
         END DO
         IF (J .LT. 0) THEN
!
!         Search from southeast corner west and exit if nothing found.
!
          J = 0
          DO WHILE (I .GE. 0 .AND. .NOT. ClosestStation(I, J))
           I = I - 1
          END DO
          IF (I .LT. 0) RETURN
         END IF
        END IF
       END IF
      ELSE
       IF (I .GE. 0) THEN
        IF (J .GT. MapHeight) THEN
!
!        Station is north of map...
!
!        Search from northwest corner east and exit if nothing found.
!
         I = 0
         J = MapHeight
         DO WHILE (I .LE. MapWidth .AND. .NOT. ClosestStation(I, J))
          I = I + 1
         END DO
         IF (I .GT. MapWidth) RETURN
        ELSE
         IF (J .GE. 0) THEN
!
!         Station is in map...
!
!         Search north until out of polygon.
!
          DO WHILE (J .LE. MapHeight .AND. ClosestStation(I, J))
           J = J + 1
          END DO
!
!         Go back into polygon.
!
          J = J - 1
         ELSE
!
!         Station is south of map...
!
!         Search from southwest corner east and exit if nothing found or
!         continue if something found.
!
          I = 0
          J = 0
          DO WHILE (I .LE. MapWidth .AND. .NOT. ClosestStation(I, J))
           I = I + 1
          END DO
          IF (I .GT. MapWidth) RETURN
!
!         Search north until out of polygon.
!
          DO WHILE (J .LE. MapHeight .AND. ClosestStation(I, J))
           J = J + 1
          END DO
!
!         Go back into polygon.
!
          J = J - 1
         END IF
        END IF
       ELSE
        IF (J .GT. MapHeight) THEN
!
!        Station is northwest of map...
!
!        Search from northwest corner east and continue if nothing found.
!
         I = 0
         J = MapHeight
         DO WHILE (I .LE. MapWidth .AND. .NOT. ClosestStation(I, J))
          I = I + 1
         END DO
         IF (I .GT. Mapwidth) THEN
!
!         Search from northwest corner south and exit if nothing found.
!
          I = 0
          DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
           J = J - 1
          END DO
          IF (J .LT. 0) RETURN
         END IF
        ELSE
         IF (J .GE. 0) THEN
!
!         Station is west of map...
!
!         Search from northwest corner south and exit if nothing found.
!
          I = 0
          J = MapHeight
          DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
           J = J - 1
          END DO
          IF (J .LT. 0) RETURN
         ELSE
!
!         Station is southwest of map...
!
!         Search from northwest corner south and continue if nothing found.
!
          I = 0
          J = MapHeight
          DO WHILE (J .GE. 0 .AND. .NOT. ClosestStation(I, J))
           J = J - 1
          END DO
          IF (J .LT. 0) THEN
!
!          Search from southwest corner east and exit if nothing found.
!
           J = 0
           DO WHILE (I .LE. MapWidth .AND. .NOT. ClosestStation(I, J))
            I = I + 1
           END DO
           IF (I .GT. MapWidth) RETURN
          END IF
         END IF
        END IF
       END IF
      END IF
!
!     Edge of polygon (or map) reached, store coordinates
!     and initialize lower-most and upper-most ordinates.
!
      iStart = I
      jStart = J
      Ylower = J
      Yupper = J
      Lrow(J) = I
      Rrow(J) = I
!
!     Initialize "last-point-found" and "first-point-found"
!     coordinates to impossible values and set FirstCell.
!
      iLast = MapWidth * 2
      jLast = MapHeight * 2
      iFirst = MapWidth * 2
      jFirst = MapHeight * 2
      FirstCell = .TRUE.
!
!     Search for edge in clockwise pattern.
!
      DO WHILE (NoPatternRotations .LT. 6)
       Ii = I
       Jj = J
       Search = 1
       DO WHILE (Search .LT. 4)
        Ii = Ii + SearchPatternX(Search, NoPatternRotations)
        Jj = Jj + SearchPatternY(Search, NoPatternRotations)
        IF (Ii .LE. MapWidth  .AND. Ii .GE. 0 .AND.                        &
     &      Jj .LE. MapHeight .AND. Jj .GE. 0) THEN
          IF (ClosestStation(II, JJ)) THEN
!
!          Found next edge point; test to see if first and store if not.
!
           I = Ii
           J = Jj
           IF (iLast .EQ. iStart .AND. jLast .EQ. jStart .AND.             &
     &      I .EQ. iFirst .AND. J .EQ. jFirst) RETURN
           IF (FirstCell) THEN
            iFirst = I
            jFirst = J
            FirstCell = .FALSE.
           END IF
           IF (J .LT. Ylower) Ylower = J
           IF (J .GT. Yupper) Yupper = J
           IF (I .LT. Lrow(J)) Lrow(J) = I
           IF (I .GT. Rrow(J)) Rrow(J) = I
           iLast = I
           jLast = J
           Search = 0
          END IF
        END IF
        Search = Search + 1
       END DO
!
!      Rotate search pattern; if search pattern changed more than 5 times, have
!      degenerate case of one point is only thing on polygon edge and on map.
!
       NoPatternRotations = NoPatternRotations + 1
      END DO

      RETURN

      END SUBROUTINE ThiessenPolygon

!-------------------------------------------------------------------------
!
      SUBROUTINE ComputeMapCoordinates (Lat, Long, RM, XO, YO, Rad, X, Y)
!
!       This routine accepts the latitude and longitude of a point and
!       returns the x and y map coordinates of the point representing
!       the 'horizontal' and 'vertical' distances, respectively, from
!       an 'origin' on the map of (0.5, 0.5).
!
!       The map is a polyconic projection about a central meridian
!       (e.g. 88.0000 degrees west of the prime meridian for Superior).
!
!       On entry,
!       Lat is latitude in degrees and decimal degrees, positive from the
!                equator north.
!       Long is longitude in degrees and decimal degrees, positive from the
!                prime meridian east (negative west of the p. m.).
!
!       Calibrated map parameters:
!          RM  = the reference meridian (e.g. 88.00 for Superior)
!          Rad = calibrated radius of the earth in km
!          XO  = x offset
!          YO  = y offset
!
!       On exit,
!          x is distance in kilometers from point (0.5, 0.5) right.
!          y is distance in kilometers from point (0.5, 0.5) up.
!
!
      IMPLICIT NONE
      REAL, INTENT(IN)  :: Lat, Long, RM, XO, YO, Rad
      REAL, INTENT(OUT) :: X, Y
      REAL              :: Lt, Ln, Tp, Snpl

!
!  Get latitude in radians and longitude in radians from reference meridian.
!
      Lt = Lat / 57.29578
      Ln = (Long - RM) / 57.29578

      Tp = TAN(Lt)
      Snpl = SIN(Lt) * Ln
      X = Rad * SIN(Snpl) / Tp
      Y = Rad * (Lt + (1. - COS(Snpl)) / Tp)
      X = X + XO
      Y = Y + YO
      RETURN

      END SUBROUTINE ComputeMapCoordinates


      END MODULE DISAVSUBS


! *******************************************************************
! *******************************************************************
! *******************************************************************
! *******************************************************************

MODULE DisavMain
CONTAINS

!---------------------------------------------------------------------
!  Converted main program to a subroutine so that it can be compiled
!  as a DLL by Tim Hunter on 10aug98.  Also at this time I eliminated
!  the requirement that this file be a template and the program
!  EPDSAVMT be used to generate a fortran source file for each lake.
!---------------------------------------------------------------------
      SUBROUTINE DO_DISAVMET(ErrStatus, PassHnd)
      USE DISAVSUBS
      
      IMPLICIT    NONE
      INTEGER, INTENT(INOUT)  :: ErrStatus              !  0=OK,  otherwise an error
      INTEGER, INTENT(IN)     :: PassHnd                !  assigned to WinHandle

      INTEGER            :: I, IOS, UNum
      CHARACTER (LEN=3)  :: Bsn
      CHARACTER (LEN=15) :: FName


      WinHandle = PassHnd

      FName = 'BSNNM'
      UNum = GetFreeUnitNumber()
      OPEN(UNIT=UNum, FILE=TRIM(FName), STATUS='OLD', RECL=12,                &
     &     ACCESS='DIRECT', FORM='FORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'Error opening file '//TRIM(FName)//'!'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorLevel = 1
         ErrStatus = ErrorLevel
         RETURN
      END IF
      CALL FileWasOpened(UNum)

      READ(UNum, 1101, REC=1, IOSTAT=IOS) Bsn
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'Error reading from file '//TRIM(FName)//'!'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorLevel = 1
         ErrStatus = ErrorLevel
         RETURN
      END IF

      CLOSE(UNum)
      CALL FileWasClosed(UNum)

      I = SetParams(Bsn)

      IF (I .NE. 0) THEN
         ErrorMessage = 'Error setting parameter values'   ! ErrorMessage is in GLERLUTIL
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         ErrorLevel = 1
         ErrStatus = ErrorLevel
         RETURN
      END IF

      CALL MainRoutine(Bsn)

      ErrStatus = ErrorLevel                       !  ErrorLevel is in GLERLUTIL
      IF (ErrStatus .GT. 0) THEN
        CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
        RETURN
      END IF

 1101 FORMAT(A3)

      END SUBROUTINE DO_DISAVMET

END MODULE DisavMain
