MODULE LUMPPRV_Main

      USE FcstUtil

      INTEGER, PARAMETER :: MAXDAYS = 33000     ! ~90 years

CONTAINS

      SUBROUTINE DO_LUMPPRV(ErrStatus, HandleIn, BasnLand)

      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: BasnLand     ! 'BASN' or 'LAND'

      INTEGER (KIND=INTEGER2)                              :: BsnID
      INTEGER (KIND=INTEGER2), DIMENSION(3)                :: Dat
      INTEGER (KIND=INTEGER2), DIMENSION(3, MAXDAYS)       :: OutVals
      INTEGER (KIND=INTEGER2), DIMENSION(3, MAXDAYS, 0:30) :: DataVals

      INTEGER :: I, J, Ndx, Sub, RecNum, Subbasins, Sub1, Sub2, IOS
      INTEGER :: Day, Mon, Year, StartSeq, EndSeq, PrvRecNum
      INTEGER :: Seq1, Seq2, SSeq, ESeq, SeqNum
      INTEGER :: U11, U21, U31, U41

      REAL                  :: TotV, TotA
      REAL, DIMENSION(0:30) :: Area

      CHARACTER (LEN=3)  :: Bsn
      CHARACTER (LEN=4)  :: OverTheLake
      CHARACTER (LEN=8)  :: RNum
      CHARACTER (LEN=10) :: LakeName
      CHARACTER (LEN=99) :: File11, File21, File31, File41, FNameTmp


      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen(); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Read the lake name and pertinent dates from BSNNM.
!
      File11 = 'BSNNM'
      U11 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U11, FILE=TRIM(File11), STATUS='OLD', RECL=12,            &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U11, 1101, REC=1, ERR=812) LakeName
      CALL Caps(LakeName); IF (ErrorLevel .NE. 0) GOTO 899
      Bsn = LakeName(1:3)
      READ(U11, 1102, REC=2, ERR=812) Subbasins, OverTheLake
      READ(U11, 1103, REC=3, ERR=812) Day, Mon, Year
      CALL DateSequence(Day, Mon, Year, StartSeq)
      IF (ErrorLevel .NE. 0) GOTO 899
      READ(U11, 1103, REC=4, ERR=812) Day, Mon, Year
      CALL DateSequence(Day, Mon, Year, EndSeq)
      IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(U11)
      CALL FileWasClosed(U11); IF (ErrorLevel .NE. 0) GOTO 899

      IF (EndSeq-StartSeq+1 .GT. MAXDAYS) THEN
         ErrorMessage = 'LUMP_PRV: Total number of days exceeds maximum allowed!'
         CALL PassMsg
         WRITE(ErrorMessage, 9001) EndSeq-StartSeq+1, MAXDAYS
         CALL PassMsg; GOTO 898
      ENDIF

!
!  Read the AREA.xxx file
!
      File21 = 'AREA.'//Bsn
      U21 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
      CALL FileWasOpened(U21); IF (ErrorLevel .NE. 0) GOTO 899
      DO 25, Sub = 1, Subbasins
        READ(U21, 2101, ERR=822) Area(Sub)
  25  CONTINUE
      READ(U21, 2101, ERR=822) Area(0)
      CLOSE(U21)
      CALL FileWasClosed(U21); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Are we computing overland or overlake values?
!
      IF (BasnLand .EQ. 'LAND') THEN
         Sub1 = 1
         Sub2 = Subbasins
      ELSE IF (BasnLand .EQ. 'BASN') THEN
         Sub1 = 0
         Sub2 = Subbasins
      ELSE
         ErrorMessage = 'LUMP_PRV: Bad value for parameter BasnLand!'
         CALL PassMsg; GOTO 898
      ENDIF
      
!
!  Read the data from the xxxnn.PRV files
!
      DO 37, Sub = Sub1, Sub2
         WRITE(File31, 8001) Bsn, Sub, 'PRV'
         U31 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=U31, FILE=TRIM(File31), STATUS='OLD', RECL=6,          &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
         CALL FileWasOpened(U31); IF (ErrorLevel .NE. 0) GOTO 899

         PrvRecNum = 1
         READ(U31, REC=PrvRecNum, ERR=832) BsnID, SSeq
         PrvRecNum = 2
         READ(U31, REC=PrvRecNum, ERR=832) ESeq
         Seq1 = MAX(SSeq, StartSeq)
         Seq2 = MIN(ESeq, EndSeq)

         IF (Seq2-Seq1+1 .GT. MAXDAYS) THEN
            ErrorMessage = 'LUMP_PRV: Number of days exceeds maximum allowed!'
            CALL PassMsg
            WRITE(ErrorMessage, 9001) Seq2-Seq1+1, MAXDAYS
            CALL PassMsg; GOTO 898
         ENDIF

         DO SeqNum = Seq1, Seq2
            PrvRecNum = (SeqNum - SSeq + 1) + 2
            Ndx = SeqNum - StartSeq + 1
            READ(U31, REC=PrvRecNum, IOSTAT=IOS) (Dat(I), I=1,3)
            IF (IOS .NE. 0) THEN
               WRITE(ErrorMessage, 9002) IOS
               CALL PassMsg
               WRITE(ErrorMessage, 9003) Seq1, Seq2, SeqNum, SSeq, ESeq
               CALL PassMsg
               
               WRITE(FNameTmp, 8101) Bsn, Sub, 'PRV'
               CLOSE(U31)
               CALL CopyFile(TRIM(File31), TRIM(FNameTmp))
               
               GOTO 832
            END IF
            DO I = 1, 3
               DataVals(I,Ndx,Sub) = Dat(I)
            END DO
         END DO

         CLOSE(U31)
         CALL FileWasClosed(U31); IF (ErrorLevel .NE. 0) GOTO 899
  37  CONTINUE

!
!     Compute areal averages
!
      DO I = 1, 3
         DO J = StartSeq, EndSeq
            Ndx = J - StartSeq + 1
            TotV = 0.0
            TotA = 0.0
            DO Sub = Sub1, Sub2
               IF (DataVals(I,Ndx,Sub) .GT. -9999) THEN
                  TotV = TotV + DataVals(I,Ndx,Sub)*Area(Sub)
                  TotA = TotA + Area(Sub)
               ENDIF
            END DO
            IF (TotA .GE. 1.0) THEN
               OutVals(I,Ndx) = TotV / TotA
            ELSE
               OutVals(I,Ndx) = -9999
            END IF
         END DO
      END DO

!
!     Write the output file
!
      IF (TRIM(BasnLand) .EQ. 'LAND') THEN
         File41 = Bsn // 'LD.PRV'
      ELSE
         File41 = Bsn // 'BS.PRV'
      ENDIF

      U41 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U41, FILE=TRIM(File41), STATUS='REPLACE', RECL=6,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
      CALL FileWasOpened(U41); IF (ErrorLevel .NE. 0) GOTO 899

      Seq1 = StartSeq
      Seq2 = EndSeq
      WRITE(U41, REC=1, ERR=843) BsnID, Seq1
      WRITE(U41, REC=2, ERR=843) Seq2

      DO 46, SeqNum = Seq1, Seq2
         RecNum = (SeqNum - Seq1 + 1) + 2
         Ndx = SeqNum - StartSeq + 1
         Dat(:) = OutVals(:,Ndx)
         WRITE(U41, REC=RecNum, ERR=843) Dat
  46  CONTINUE

      CLOSE(U41)
      CALL FileWasClosed(U41); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Copy the correct file to xxxALL.PRV
!     If BasnLand was 'BASN' then we didn't create a useful file (in terms
!     of AHPS later usage) so we won't do anything.
!
      IF (TRIM(BasnLand) .EQ. 'LAND') THEN
         IF (TRIM(OverTheLake) .EQ. 'BASN') THEN
            File11 = File41
         ELSE
            File11 = Bsn//'00.PRV'
         END IF
         File21 = Bsn//'ALL.PRV'
         CALL CopyFile(File11, File21); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'LUMP_PRV: Error opening file '//TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LUMP_PRV: Error reading file '//TRIM(File11)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'LUMP_PRV: Error writing file '//TRIM(File11)
      CALL PassMsg; GOTO 898

  821 ErrorMessage = 'LUMP_PRV: Error opening file '//TRIM(File21)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'LUMP_PRV: Error reading file '//TRIM(File21)
      CALL PassMsg; GOTO 898

  831 ErrorMessage = 'LUMP_PRV: Error opening file '//TRIM(File31)
      CALL PassMsg; GOTO 898
  832 WRITE(RNum, '(I0)') PrvRecNum
      ErrorMessage = 'LUMP_PRV: Error reading record '//TRIM(RNum)//' of file '//TRIM(File31)
      CALL PassMsg; GOTO 898
  833 WRITE(RNum, '(I0)') PrvRecNum
      ErrorMessage = 'LUMP_PRV: Error writing record '//TRIM(RNum)//' of file '//TRIM(File31)
      CALL PassMsg; GOTO 898

  841 ErrorMessage = 'LUMP_PRV: Error opening file '//TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'LUMP_PRV: Error reading file '//TRIM(File41)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'LUMP_PRV: Error writing file '//TRIM(File41)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_LUMPPRV...'
      CALL PassMsg
      GOTO 999

  999 CALL CloseAllFiles()     ! guarantee that all files are closed
      ErrStatus = ErrorLevel
      RETURN
!
!  FORMAT statements
!
 1101 FORMAT(A10)
 1102 FORMAT(I3, 3X, A4)
 1103 FORMAT(2I3, I5)
 2101 FORMAT(E13.6E2)

 8001 FORMAT(A3, I2.2, '.', A3)
 8101 FORMAT(A3, I2.2, '_', A3, '.TMP')
 9001 FORMAT(3X, 'Number of days specified=', I0, 2X, 'MaxDays=', I0)
 9002 FORMAT('LUMP_PRV: Error has been encountered. IOSTAT=[', I0, ']  Debug Information:')
 9003 FORMAT('   Seq1=', I0, '  Seq2=',I0, '  SeqNum=',I0, '  SSeq=',I0, '  ESeq=',I0)

      END SUBROUTINE DO_LUMPPRV

END MODULE LUMPPRV_Main

